! ===========================================================================================================
! module communication.f90
!
! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who    what
! 20191126 VH     Set up variables used to select JULD, times, qc, for non-MCLines files (TIME_ASCENT_END &
!                 TIME_TRANS_START) measurement codes (MC600,MC700,MC589,MC590 & positions for ASC; 
!                 MC100, MC090, MC099 for DSC), arrays for GMT testing, set badgps_test_hi as GMT_year+2.
! 20190904 VH     Add variables to capture the GMT date, including BADGPS to debug bad gps fixes.
! 20190708 VH     Add a new subroutine, 'flush_forlog'. Useful if segmentation faults are occurring, make
!                 a call to this to close & reopen the forlog, to identify where the faults are coming from`
! 20190521 VH     AAdd global variables for DOGRID testing (used in qc04, on-land and qc07, regional test.
! 20190416 VH&JN  Initialize aomlid, add logicals for special ebugging, modify what some DBLEV values mean; 
!                 eliminate posn_from, use posn_cmnt in to data_arrays
! 20180920 JN     Add recordlength to open statement for the GTS file
! 20180724 VH     INTERPOLATION/ICE DETECTION: DBLEV/debug parameters for int/ice testing
!                 Finalize cyc<6 warning/lognote
! 20180619 VH     New variable, float_type (first part of inst_type) to be used later
!                 Add variables for PROFILE_UPLOAD_TIME. QC
!                 Change START_OF_TRANSMISSION + EPOCH to 14ch variables
!                 Revise warning & lognote msg to include aomlid & cyc
!                 Do not define is_numeric and is_all_numeric in the functions is_numeric,is_all_numeric vargo)
!                 Add aomlid & cycle number to lognotes & warhnings
! 20180417 VH     Added sub direct_exit, eliminate critical_warning. Warning of "defaulted env var
!                 not found" only  if DEBUG is on. General WARNINGrevisions.
! 20180307 VH     Extensive changes to create BR file, new sub: lognote - usr for former warnings
! 20170712 VH     Add timestamp to first & last log entrh; new subrotine critical_warning (for cases where
!                 we want toe output to go to GDAC/GTS)
! 20170708 JN     New function is_all_numeric, to make sure all of a string has only n umeric chars.
! 20170718 CA&VH  Restore write statement to add reportline to report file
! 20170712 all    Release of qcplus v1.1 - many changes to all, some documented/tracked, some not
! 20160511 all    Initial release of qcplus v1.1.0
!
! ===========================================================================================================
! Notes:

! This module contains variable/array definitions used by all program elements to provide communication, both
! between elements, and with the outside (GETARG,GETENV,etc.).
! ===========================================================================================================

module communication

implicit none

! ===========================================================================================================
! moedule varialbe/array declarations
! ===========================================================================================================

integer, parameter :: STD = 6      ! lun for standard output
integer, parameter :: LOG = 8      ! lun for the forlog log file (NOT standard output)
integer, parameter :: ERR = 50     ! lun for the error file
integer, parameter :: TRC = 48     ! lun for the trace file
integer, parameter :: DBG = 6      ! lun for debug statements (standard output - will also go to forlog)
integer, parameter :: INP = 24     ! general input unit
integer, parameter :: OUT = 52     ! general output unit
integer, parameter :: TAB = 10     ! config/traj/tech tables
integer, parameter :: MSC = 31     ! NCMETA MISS_CNFG file for missing config parameters
integer, parameter :: MSS = 32     ! NCMETA MISS_SNSR file for missing sensor_model definitions
integer, parameter :: MST = 33     ! NCTECH MISS_TECH file for missing technicl parameemrs
integer, parameter :: REP = 49     ! lun for results table

integer :: resolution(4) = (/ 0, 0, 0, 0 /);; public resolution; ! TEMMPORARY HOME MOVE TO data_arrays LATER


character(len=128) :: pathname_phydir, pathname_metadir
character(len=128) :: pathname_aomlid_qcdir, pathname_failure_qcdir
character(len=128) :: pathname_failure_ncdir

character(len=128) :: pathname_forlog;
character(len=128) :: pathname_error;
character(len=128) :: pathname_trace;
character(len=128) :: pathname_report; logical :: rep_open; ! Is the report file open?
character(len=256) :: repmsg
character(len=128) :: pathname_config_table,	pathname_config_table_skip	! Used to make the ncmeta file
character(len=128) :: pathname_sensor_model_table,pathname_sensor_params_table      ! Used to id sensor models (no SKIP), used for parameters in meta.nc file
character(len=128) :: pathname_traj_table,	pathname_traj_table_skip	! Used to make the nc traj file
character(len=128) :: pathname_traj_mc_table
character(len=128) :: pathname_tech_table,	pathname_tech_table_skip	! Used to make the nc tech file
character(len=128) :: pathname_config_miss; logical :: msc_open; ! NCMETA MISS_CNFG track missing configuretion parameters
character(len=128) :: pathname_sensor_miss; logical :: mss_open; ! NCMETA MISS_SNSR track missing sensor_model definitions
character(len=128) :: pathname_tech_miss;   logical :: mst_open; ! NCTECH MISS_TECH track missing technical parameters
character(len=128) :: pathname_ftpgts;
character(len=256) :: ftpmsg;
character(len=128) :: pathname_2tesac;
character(len=128) :: pathname_tesac;
character(len=128) :: topo_file
logical            :: topo_file_checked;
logical            :: topo_file_exists;
character(len=256) :: tesmsg; public tesmsg;

logical :: err_open,log_open,trc_open,inp_open,out_open
logical :: make_qcprof = .false.;
logical :: make_ncprof = .false.;
logical :: make_ncmeta = .false.;
logical :: make_nctraj = .false.;
logical :: make_nctech = .false.;
logical :: make_ncbiop = .false.;
logical :: make_ncbiot = .false.;
logical :: make_tesac  = .false.;
logical :: tesac_created = .false.;


                                                !                                         123456789012345678901
integer, parameter :: MAXPROFCHR = 24		! max number of chars in the profile name aoml_transid_ccccD.phy
character(len=128) :: profinp			! ARG in, long form of profile
character(len=MAXPROFCHR) :: profile		! profinp is 128, profile is MAXPROFCHR
character(len=MAXPROFCHR) :: qcfile,ncfile	! For those times when a previous qc file is being read
character(len=6)          :: qcextn,ncextn
character(len=128) :: float_name
character(len=128) :: wmoid
character(len=4)   :: aomlid = '0000';          ! Initialize to noe aomlid yet
character(len=5)   :: ccyc;
integer :: thiscyc, prevcyc
integer :: begcyc, adjcyc	! Used to find the CONFIG_MISSION_NUMBER
integer :: modval, nonval	! What is the mission number for mod*thiscycle)==zero, and /=zero
character(len=1)   :: ans1		! used as answer from get_default_envvar
character(len=4)   :: ans4		! used as answer from get_default_envvar
character(len=1)   :: string1
character(len=2)   :: string2
character(len=4)   :: string4

character(len=1)   :: chr1		! generic character string
character(len=2)   :: chr2            ! generic character string
character(len=3)   :: chr3            ! generic character string
character(len=4)   :: chr4            ! generic character string
character(len=5)   :: chr5            ! generic character string
character(len=6)   :: chr6            ! generic character string
character(len=7)   :: chr7            ! generic character string
character(len=8)   :: chr8            ! generic character string

! ! which files requested, which non, which get written, which not
! filmsg = 'FFFFFFFF'; reports disaster beforeany writtes are attempted
character(len=8) :: filmsg = 'FFFFFFFF'; 

! These are the fill values
integer, parameter :: FILLINT = 99999         ! fill value for integers
real, parameter :: FILL = -999.0      ! data fill value
real, parameter :: FILT = -9.0	! some apex tso use -9 as fill value for TEMP
real, parameter :: FTST = FILL+10     ! test fill values, may be 'adjusted' (by error)
real, parameter :: FILLNC = 99999.0   ! data fill value for the nc data arrays
character(len=1), parameter  :: FILLCHAR = ' ';                       ! char fill value for nc flags
character(len=14), parameter :: FILL14CH = '99999999999999';          ! char fill value for YYYYMMDDHHMMSS
character(len=19), parameter :: FILL19CH = '9999 99 99 99 99 99';     ! char fill value for YYYY MM DD HH MM SS
double precision, parameter  :: FILLDBLE = 999999.0                   ! dble fill value for JULD,JULD_LOCATION
double precision, parameter  :: FILLPOSN = 99999.0                    ! dbl fill value for LOCATIONS (ie, LAT/LON)
double precision, parameter  :: FILLCNFPAR = 99999.0                  ! dbl fill value for CONFIG_PARAM_VALUE,LAUNCH_CONFIG_PARAM_VALUE
real,parameter :: DENGRAD_FV = .9999  ! density gradient fill value

character(len=14), parameter :: FLDT = '99999999999999';				! Shorthand fill value for date/time
character(len=4), parameter :: FLYR = '9999';						! Shorthand fill value for year
character(len=1), parameter :: Q1 = '1', Q4 = '4', Q9 = '9', QG = 'G';	! Shorthand defs for QC=1,4,9,G (G=badgps_fix

integer gloc		! Global version of the 'loc' for calls to CKE (Check nc exit status)
!  set this in ncprof_arrays.f90 module integer           :: var_CYCLE_NUMBER = FILLINT;
character(len=16) :: inst_type; public inst_type		! OUR instruemnt type
character(len=16) :: float_type; ! The first part of the inst_type (APEX, APEXIR,SOLO, etc)
character(len=5)  :: inst_type_short; public inst_type_short	! short instruemnt type (for report line)
character(len=7)  :: trans_type;					! transmisison type from meta file
character(len=1)  :: chr_DIRECTION = FILLCHAR;			! STANDARD var for nc
character(len=2)  :: chr_DATA_CENTRE = 'AO';		! that's us!	! STANDARD var for nc
character(len=32) :: chr_DC_REFERENCE = FILLCHAR;	! set in qc: unique identifier in data centrr
character(len=8)  :: chr_PLATFORM_NUMBER = FILLCHAR;
character(len=64) :: var_PI_NAME = FILLCHAR;				! from meta to nc files
character(len=64) :: chr_PROJECT_NAME = 'US ARGO PROJECT';		! STANDARD var for nc
character(len=4)  :: chr_DATA_STATE_INDICATOR = '2B';			! STANDARD var for nc - No visual check yet, so not 2B+
character(len=1)  :: chr_DATA_MODE = 'R';		! set in qc: A if P or S adjusted, R otherwise
character(len=64) :: chr_PLATFORM_TYPE = FILLCHAR;			! from meta to nc files`
character(len=16) :: var_FLOAT_SERIAL_NO = FILLCHAR;
character(len=64) :: var_TRANSMISSION_NO = FILLCHAR;
character(len=4)  :: var_INTERNAL_ID = FILLCHAR;
character(len=32) :: var_FIRMWARE_VERSION = FILLCHAR;
character(len=4)  :: var_WMO_INST_TYPE = FILLCHAR;
character(len=8)  :: chr_POSITIONING_SYSTEM = FILLCHAR;;
character(len=67) :: POSITIONING_SYSTEM_from = FILLCHAR;
character(len=4)  :: chr_WMO_RECORDER_TYPE = '99';

integer            :: int_CONFIG_MISSION_NUMBER;	public int_CONFIG_MISSION_NUMBER;	! set in meta or phy or in qcplus itself
character(len=2)   :: config_mission_type;		public config_mission_type;	! S=single, D=double, C=cycle P=from phy file
character(len=256) :: config_mission_comment;		! comment on config_misison_number (launch or what?)
logical :: config_lines_present = .false.
logical :: config_float_mission_present = .false.	! Can we get the CONFIG_MISSION_NUMBER from the CONFIG FLOAT MISSION?
logical :: double_nominal_prof_pres = .false.		! Are there two nominal profile pres values?
logical :: double_nominal_drft_pres = .false.		! Are therd two nominal drift  pres values?

integer :: STUCK_T, STUCK_S, STUCK_O	! STUCK VALUE INDICATOR for T,S,O Set to 1 AND ADD log entry for constant value
integer :: JUMPC_T, JUMPC_S		! JUMP CHECK INDICATOR for T,S    Set to 1 and add log entry to check sensor drift
integer :: FROZN_T, FROZN_S		! FROZEN_PROFLE_INDICATOR for T,S Set to 1 and add log message if profile frozen

! Note that TIME_TRANS_START and TIME_TRANS_EPOCH are converted from 19ch to 14ch in read_phy 
character(len=19) :: TIME_CH19;
integer :: MC_yr, MC_mn, MC_dd, MC_hh, MC_mm, MC_ss

! MC codes to look for: MC100 MC600 MC700 MC090 MC099 MC589 MC590 - Note that MC703 is only available in the pos block
logical, parameter :: NO = .false.; ! Note that FLDT and Q9 are parameters defined earlier, but little, used except here
logical :: TIME_ASCENT_END_found  = NO; character(len=14) :: TIME_ASCENT_END  = FLDT; character(len=1)  :: TIME_ASCENT_END_QC  = Q9;
logical :: TIME_TRANS_START_found = NO; character(len=14) :: TIME_TRANS_START = FLDT; character(len=1)  :: TIME_TRANS_START_QC = Q9;
logical :: TIME_TRANS_EPOCH_found = NO; character(len=14) :: TIME_TRANS_EPOCH = FLDT; character(len=1)  :: TIME_TRANS_EPOCH_QC = Q9;
logical :: TIME_MC100_found       = NO; character(len=14) :: TIME_MC100       = FLDT; character(len=1)  :: TIME_MC100_QC       = Q9;
logical :: TIME_MC600_found       = NO; character(len=14) :: TIME_MC600       = FLDT; character(len=1)  :: TIME_MC600_QC       = Q9;
logical :: TIME_MC700_found       = NO; character(len=14) :: TIME_MC700       = FLDT; character(len=1)  :: TIME_MC700_QC       = Q9;
logical :: TIME_MC090_found       = NO; character(len=14) :: TIME_MC090       = FLDT; character(len=1)  :: TIME_MC090_QC       = Q9;
logical :: TIME_MC099_found       = NO; character(len=14) :: TIME_MC099       = FLDT; character(len=1)  :: TIME_MC099_QC       = Q9;
logical :: TIME_MC589_found       = NO; character(len=14) :: TIME_MC589       = FLDT; character(len=1)  :: TIME_MC589_QC       = Q9;
logical :: TIME_MC590_found       = NO; character(len=14) :: TIME_MC590       = FLDT; character(len=1)  :: TIME_MC590_QC       = Q9;
logical :: TIME_MC703_1_found     = NO; character(len=14) :: TIME_MC703_1     = FLDT; character(len=1)  :: TIME_MC703_1_QC     = Q9;
logical :: TIME_MC703_2_found     = NO; character(len=14) :: TIME_MC703_2     = FLDT; character(len=1)  :: TIME_MC703_2_QC     = Q9;
logical :: TIME_MC703__needed     = NO;
integer :: TIME_MC703_1_ipos = 0, TIME_MC703_2_ipw = 0;

logical :: TIME_ASCENT_END_good   = NO;
logical :: TIME_TRANS_START_good  = NO;
logical :: TIME_TRANS_EPOCH_good  = NO;
logical :: TIME_MC100_good        = NO;
logical :: TIME_MC600_good        = NO;
logical :: TIME_MC700_good        = NO;
logical :: TIME_MC090_good        = NO;
logical :: TIME_MC099_good        = NO;
logical :: TIME_MC589_good        = NO;
logical :: TIME_MC590_good        = NO;
logical :: TIME_MC703_1_good      = NO;
logical :: TIME_MC703_2_good      = NO;

logical :: PROFILE_UPLOAD_TIME_found       = .false.;
logical :: PROFILE_UPLOAD_TIME_good        = .false.;
character(len=14) :: PROFILE_UPLOAD_TIME   = '99999999999999';;
character(len=1) :: PROFILE_UPLOAD_TIME_QC = '9';;

logical             :: UP_TIME_found
real              :: UP_TIME_HOURS, UP_TIME_DAYS

logical           :: TIME_DSCENT_START_found,  TIME_TRANS_END_found
character(len=19) :: TIME_DSCENT_START,        TIME_TRANS_END
character(len=1)  :: TIME_DSCENT_START_QC,     TIME_TRANS_END_QC

integer :: GMT_year,  GMT_mon, GMT_day;    ! Used to trap BADGPS fixes
integer :: badgps_test_hi, badgps_test_lo; ! Used to trap BADGPS fixes
integer :: JULD_year, JULD_mon,JULD_day; public JULD_year, JULD_mon,JULD_day; ! profile (GTS, ...) date & grey list check
integer :: JULD_hr,   JULD_mn, JULD_sc;  public JULD_hr,   JULD_mn, JULD_sc;  ! profile (GTS, ...) date
integer :: JLOC_year, JLOC_mon,JLOC_day; public JLOC_year, JLOC_mon,JLOC_day; ! compare in grey list check
double precision  :: prof_JULD = FILLDBLE;
character(len=20) :: char_JULD;
character(len=1) :: prof_JULD_QC = '9';
character(len=80) :: juld_from = 'UNDEFINED';
character(len=14) :: prof_DATE_TIME_PROFILE = '99999999999999';
double precision  :: prof_JULD_LOCATION = FILLDBLE;
double precision  ::  prof_LATITUDE  = FILLPOSN; real :: real_LATITUDE  = FILL;
double precision  ::  prof_LONGITUDE = FILLPOSN; real :: real_LONGITUDE = FILL;
character(len=14) ::  prof_DATE_TIME_LOCATION = '99999999999999';
character(len=1)  ::  prof_POSITION_QC = '0';     integer :: int_POSITION_QC = 0;
character(len=1)  :: prof_JULD_LOCATION_qc = '0'; integer :: int_JULD_LOCATION_QC = 0;
! These are the possible outcomes of qc tests:
integer, parameter :: NOTF = 0;    ! FLAG VALUE used for flags not evaluated (lev 1 * nlev in spike, etc)
integer, parameter :: NOTD =-1;    character(len=1), parameter :: CNOTD = "N";     ! flag cannot be determined
integer, parameter :: PASS = 0;    character(len=1), parameter :: CPASS = "P";     ! flag for pass
integer, parameter :: FAIL = 1;    character(len=1), parameter :: CFAIL = "F";     ! flag for fail
character(len=22),parameter :: report(-1:1) = (/ &
"can not be determined.",&
"passed.               ",&
"FAILED.               "/)
character(len=22),parameter :: flagreport(0:4) = (/ &
"can not be determined.",&
"passed.               ",&
"not used              ",&
"not used              ",&
"FAILED.               "/)

! These are the possible return values for non-qc test functions
integer, parameter :: SUCCESS  = 0; ! Success
integer, parameter :: FAILURE  = 1; ! Failure

! Note that you can turn overall DEBG off, but still access DBLEV
! This makes for less chatty but more focused debug messages

logical :: DEBUG = .false.;	! debug on or off, default to off
integer :: DBLEV = 0;		! dblev zero unless changed by the DBLEV env var - SET INDEPENDENTLY OF DEBUG
integer :: MAXDBLEV = 299 	! This is the hightest DBLEV we have so far (used in qcplus itself);
logical :: DBLEVEXIT = .false.	! exit early due to value of DBLEV - used by call dblev_exit("exit message")
logical :: DBBIO = .false.;	! Log the input and output

! integer, parameter :: DBLEV== 0    ! minimal debugs - do NOT put new lines in the qc file
!                                     ! DBLEV>0 put new lines in the qc file
! integer, parameter :: DBLEV== 1    ! OPNCLS: report when external files are opened or closed NOT CURRENTLY USED - general purpose
! integer, parameter :: DBLEV== 2    ! qc test - qc02_impossible_date, juldqc
! integer, parameter :: DBLEV== 3    ! qc test - qc03_impossible_location
! integer, parameter :: DBLEV== 4    ! qc test - qc04_position_on_land
! integer, parameter :: DBLEV== 5    ! qc test - qc05_impossible_speed
! integer, parameter :: DBLEV== 6    ! qc test - qc06_gross_test
! integer, parameter :: DBLEV== 7    ! qc test - qc07_regional_test
! integer, parameter :: DBLEV== 8    ! qc test - qc08_pres_increase
! integer, parameter :: DBLEV== 9    ! qc test - qc09_spike_test
! integer, parameter :: DBLEV==10    ! orbcom processing
! integer, parameter :: DBLEV==11    ! qc test - qc11_gradient_test
! integer, parameter :: DBLEV==12    ! bounce processing
! integer, parameter :: DBLEV==13    ! qc test - qc13_stuck_value
! integer, parameter :: DBLEV==14    ! qc test - qc14_density_inversion
! integer, parameter :: DBLEV==15    ! qc test - qc15_grey_list
! integer, parameter :: DBLEV==16    ! qc test - qc16_jump_check
! integer, parameter :: DBLEV==17    ! NOT CURRENTLY USED
! integer, parameter :: DBLEV==18    ! qc test - qc18_frozen_profile
! integer, parameter :: DBLEV==19    ! qc test - qc19_deepest_pressure
! integer, parameter :: DBLEV==20    ! qc test - qc20_quest_argos_posn -TRAJ only
! integer, parameter :: DBLEV==21    ! qc test - qc21_ns_unpumped_ctd.f90 (STS - psal present)
! integer, parameter :: DBLEV==22    ! qc test - qc22_ns_mixed_air_water.f90(NST - near-surface temp, no psal) 122 force adj
! integer, parameter :: DBLEV==23    ! qc test - qca1_climatology.f90
! integer, parameter :: DBLEV==24    ! qc test - qca2_analysis.f90
! integer, parameter :: DBLEV==25    ! local to gmt conversion
! integer, parameter :: DBLEV==26    ! Near-surface Both NST and STS (for both qc21 * qc22)
! integer, parameter :: DBLEV==27    ! NOT CURRENTLY USED
! integer, parameter :: DBLEV==28    ! write_qcprof:write data HDR cols, first/last data line only in qcprof
! integer, parameter :: DBLEV==29    ! qc test flags
! integer, parameter :: DBLEV==30    ! ZEROLINES zerolines1, zerolkines2
! integer, parameter :: DBLEV==31    ! hist_qctest idios hex thingy FIND & FIX
! integer, parameter :: DBLEV==32    ! measurement code information
! integer, parameter :: DBLEV==33    ! JOB_STATUS determination and use
! integer, parameter :: DBLEV==34    ! check qc COL names
! integer, parameter :: DBLEV==35    ! read_pqc and almost all nammer of pqc related items
! integer, parameter :: DBLEV==36    ! read_pqc timing (starts/diffs,etc)
! integer, parameter :: DBLEV==37    ! NOT CURRENTLY USED (need to purge old)
! integer, parameter :: DBLEV==38    ! profile_<>_qc (also prof_<>_qc
! integer, parameter :: DBLEV==39    ! config_mission_number (CMN)
! integer, parameter :: DBLEV==40    ! MAXFLAG tracking - should affect only JOB_STATUS
! integer, parameter :: DBLEV==41    ! ICE TEST ice_lines_present = .false.;
! integer, parameter :: DBLEV==42    ! ICE TEST this_lat(1) = 50.0; (999 lines)
! integer, parameter :: DBLEV==43    ! ICE TEST npos = 2
! integer, parameter :: DBLEV==44    ! NOT CURRENTLY USED
! integer, parameter :: DBLEV==45    ! readung the flags from the OLD FILE FIND & FIX
! integer, parameter :: DBLEV==46    ! NOT CURRENTLY USED
! integer, parameter :: DBLEV==47    ! meta file info
! integer, parameter :: DBLEV==48    ! Track value for int_CONFIG_MISSION_NUMBER
! integer, parameter :: DBLEV==49    ! Track value for chr_POSITIONING_SYSTEM
! integer, parameter :: DBLEV==50    ! DPLS, determination of dpls, date/posn/land/speed flags
! integer, parameter :: DBLEV==51    ! NO pres adj, NO psal adj
! integer, parameter :: DBLEV==52    ! DO pres adj, NO psal adj
! integer, parameter :: DBLEV==53    ! NO pres adj, DO psal adj
! integer, parameter :: DBLEV==54    ! DO pres adj, DO psal adj
! integer, parameter :: DBLEV==60    ! debug all FILMSG changes and continue execution
! integer, parameter :: DBLEV==61    ! debug write_qcprof - currenlty mp fimctopm
! integer, parameter :: DBLEV==62    ! debug write_ncprof - exit after create_ncfile
! integer, parameter :: DBLEV==63    ! debug write_ncmeta - exit after create_ncfile
! integer, parameter :: DBLEV==64    ! debug write_nctech - exit after create_ncfile
! integer, parameter :: DBLEV==65    ! debug write_nctraj - exit after create_ncfile
! integer, parameter :: DBLEV==66    ! debug write_ncbiop - exit after create_ncfile
! integer, parameter :: DBLEV==67    ! debug write_ncbiot - exit after create_ncfile
! integer, parameter :: DBLEV==68    ! debug write_tesac  - currenlty no function
! integer, parameter :: DBLEV==69    ! debug exit afterAcreate_ncfile
! integer, parameter :: DBLEV==70-79 ! Used for various dbgpos choices in pick_profile_pos
! integer, parameter :: DBLEV==80-89 ! Reserved for use by Charita for the TRAJNC parts (a bit obsolete)

! integer, parameter :: DBLEV==87    ! Special LEV that allows profiles with lat==lon to be processed
! integer, parameter :: DBLEV==90    ! debug SURF_PRES, MRVSP, PRES_ADJ, PRES_OFFSET, no early exit
! integer, parameter :: DBLEV==91    ! debug SURF_PRES, exit after read_phy
! integer, parameter :: DBLEV==92    ! debug SURF_PRES, MRVSP, exit before adjust_pres
! integer, parameter :: DBLEV==93    ! debug SURF_PRES, MRVSP, PRES_ADJ, PRES_OFFSET, exit after adjust_pres
! integer, parameter :: DBLEV==94    ! debug SURF_PRES, MRVSP, PRES_ADJ, PRES_OFFSET, exit after write_qcprof
! integer, parameter :: DBLEV==95    ! debug FORMAT_VERSION 3.0/3.1
! integer, parameter :: DBLEV==96    ! NOT CURRENTLY USED
! integer, parameter :: DBLEV==97    ! tracl Enter/Leave subroutines
! integer, parameter :: DBLEV==98    ! NOT CURRENTLY USED
! integer, parameter :: DBLEV==99    ! HDR

! Special cases of "dual" meanings for input DBLEV

! Special cases directing how to debug the write_* functions - used with DBLEV between 60 and 69
! if DBLEV >= 60 and DBLEV < 70

! If ANY of the DBLEV>200, then exit only after ALL write_nc<> subs have been called
logical :: dbgnc_exit_after_all_ncwrites = .false.

logical :: dbgnc_rtrn_after_defdims = .false.;	! DBLEV 201to210 exit nc file creation after definitions defined
logical :: dbgnc_rtrn_after_commons = .false.;	! DBLEV 211to220 exit nc file creation after common variables defined
logical :: dbgnc_rtrn_after_defvars = .false.;	! DBLEV 221to230 exit nc file creation after individual variables defined
logical :: dbgnc_rtrn_after_defpars = .false.;	! DBLEV 231to240 exit nc file creation after params defined, leave define mode
logical :: dbgnc_rtrn_after_putcoms = .false.;	! DBLEV 241to250 exit nc file creation after common vars loaded (put)
logical :: dbgnc_rtrn_after_putvars = .false.;	! DBLEV 251to260 exit nc file creation after common vars loaded (put)
logical :: dbgnc_rtrn_after_putpars = .false.;	! DBLEV 261to270 exit nc file creation after common vars loaded (put)

logical :: OUTPUT_LEVS = .false.	! Do you want to output a set of levels to the log file
integer :: beglev = 0			! What levels shoud be printed (Zeroes mean all levels)
integer :: endlev = 0			! What levels shoud be printed (Zeroes mean all levels)
public OUTPUT_LEVS, beglev, endlev

integer, parameter :: SZC=448		! Size of the cline * msg variables
character(len=SZC)  :: msg, msgtmp;
character(len=SZC)  :: msg1, msg2	!
character(len=2*SZC):: bigmsg		! BIG message
character(len=SZC) :: msginp
character(len=SZC) :: tsbeg = 'N/A';               ! time-stamped messages
character(len=SZC) :: tsend = 'N/A';               ! time-stamped messages
character(len=SZC) :: tserr = 'N/A';               ! time-stamped messages
integer :: nwarn  = 0; ! counter for warnings
integer, parameter :: NWRNMAX = 1500; character(len=SZC) :: wrnmsg, wrnmsgs(NWRNMAX);
character(len=SZC) :: logmsg; ! messages for lognote
logical :: errors_warnings_exist = .false.; ! Errors and/or warnings exist
character(len=SZC) :: errmsg0, errmsg(9);  integer :: nerrmsg;
integer :: nerror  = 0; ! counter for NON-FATAL errors
integer, parameter :: NERRMAX = 1500; character(len=SZC) :: errmsgs(NERRMAX);
character(len=SZC) :: trcmsg(10); integer :: ntrcmsg;

! variables to read data
logical :: HED, POS, DAT              ! where we are in the file - header, position or data
character(len=SZC) :: cline		! generic input line
character(len=40)  :: variable	! first 40 chars - name of the variable
character(len=(SZC-40)) :: value	! the value of that variable
character(len=(SZC+16)) :: cmsg	! wrkte the cline into cmsg - RTErr if too large

integer :: EXIT_STATUS; public EXIT_STATUS;	! 0=-good profile, 1= bad profile, -1=program error
integer :: JOB_STATUS;  public JOB_STATUS	! use the various stats defs below
logical :: prof_on_land
integer,parameter ::  NSTATUS=19
integer,parameter ::  partially_qced_status         = 0;   logical :: partially_qced
integer,parameter ::  passed_complete_status        = 1;   logical :: passed_complete
integer,parameter ::  passed_incomplete_status      = 2;   logical :: passed_incomplete
integer,parameter ::  no_data_lines_000_status      = 3;   logical :: no_data_lines_000;
integer,parameter ::  failed_incomplete_status      = 4;   logical :: failed_incomplete;
integer,parameter ::  no_data_lines_nnn_status      = 5;   logical :: no_data_lines_nnn;
integer,parameter ::  profile_failed_status         = 6;   logical :: profile_failed
integer,parameter ::  sec_prof_failed_status        = 7;   logical :: sec_prof_failed
integer,parameter ::  anals_clima_failed_status     = 8;   logical :: anals_clima_failed
integer,parameter ::  deep_pres_failed_status       = 9;   logical :: deep_pres_failed
integer,parameter ::  date1_posn1_land1_spd4_status =10;   logical :: date1_posn1_land1_spd4
integer,parameter ::  date1_posn1_land4_spd1_status =20;   logical :: date1_posn1_land4_spd1
integer,parameter ::  date1_posn1_land4_spd4_status =30;   logical :: date1_posn1_land4_spd4
integer,parameter ::  date1_posn4_land4_spd4_status =40;   logical :: date1_posn4_land4_spd4
integer,parameter ::  date4_posn1_land1_spd4_status =50;   logical :: date4_posn1_land1_spd4
integer,parameter ::  date4_posn1_land4_spd4_status =60;   logical :: date4_posn1_land4_spd4
integer,parameter ::  date4_posn4_land4_spd4_status =70;   logical :: date4_posn4_land4_spd4
integer,parameter ::  date8_posn4_land4_spd4_status =80;   logical :: date8_posn4_land4_spd4
integer,parameter ::  corrupt_phy_status           =500;   logical :: corrupt_phy
integer,parameter ::  execution_failure_status     =900;   logical :: execution_failure
character(len=64) :: status_explaation(0:NSTATUS)
DATA status_explaation / &
" prof:  0, partially QCed (only 1 data line)                    ", &
" prof:  1, passed complete                                      ", &
" prof:  2, passed incomplete                                    ", &
" prof:  3, no data lines, profile=000                           ", &
" prof:  4, failed incomplete                                    ", &
" prof:  5, no data lines, profile>000                           ", &
" prof:  6, profile failed (primary)                             ", &
" prof:  7, secondary failed, primary ok                         ", &
" prof:  8, analysys/climatology failed all other tests ok       ", &
" prof:  9, deep pressure test failed                            ", &
" dpls: 10, date=1 ok  posn=1 ok  land=1 ok  spd=4 bad           ", &
" dpls: 20, date=1 ok  posn=1 ok  land=4 bad spd=1 ok            ", &
" dpls: 30. date=1 ok  posn=1 ok  land=4 bad spd=4 bad           ", &
" dpls: 40, date=1 ok  posn=4 bad land=0 n/a spd=0 n/a           ", &
" dpls: 50. date=4 bad posn=1 ok  land=1 ok  spd=0 n/a           ", &
" dpls: 60. date=4 bad posn=1 ok  land=4 bad spd=0 n/a           ", &
" dpls: 70. date=4 bad posn=4 bad land=0 n/a spd=0 n/a           ", &
" dpls: 80. date=8 int posn=4 bad land=4 bad spd=0 n/a           ", &
" dpls: 90. date=8 int posn=4 bad land=0 n/a spd=0 n/a           ", &
" perr: 100. PROFILE ERROR corrupt phy file                      "  &
/

! These refer to the phy sections - NoPOS = There is NO position block
logical :: NoHED, NoPOS, NoDAT, OneDAT;

! These refer to the four date/posn/land/speed
logical :: date_bad,  posn_bad,  land_bad,  speed_bad
logical :: date_good,  posn_good,  land_good,  speed_good
integer :: date_flag, posn_flag, land_flag, speed_flag

integer :: ncl;	! nc-call locatins - kept track of in the call to ckr, the funcitn that checks ret vals from nc calls
integer :: ckr;	! the functin that checks the return value

! variables that are used by any of ncmeta, nctraj, nctech or all
integer :: cyc_count

character(len=4) :: gnctype = 'CLOSED';    ! What type of nc file: prof,meta,traj,tech,biop,biot?

logical :: ncmeta_noprof = .false.	! Is this a case where we want ncmeta, but do not have profiles?

character(len=106) :: SEPEQLS = &
    "==========================================================================================================";
character(len=106) :: SEPPLUS = &
    "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
character(len=106) :: SEPMNUS = &
    "----------------------------------------------------------------------------------------------------------";
character(len=116) :: cls1 = &   
    "!                   status  files                                             pres  temp       psal     dg";
character(len=116) :: cls2 = &   
    "!Profile             JS ES  QRMTtBbG itype dpls date     lat  lon  nlev P T S  gid  grsvacCJF  grsvacCJF D";


character(len=11) :: HEDR;  ! write out a REGULAR header for output debug lines
character(len=46) :: HEDN;  ! Write out a FILMSG header for output debug WRTFILE lines (netcdf_arrays only)
character(len=35) :: HEDF;  ! Write out a FILMSG header for output debug WRTFILE lines (all other write_*)
                  ! FILMSG NCMETA i: 3 val: F filmsg: nnFnnnnn
                  ! 123456789012345678901234567890123456789012 - 41 ch long 

integer           :: WRTINDX = 0;        ! What is the index in filmsg of the file being written out?
character(len=6)  :: WRTFILE = 'QCPLUS'; ! What file is being written out?

integer, parameter :: LQC=1, LNC=2, LMETA=3, LTECH=4, LTRAJ=5, LBIOP=6, LBIOT=7, LTSAC=8;

integer :: lns; ! length of a string
integer :: nlt; ! used for some length(trim(<>)) cases
character(len=80) :: c80; ! A generic character strin 80ch long
character(len=256) :: str; ! A miscelaneous string - used in arrays for nf_ calls
integer :: ls

integer :: l;     ! use only in filmsg(l:l) - not as a counter anywhere else
equivalence (WRTINDX,l); ! shorten/simplify filmsg(WRTINDX:WRTINDX) to just plain filmsg(l:l);

! ===========================================================================================================
! Optional extra debug indicators

logical :: DBGIO  = .false.;  ! Do we want to write out inp/out informaiton?
logical :: DBGQCT = .false.; ! Do we want to debug the the HISTORY/qc test values?
logical :: DBGDIR = .false.; ! Do we want to debug the DIRECTION debugs?
logical :: DBGFIX= .false.; ! Do we want to debug the bad gps fix checks
logical :: DBGMCL = .false.; ! Do we want to debug the MC lines in read_phy?
logical :: DBGPOS = .false.; ! Do we want to debug the posiiton determination?
logical :: DBGSOT = .false.; ! Do we want to debug The Early STARTO F TRANSMISSION test?
logical :: DBGVSS = .false.; ! Do we want to debug the vertical sampling scheme, near-surface sampling, NST/STS?
logical :: DOGRID = .false.; ! Do we want to debug and output a grid of points?
logical :: BADGPS = .false.; ! Do we want to debug the detection and correction of bad gps fixes?
logical :: DBGSTS = .false.; ! Do we want to debug near_surface STS test
logical :: DBGNST = .false.; ! Do we want to debug near_surface NST test

! DBGRID variables - red from env var DBGRID
character(len=80) :: gridpar = " ";
character(len=8) :: gridloc = "GLOBAL";
integer :: inpol = 0;
real :: grxlon, grylat
real :: grinc = 1.0;
real :: grxlonlo = -180.0; real :: grxlonhi =  180.0;
real :: grylatlo =  -90.0; real :: grylathi =   90.0;

integer :: tia, tib, tic; ! Used to test crash recovery:  insert tia = 4; tib = 0; tic = tia/tib; ! crash it

! ===========================================================================================================

contains

! ===========================================================================================================
! module functions/subroutines
! ===========================================================================================================

subroutine notopen(caller,sub,file)

    character(*), intent( in )     :: caller
    character(*), intent( in )     :: sub
    character(*), intent( in )     :: file

    ! this is only called if DEBUG is on
    write(STD,'("DBG WARNING ",a," called by ",a," tried to write to unopened ",a," file")')sub,trim(caller),file;	! In notopen

end subroutine notopen


! ===========================================================================================================
subroutine wrtstd(caller,msg) ! write a line to STD (standard output), also TRC & LOG, if open

    character(*), intent( in )     :: caller
    character(*), intent( in )     :: msg

    if(DEBUG)then; write(msgtmp,'("STD ",a)')trim(msg); else; msgtmp = trim(msg); endif
    write(STD,'(a)')trim(msgtmp);											! In wrtstd
    if(trc_open)then; write(TRC,'(a)')trim(msgtmp); elseif(DEBUG)then; call notopen(caller,"wrtstd","TRC"); endif;	! In wrtstd
    if(log_open)then; write(LOG,'(a)')trim(msgtmp); elseif(DEBUG)then; call notopen(caller,"wrtstd","LOG"); endif; 	! In wrtstd

end subroutine wrtstd

! ===========================================================================================================
subroutine wrttrc(caller,msg) ! write a line to TRC & LOG if open

    character(*), intent( in )     :: caller
    character(*), intent( in )     :: msg

    if(DEBUG)then; write(msgtmp,'("TRC ",a)')trim(msg); else; msgtmp = trim(msg); endif
    if(trc_open)then; write(TRC,'(a)')trim(msgtmp); elseif(DEBUG)then; call notopen(caller,"wrttrc","TRC"); endif;	! In wrttrc
    if(log_open)then; write(LOG,'(a)')trim(msgtmp); elseif(DEBUG)then; call notopen(caller,"wrttrc","LOG"); endif;	! In wrttrc

end subroutine wrttrc

! ===========================================================================================================
subroutine wrterr(caller,msg) ! write an error line to STD, ERR, TRC & LOG, if open
!
! Note that wrterr can be used for errors AND warnings - it is the only way to write to the err file
! EXCEPT for writing the final timestamp/exit message, which is done in close_files
!

    character(*), intent( in )     :: caller
    character(*), intent( in )     :: msg

    integer :: i

    if(DEBUG)then;
	write(msgtmp,'("ERR ",a)')trim(msg);
    else;
	write(msgtmp,'(a)')trim(msg);
    endif

    write( STD,'(a)')trim(msgtmp);						! In wrterr
    if(err_open)then;
	if(len(trim(tserr)) > 0) write(ERR,'(a)')trim(tserr); tserr = '';	! In wrterr
	write(ERR,'(a)')trim(msgtmp);     		                	! In wrterr
    endif
    if(trc_open)then; write(TRC,'(a)')trim(msgtmp); endif;			! In wrterr
    if(log_open)then; write(LOG,'(a)')trim(msgtmp); endif;			! In wrterr

end subroutine wrterr

! ===========================================================================================================
subroutine wrtlog(caller,msg); ! Write a line to the LOG

    character(*), intent( in )     :: caller
    character(*), intent( in )     :: msg

    if(msg(1:4) == "DBG ")then;
	msgtmp = trim(msg);
    elseif(DEBUG)then;
	write(msgtmp,'("LOG ",a)')trim(msg); 
    else;
	 msgtmp = trim(msg); 
    endif	! In wrtlog

    if(log_open)then;
	write(LOG,'(a)')trim(msgtmp);                                   ! In wrtlog 
    elseif(index(msgtmp,"open_new_file  unit:   8 type: LOG") > 0)then
	write(LOG,'(a)')trim(msgtmp);                                   ! In wrtlog (log_open has not yet set to true
    elseif(DEBUG)then;
	call notopen(caller,"wrtlog","LOG");                      ! DBG ! In wrtlog
    endif

end subroutine wrtlog

! ===========================================================================================================
subroutine wrtlin(caller,msg); ! Write an unannotated line to the LOG (ie, not 'LOG" atted at beginning for DEBUG)

    character(*), intent( in )     :: caller
    character(*), intent( in )     :: msg

    if(log_open)then; write(LOG,'(a)')trim(msgtmp); elseif(DEBUG)then; call notopen(caller,"wrtlog","LOG"); endif;      ! In wrtlin

end subroutine wrtlin

! ===========================================================================================================
subroutine wrtdbg(caller,msg) ! write a DEBUG line to STD & LOG if open (RETURN if DEBUG off)

    character(*), intent( in )     :: caller
    character(*), intent( in )     :: msg

    if(msg(1:4) == "DBG ")then;
        msgtmp = trim(msg);
    elseif(DEBUG)then;
        write(msgtmp,'("DBG ",a)')trim(msg);
    else;
         msgtmp = trim(msg);
    endif

    if(DEBUG) write(STD,'(a)')trim(msgtmp);
    if(log_open)then; write(LOG,'(a)')trim(msgtmp); else; call notopen(caller,"wrtdbg","DBG"); endif

end subroutine wrtdbg

! ===========================================================================================================
subroutine wrtdbgtag(caller,msg) ! write a DEBUG line to STD & LOG if open (RETURN if DEBUG off) TAG=mark caller

    character(*), intent( in )     :: caller
    character(*), intent( in )     :: msg

    character(len=20) :: lefty  ! left-justified string
    character(len=SZC) :: dbgmsg

    if(DEBUG)then;
        lefty = adjustl(caller);
        write(dbgmsg,'("DBG LEV",i2.2,x,a16,x,a)')DBLEV,trim(lefty),trim(msg);
        write(STD,'(a)')trim(dbgmsg);                                                                              ! In wrtlev
        if(log_open)then; write(LOG,'(a)')trim(dbgmsg); else; call notopen(caller,"wrtdbgtag","LOG"); endif;
    endif

end subroutine wrtdbgtag

! ===========================================================================================================
subroutine wrtlev(caller,msg) ! write a DEBUG line to STD & LOG if open (DBLEV added to message) (RETURN if DEBUG off)

    character(*), intent( in )     :: caller
    character(*), intent( in )     :: msg

    character(len=20) :: lefty  ! left-justified string

    if(DEBUG)then;
	lefty = adjustl(caller);
	write(msgtmp,'("DBG LEV",i2.2,x,a,x,a)')DBLEV,trim(lefty),trim(msg);
	write(msgtmp,'("DBGLEV LEV",i2.2,x,a,x,a)')DBLEV,trim(lefty),trim(msg);
	write(STD,'(a)')trim(msgtmp);									  ! DBG	! In wrtlev
	if(log_open)then; write(LOG,'(a)')trim(msgtmp); else; call notopen(caller,"wrtdbg","LOG"); endif; ! DBG	! In wrtlev
    endif

end subroutine wrtlev

! ===========================================================================================================
subroutine wrtflm(caller,tmp); ! Annotate changes in the 'filmsg' string, write to LOG

    character(*), intent( in )     :: caller
    character(*), intent( in )     :: tmp

    character(len=20) :: lefty  ! left-justified string

    lefty = adjustl(caller);

    if(WRTINDX == 0)then;
	write(msg,'("FILMSG ",a,a9," indx:",i2," val: X from: ",a20,x,a)')WRTFILE,filmsg,l,lefty,trim(tmp);
    else
	write(msg,'("FILMSG ",a,a9," indx:",i2," val:",a2," from: ",a20,x,a)')WRTFILE,filmsg,l,filmsg(l:l),lefty,trim(tmp);
    endif
    if(DEBUG) write(STD,'(a)')trim(msg);
    if(log_open)then; write(LOG,'(a)')trim(msg); elseif(DEBUG)then; call notopen(caller,"wrtlog","LOG"); endif;      ! In wrtflm

end subroutine wrtflm

! ===========================================================================================================
subroutine levmsg(caller,tlev,tmsg) ! Write a DBG msg only if DEBUG is ON and DBLEV is tlev (test lev)

    character(*), intent( in )     :: caller
    integer ,     intent( in )     :: tlev
    character(*), intent( in )     :: tmsg

    character(len=20) :: lefty  ! left-justified string

    if(.not. DEBUG)then; write(STD,*)'TEST levmsg - debug is NOT on, just reutrnjiwth no action'; return; endif
    if(DBLEV /= tlev)then; write(STD,*)'TEST levmsg - DBLEV is NOT equal to tlev, just reutrnjiwth no action'; return; endif

    lefty = adjustl(caller);
    if(tmsg(1:4) == "DBG ")then;
        msgtmp = trim(tmsg);
    else
        write(msgtmp,'("DBG ",a)')trim(tmsg);
    endif

    write(STD,'(a)')trim(msgtmp); ! DBG In communication levmsg
    if(log_open)then; write(LOG,'(a)')trim(msgtmp); else; call notopen(caller,"wrtdbg","DBG"); endif

end subroutine levmsg
! ===========================================================================================================
subroutine dbgmsg(caller,tmsg) ! Write a DBG msg only if DEBUG is ON

    character(*), intent( in )     :: caller
    character(*), intent( in )     :: tmsg

    character(len=20) :: lefty  ! left-justified string

    if(.not. DEBUG) return

    lefty = adjustl(caller);
    if(tmsg(1:4) == "DBG ")then;
        msgtmp = trim(tmsg);
    else
        write(msgtmp,'("DBG ",a)')trim(tmsg);
    endif

    write(STD,'(a)')trim(msgtmp); ! DBG In commuication dbgmsg
    if(log_open)then; write(LOG,'(a)')trim(msgtmp); else; call notopen(caller,"wrtdbg","DBG"); endif

end subroutine dbgmsg

! ===========================================================================================================

subroutine to_upper(str)
  character(*), intent(in out) :: str
  integer :: i

    do i = 1, len(str)
	select case(str(i:i))
	case("a":"z")
	    str(i:i) = achar(iachar(str(i:i))-32)
	end select
    enddo

end subroutine to_upper
! ===========================================================================================================
subroutine set_uppercase(istr,ostr);
  character(*), intent( in ) :: istr
  character(*), intent( out) :: ostr
  integer i;

    ls = len(istr);
    do i = 1, ls;
	select case(istr(i:i))
	case("a":"z")
	    ostr(i:i) = achar(iachar(istr(i:i))-32)
	end select
    enddo

end subroutine set_uppercase
! ===========================================================================================================




  ! below are the OLD write routines -------------------------------------

subroutine error_report(caller,errstatus,nerrmsg,errmsg)
    character(*), intent( in )     :: caller            ! where did the error occur?
    integer, intent( in )          :: errstatus;        ! Not actually used
    integer, intent( in )          :: nerrmsg;          ! How many error lines are there
    character(len=*), intent( in ) :: errmsg(*);        ! What happened

    character(len=12),parameter :: sub = 'error_report';

    integer :: i,ie				! counters

    if(DEBUG) call new_dbg_enter(sub);

    errors_warnings_exist = .true.;             ! There are non-fatal errors

    nerror = nerror + 1;			! counter for NON-FATAL errors

    if(nerror == NERRMAX)then
        write(errmsgs(NERRMAX),'("WARNING number of NON-FATAL errors, exceeds max: ",i5," Do NOT report any more")')NERRMAX
	call wrterr(caller,errmsgs(NERRMAX)) ! in error_report
        return
    elseif(nerror > NERRMAX)then
	return
    endif

    if(DEBUG) call new_dbg_enter(sub);

    call wrterr(sub,SEPMNUS);
    if(DEBUG)then
	write(msg,'("(0) NON-FATAL ERROR ",i3.3," reported for profile ",a," on call from ",a)')errstatus,trim(profile), &
	    trim(caller); 
    else
	write(msg,'("NON-FATAL ERROR ",i3.3," reported for profile ",a," on call from ",a)')errstatus,trim(profile), &
	    trim(caller);
    endif
    call wrterr(sub,msg);
    ! First, document where they came from:
    if(trim(errmsg0) == 'n/a') then
	write(msg,'("NON-FATAL ERROR ",i3.3," occurred in ",a," on profile : ",a20)')errstatus,trim(caller),profile
        errmsgs(nerror) = trim(msg);
    else
        errmsgs(nerror) = trim(errmsg0);
        errmsg0 = 'n/a';
    endif
    ! Now, see if this was called from a subroutine that is trying to WRITE stuff

    if(WRTINDX > 0)then
	write(HEDF,'("FILMSG ",a," err l: ",i1," var/filmsg: ")')WRTFILE,WRTINDX; 1000 format(a,a1,a9,x,a)
	if(DEBUG) write(msg,'("In error report: ",a)')trim(HEDF); call wrtlog(sub,msg);
	filmsg(l:l) = 'f'; call wrtflm(caller,'A NON-FATAL error has occurred');
	
	write(msg,1000)HEDF,filmsg(l:l),filmsg,'A NON-FATAL error occurred opening the output file'; call wrterr(caller,msg);
    endif

    ! Next, write out the rest of the error messages
    do ie=1,nerrmsg
	write(msg,'("",a)')trim(errmsg(ie));
	if(DEBUG)then; msgtmp = trim(msg); write(msg,'("(",i1,") ",a)')ie,trim(msgtmp); endif;
	call wrterr(caller,msg); ! report the NON-FATAL errors ! in error_report
    enddo

    ! Finally, save the first errmsg, to print out at the end
    if(trim(errmsg0) == 'n/a') then
	errmsgs(nerror) = errmsg(1);	! Just save the first line
    else
	errmsgs(nerror) = errmsg0;
	errmsg0 = 'n/a';
    endif 

    if(DEBUG) call new_dbg_leave(sub); 

end subroutine error_report

subroutine warning(caller,status,tmpmsg)
    character(*), intent( in ) :: caller;
    integer,      intent( in ) :: status; ! Note that the status is NOT USED, but is checked
    character(*), intent( in ) :: tmpmsg;

    character(len=SZC) :: warnmsg;
    character(len=7)   :: warnbeg;

    integer :: tstwrn

    character(len=7),parameter :: sub = 'warning';

    ! Make sure you don't fill the forlog with dozens of warings
    nwarn = nwarn + 1;
    if(nwarn == NWRNMAX)then
        nwarn = NWRNMAX;
        write(wrnmsgs(NWRNMAX),'("WRN number of warnings, exceeds max: ",i5," Do NOT report any more")')NWRNMAX
        return
    elseif(nwarn > NWRNMAX)then
        return
    endif

    if(DEBUG)then;
        warnbeg = "DBG Id:";
    else
        warnbeg = "Id:";
    endif

    if(thiscyc > 999)then
        write(warnmsg,'(a,x,a4," cyc:",i4.4,x,a)')trim(warnbeg),aomlid,thiscyc,trim(tmpmsg);
    else
        write(warnmsg,'(a,x,a4," cyc:",i4.3,x,a)')trim(warnbeg),aomlid,thiscyc,trim(tmpmsg);
    endif

    wrnmsgs(nwarn) = trim(warnmsg);                                     ! Stash to print later

end subroutine warning

subroutine lognote(caller,status,tmpmsg)
    character(*), intent( in ) :: caller
    integer,      intent( in ) :: status
    character(*), intent( in ) :: tmpmsg

    character(len=SZC) :: notemsg;
    character(len=7) :: notebeg;
    integer :: nc; ! Number of chrs in the message

    character(len=7),parameter :: sub = 'lognote';

    ! There will be no check for the note code

    if(DEBUG)then;
	notebeg = "DBG Id:";
    else
	notebeg = "Id:";
    endif

    nc = len(trim(tmpmsg));

    if(aomlid == '0000')then
	write(notemsg,'(a,x,a4," cyc: 0000  LOGNOTE",a)')trim(notebeg),tmpmsg(8:nc);
    elseif(thiscyc > 999)then
	write(notemsg,'(a,x,a4," cyc:",i4.4,x,"LOGNOTE",a)')trim(notebeg),aomlid,thiscyc,tmpmsg(8:nc);
    else
	write(notemsg,'(a,x,a4," cyc:",i4.3,x,"LOGNOTE",a)')trim(notebeg),aomlid,thiscyc,tmpmsg(8:nc);
    endif

    call wrtlog(caller,notemsg); ! in subroutine lognote, put into the forlog
    if(DEBUG) write(DBG,'(a)')trim(notemsg);

end subroutine lognote

subroutine check_inp(caller,pathname,tline)
    character(*), intent( in )     :: caller
    character(*), intent( in )     :: pathname
    character(*), intent( in )     :: tline

    character(len=9), parameter :: sub = 'check_inp';

    ! See if the input line was completely filled
    if(len(trim(tline)) == SZC)then
	write(wrnmsg,'("WARNING 106 check_inp found an input line that is too long: ",a)')tline(1:80); ! WRN
	call warning(sub,106,wrnmsg); ! WARNING 106 ! WRN
   endif
end subroutine check_inp

subroutine check_hed(caller,pathname,tline)
    character(*), intent( in )     :: caller
    character(*), intent( in )     :: pathname
    character(*), intent( in )     :: tline

    character(len=9), parameter :: sub = 'check_hed';

    ! See if the input line was completely filled
    if(len(trim(tline)) == SZC)then
        write(bigmsg,'("check_hed called by ",a," INP file: ",a," inp line too long: ",a)')trim(caller),trim(pathname),trim(tline);
	write(wrnmsg,'("WARNING 107 check_hed found an input line that is too long: ",a)')tline(1:80);
	call warning(sub,107,wrnmsg); ! WARNING 107 ! WRN
    endif
    ! See if the line has something in col 41 so it has some knd of 'value'
    if(len(trim(tline)) < 41)then
	if(make_nctraj)then
	    write(errmsg(1),'("FATAL ERROR 522 in ",a," file: ",a)')trim(caller),trim(pathname);
	    write(errmsg(2),'("FATAL ERROR 522 empty value in header line w. variable/value pair (TRAJ=T)")')
	    write(errmsg(3),'("FATAL ERROR 522 line: ",a)')trim(tline);
	    call error_exit(caller,522,3,errmsg); ! check_hed in communication - short line in traj phy file
	elseif(trim(caller) == "read_meta")then
	    write(errmsg(1),'("FATAL ERROR 934 in ",a," file: ",a)')trim(caller),trim(pathname);
	    write(errmsg(2),'("FATAL ERROR 934 empty value in header line w. variable/value pair (TRAJ=F)")')
	    write(errmsg(3),'("FATAL ERROR 934 line: ",a)')trim(tline);
	    call error_exit(caller,934,3,errmsg);; ! check_hed in communication - short line in meta file
	else
	    write(errmsg(1),'("NON-FATAL ERROR 218 in check_hed called by ",a," file: ",a)')trim(caller),trim(pathname);
	    write(errmsg(2),'("NON-FATAL ERROR 218 empty value in header line w. variable/value pair (TRAJ=F)")')
	    write(errmsg(3),'("NON-FATAL ERROR 218 line: ",a)')trim(tline);
	    ! Not getting into the STD - write to STD if DEBUG
	    call error_report(caller,218,3,errmsg);; ! check_hed in communication - short line in pqc file
	endif
    endif
end subroutine check_hed

subroutine dbg_enter(sub)
    character(*), intent( in )     :: sub
    character(len=52) :: linend = ' ================================================ >>';
    write(msg,'(" << Enter ",a16,a)')sub,linend; call wrtdbg(sub,msg);
end subroutine dbg_enter

subroutine new_dbg_enter(sub)
    character(*), intent( in )     :: sub
    character(len=100) :: dbgmsg
    integer :: ln;
    ln = 100 - 18 - len(sub); ! Size of beg ===, subname,end ===.
    write(dbgmsg,'(a10," Enter ",a,x,a)')SEPEQLS(1:10),trim(sub),SEPEQLS(1:ln);
    write(STD,'(a)')trim(dbgmsg); write(LOG,'(a)')trim(dbgmsg);                               ! In new_dbg_enter
end subroutine new_dbg_enter

subroutine new_dbg_leave(sub)
    character(*), intent( in )     :: sub
    character(len=100) :: dbgmsg
    integer :: ln;
    ln = 100 - 18 - len(sub); ! Size of beg ===, subname,end ===.
    write(dbgmsg,'(a10," Leave ",a,x,a)')SEPEQLS(1:10),trim(sub),SEPEQLS(1:ln);
    write(STD,'(a)')trim(dbgmsg); write(LOG,'(a)')trim(dbgmsg);                               ! In new_dbg_leave
end subroutine new_dbg_leave

subroutine new_dbg_leave_logical(sub,result)
    character(*), intent( in )     :: sub
    logical, intent( in )          :: result ! Did it succeed?
    character(len=100) :: dbgmsg
    character(len=120) :: tmsg
    integer :: ln;
    ln = 100 - 18 - len(sub) - 9; ! Size of beg ===, subname,tag,result,end ===.
    write(dbgmsg,'(a10," Leave ",a," result: ",l1,x,a)')SEPEQLS(1:10),trim(sub),result,SEPEQLS(1:ln);
    write(STD,'(a)')trim(dbgmsg); write(LOG,'(a)')trim(dbgmsg);                               ! In new_dbg_leave
end subroutine new_dbg_leave_logical

subroutine dbg_enter_type(sub,type)
    character(*), intent( in )     :: sub
    character(len=4), intent( in ) :: type
    character(len=53) :: linend = '  ================================================ >>';
    write(msg,'(" << Enter ",a," for ",a4,2x,a)')sub,type,linend; call wrtdbg(sub,msg);
end subroutine dbg_enter_type

subroutine dbg_enter_file(sub,file)
    character(*), intent( in )     :: sub
    character(len=*), intent( in ) :: file
    character(len=12) :: linend = '  /////  >> ';
    write(msg,'(" << Enter ",a," for ",a4,2x,a)')sub,linend,trim(file); call wrtdbg(sub,msg);
end subroutine dbg_enter_file

subroutine dbg_leave(sub,iret)
    character(*), intent( in )     :: sub
    integer, intent( in )          :: iret
    character(len=43) :: linend = ' ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ >>';

    write(msg,'("<< Leave ",a," ret=",i4,a)')sub,iret,linend; call wrtdbg(sub,msg);
end subroutine dbg_leave

subroutine dbg_leave_type(sub,type,iret)
    character(*), intent( in )     :: sub
    character(len=4), intent( in ) :: type
    integer, intent( in )          :: iret
    character(len=43) :: linend = ' ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ >>';

    write(msg,'("Leave ",a4,8x,": << Leave ",a," for ",a,"  ret=",i4,a)')sub,sub,type,iret,linend; call wrtdbg(sub,msg);
end subroutine dbg_leave_type

subroutine dblev_exit(qc)
    character(*), intent( in )     :: qc	! Note that qc can be a string of ANY length - may be a messge, not a test number
    character(len=SZC)             :: outmsg
    character(len=10), parameter ::  sub = 'dblev_exit'

    write(outmsg,'("TEST DBLEV EXIT dblev_exit ",a)')qc;	! in dblev_exit 	! DBG
    if(DEBUG)then; call wrtdbg("dblev_exit",outmsg); call exit(999); endif		! In dblev_exit ! SAV DBG
    ! Let Vicki know if this was called, but DEBUG is OFF
    write(wrnmsg,'("WARNING 108 - dblev_exit - Let Vicki know that this was called when DEBUG was OFF!!")'); ! WRN
    call warning(sub,108,wrnmsg); ! WARNING 108 ! WRN
end subroutine dblev_exit

function timestampmsg(Input_String) result ( Output_String )
    character( * ), intent( in )     :: Input_String
    character( len( Input_String )+19 ) :: Output_String
    character(len=8)  :: date
    character(len=10) :: time
    character(len=19) :: formatted_date
    ! For testing, we have no functionality - just out=in

    call date_and_time( DATE=date, TIME=time )
    formatted_date(1:4) = date(1:4)
    formatted_date(5:5) = '-'
    formatted_date(6:7) = date(5:6)
    formatted_date(8:8) = '-'
    formatted_date(9:10)= date(7:8)
    formatted_date(11:11) = ' '
    formatted_date(12:13)= time(1:2)
    formatted_date(14:14) = ':'
    formatted_date(15:16)= time(3:4)
    formatted_date(17:17) = ':'
    formatted_date(18:19)= time(5:6)

    chr4 = date(1:4); read(chr4,*)GMT_year; ! Used to test for BADGPS
    chr2 = date(5:6);  read(chr2,*)GMT_mon; ! Used to test for BADGPS
    chr2 = date(7:8);  read(chr2,*)GMT_day; ! Used to test for BADGPS
    badgps_test_hi = GMT_year + 2; ! Used to test for BADGPS

    Output_String = formatted_date // Input_String


end function timestampmsg

logical function open_new_file(unit,pathname,type,desc)

    integer, intent(in) :: unit              ! file handler
    character(len=*), intent(in) :: pathname ! file name
    character(len=*), intent(in) :: type     ! file type (STD,LOG,etc)
    character(len=*), intent(in) :: desc     ! file_description
    character(len=256) :: commandline
    integer :: ios ! IO
    integer :: status
    logical :: file_exist

    character(len=14), parameter ::  sub = 'open_new_file'

    ! Does the file already exist - if yes the delete it

    inquire(file=pathname,exist=file_exist)
    if(file_exist)then
	write(commandline,'("/usr/bin/rm -f ",a)') pathname(1:index(pathname,"  "))
	call system(commandline)
	inquire(file=pathname,exist=file_exist)
	if(file_exist)then
	    if(DEBUG)then
		write(errmsg(1),'("FATAL ERROR 419 DBG=T open_new_file prof: ",a," cannot open a new file")')profile; !ERR
		write(errmsg(2),'("FATAL ERROR 419 DBG=T OPENING FILE for write (old file not removed) ")'); !ERR
		write(errmsg(3),'("FATAL ERROR 419 DBG=T file: ",a)') pathname(1:index(pathname,"  ")); !ERR
		write(errmsg(4),'("FATAL ERROR 419 DBG=T desc: ",a)') desc; !ERR
		call error_exit(sub,419,4,errmsg); ! DBG=T open_new_file in communication cannot remove old/create new
	    else
		write(errmsg(1),'("NON-FATAL ERROR 319 in open_new_file file: ",a)')trim(pathname);
		write(errmsg(2),'("NON-FATAL ERROR 319 DBG=F open_new_file prof: ",a," cannot open a new file")')profile; !ERR
		write(errmsg(3),'("NON-FATAL ERROR 319 DBG=F OPENING FILE for write (old file not removed) ")'); !ERR
		write(errmsg(4),'("NON-FATAL ERROR 319 DBG=F file: ",a)') pathname(1:index(pathname,"  ")); !ERR
		write(errmsg(5),'("NON-FATAL ERROR 319 DBG=F desc: ",a)') desc; !ERR
		call error_report(sub,319,5,errmsg); ! DBG=F open_new_file in communication cannot remove old/create new
		open_new_file = .false.; return;
	    endif
	endif
    endif
    open(unit,file=pathname,form='formatted',status='new',recl=1000,iostat=ios) 	   ! In open_new_file

    if(type == "LOG") write(LOG,'(a)')trim(tsbeg);                        !in open_new_file
    if(type == "TRC") write(TRC,'(a)')trim(tsbeg);                        !in open_new_file

    if(DBGIO)then; write(msg,'("DBGIO Unit ",a3," is open/new (",a,")")')type,desc; call wrtdbg(sub,msg); endif ! DBGIO
    if(ios/=0)then
	write(errmsg(1),'("NON-FATAL ERROR 220 in open_new_file file: ",a)')trim(pathname);
	write(errmsg(2),'("NON-FATAL ERROR 220 OPENING FILE for write")');
	write(errmsg(3),'("NON-FATAL ERROR 220 file: ",A)') pathname(1:index(pathname,"  "));
	write(errmsg(4),'("NON-FATAL ERROR 220 ",a)') desc;
	call error_report(sub,220,4,errmsg); ! open_new_file in communication cannot open new file
	open_new_file = .false.; return;
    endif
    if(DBGIO)then; write(msg,'("OPNCLS ",a)')trim(msg); call wrtlog(sub,msg); endif
    open_new_file = .true.

end function open_new_file

logical function open_old_file(unit,pathname,type,desc)

    integer, intent(in) :: unit              ! file handler
    character(len=*), intent(in) :: pathname ! file name
    character(len=*), intent(in) :: type     ! file type (STD,LOG,etc)
    character(len=*), intent(in) :: desc     ! file_description
    integer :: ios ! IO
    integer :: status
    logical :: file_exist

    character(len=14), parameter ::  sub = 'open_old_file'

    ! Make sure the file exists

    inquire(file=pathname,exist=file_exist)
    if(.not. file_exist)then
        write(errmsg(1),'("FATAL ERROR 803 MISSING INPUT FILE at 1")');
        write(errmsg(2),'("FATAL ERROR 803 file name: ",A)')trim(pathname);
        write(errmsg(3),'("FATAL ERROR 803 file type: ",a)') desc;
        call error_exit(sub,803,3,errmsg); ! open_old_file in communicatin - unable to open old file - does not exist
    endif
    open(unit,file=pathname,form='formatted',status='old',action='read',iostat=ios)        ! In open_old_file
    if(DBGIO)then; write(msg,'("DBGIO Unit ",a3," is open/old (",a,")")')type,desc; call wrtdbg(sub,msg); endif ! DBGIO
    if(ios/=0)then
        write(errmsg(1),'("FATAL ERROR 804 ERROR OPENING INPUT FILEat 2")');
        write(errmsg(2),'("FATAL ERROR 804 ios: ",i2," file: ",A)') ios,pathname(1:index(pathname,"  "));
        write(errmsg(3),'("FATAL ERROR 804 ",a)') desc;
        call error_exit(sub,804,3,errmsg); ! open_old_file in communicatin - unable to open old file - open failed
    endif
    if(DBGIO)then; write(msg,'("OPNCLS ",a)')trim(msg); call wrtlog(sub,msg); endif
    open_old_file = .true.

end function open_old_file

logical function open_old_file_nf(unit,pathname,type,desc);

    integer, intent(in) :: unit              ! file handler
    character(len=*), intent(in) :: pathname ! file name
    character(len=*), intent(in) :: type     ! file type (STD,LOG,etc)
    character(len=*), intent(in) :: desc     ! file_description
    integer :: ios ! IO
    integer :: status
    logical :: file_exist

    character(len=14), parameter ::  sub = 'open_old_file'

    ! Make sure the file exists

    inquire(file=pathname,exist=file_exist)
    if(.not. file_exist)then
	write(errmsg(1),'("NON-FATAL ERROR 221 in open_new_file file: ",a)')trim(pathname);
	write(errmsg(2),'("NON-FATAL ERROR 221 MISSING INPUT FILE at 1")');
	write(errmsg(3),'("NON-FATAL ERROR 221 file: ",A)') pathname(1:index(pathname,"  "));
	write(errmsg(4),'("NON-FATAL ERROR 221 ",a)') desc;
	call error_report(sub,221,4,errmsg); ! open_old_file in communication cannot find old file
	open_old_file_nf = .false.; ! The file does not exist, so was NOT opened correctly
	return
    endif
    open(unit,file=pathname,form='formatted',status='old',action='read',iostat=ios) 	   ! In open_old_file_nf
    if(DBGIO)then; write(msg,'("DBGIO Unit ",a3," is open/old (",a,")")')type,desc; call wrtdbg(sub,msg); endif ! DBGIO
    if(ios/=0)then
	write(errmsg(1),'("NON-FATAL ERROR 222 in open_old_file ile: ",a)')trim(pathname);
	write(errmsg(2),'("NON-FATAL ERROR 222 OPENING INPUT FILEat 2")');
	write(errmsg(3),'("NON-FATAL ERROR 222 ios: ",i2," file: ",A)') ios,pathname(1:index(pathname,"  "));
	write(errmsg(4),'("NON-FATAL ERROR 222 ",a)') desc;
	call error_report(sub,222,4,errmsg); ! open_old_file in communication - cannot open old fle
	open_old_file_nf = .false.; ! The file did NOT open correctly
	return
    endif
    open_old_file_nf = .true.

end function open_old_file_nf

logical function open_app_file(unit,pathname,type,desc)

    integer, intent(in)          :: unit     ! file handler
    character(len=*), intent(in) :: pathname ! file name
    character(len=*), intent(in) :: type     ! file type (STD,LOG,etc)
    character(len=*), intent(in) :: desc     ! file_description
    integer :: ios ! IO
    integer :: irecl 
    integer :: status
    logical :: file_exist

    character(len=14), parameter ::  sub = 'open_app_file'

    if(DBGIO) call new_dbg_enter(sub);

    ! Does the file already exist - if no then create it if yes then append to it

    open_app_file = .false.; ! Assume failure

    inquire(file=pathname,exist=file_exist)
            if(index(pathname,"to_gtsgateway") .gt. 0)then;
               irecl=16384
            else
               irecl=1000
    endif
    if(file_exist)then
	open(unit,file=pathname,form='formatted',position='append',recl=irecl,iostat=ios);  ! In open_app_file @1
	if(DBGIO)then; write(msg,'("DBGIO Unit ",a3," is open/app (",a,")")')type,desc; call wrtdbg(sub,msg); endif ! DBGIO
    else
	open(unit,file=pathname,form='formatted',status='new',recl=irecl,iostat=ios);	   ! In open_app_file @2
	if(DBGIO)then; write(msg,'("DBGIO Unit ",a3," is open/new (",a,")")')type,desc; call wrtdbg(sub,msg); endif ! DBGIO
    endif
    if(ios/=0)then
	write(errmsg(1),'("FATAL ERROR 805 error opening file for write/append")');
	write(errmsg(2),'("FATAL ERROR 805 file name: ",A)') pathname(1:index(pathname,"  "));
	write(errmsg(3),'("FATAL ERROR 805 file desc: ",a)') desc;
	call error_exit(sub,805,3,errmsg); ! open_app_file in communication - cannot open file for append
    endif
    if(DBGIO)then; write(msg,'("OPNCLS ",a)')trim(msg); call wrtlog(sub,msg); endif

    open_app_file = .true.; ! Report success

    if(DBGIO) call new_dbg_leave_logical(sub,open_app_file);

end function open_app_file

logical function close_file(unit,type,dscr)

    integer, intent(in)          :: unit ! file handler
    character(len=*), intent(in) :: type ! file type (STD,LOG,etc)
    character(len=*), intent(in) :: dscr ! file description

    character(len=14), parameter ::  sub = 'close_file    '

    close(unit); close_file = .false.;									! In c.close_file

    if(DBGIO)then; write(msg,'("DBGIO Unit ",a3," is closed (",a,")")')type,dscr; call wrtdbg(sub,msg); endif ! DBGIO

end function close_file

subroutine close_files()
    character(len=14), parameter ::  sub = 'close_files   '


    write(msg,'(" =-=-=-=-=-= Leave QCPLUS - ",a20,"=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=")')profile; tsend=timestampmsg(msg)
    
    if(log_open) write(LOG,1000) SEPMNUS,cls1,cls2,trim(repmsg),SEPMNUS,trim(tsend); ! LOG reportline/end timestamp ! In close_files
    if(trc_open) write(TRC,1000) SEPMNUS,cls1,cls2,trim(repmsg),SEPMNUS,trim(tsend); ! TRC reportline/end timestamp ! In close_files
    if(err_open)then;
	if(errors_warnings_exist)then;
	         write(ERR,1000) SEPMNUS,cls1,cls2,trim(repmsg),SEPMNUS,trim(tsend); ! ERR reportline/end timestamp ! In close_files
	endif
    endif
    1000 format(a,/,a,/,a,/,a,/,a,/,a);

    if(err_open) err_open=close_file(ERR,"ERR","error file");						           ! In close_files
    if(trc_open) trc_open=close_file(TRC,"TRC","trace file");						           ! In close_files
    if(log_open) log_open=close_file(LOG,"LOG","forlog file");						           ! In close_files

end subroutine close_files

subroutine add_reportline(caller)

    character(*), intent( in )     :: caller

    character(len=3) :: dot = "'.'"	! "dot" for report header
    character(len=3) :: uns = "'_'";	! "underscore" for report header

    character(len=14), parameter ::  sub = 'add_reportline'

    logical :: file_exist

    ! Does the file already exist - if not then write the header to it after open_app_file opens it,
    ! otherwise if if yes then append to it

    if(DBGIO) call new_dbg_enter(sub);

    ! The next test should NEVER be possible, but keep it in, just in case ...
    if(trim(pathname_report) == "NOT DEFINED")then
	write(errmsg(1),'("FATAL ERROR 521 - pathname_report is NOT DEFINED")')
	if(len(trim(profile)) == 0) then
	    write(errmsg(2),'("FATAL ERROR 521 profile name is also not defined)")');
	else
	    write(errmsg(2),'("FATAL ERROR 521 profile name IS defined)")');
	endif
	call error_exit(sub,521,2,errmsg); ! add_reportline in communication - test if pathname_report == "NOT DEFINED
    endif

    ! ===================================================================================
    inquire(file=pathname_report,exist=file_exist)
    out_open=open_app_file(OUT,pathname_report,"OUT","report file");

    ! The file did NOT EXIST before calling open_app_file.  It has not bee opened, but need the header
    if(.not. file_exist)then
      call add_reporthead()
    endif

    write(OUT,'(a)')trim(repmsg); ! Write out the report line itself. The header lines have already been written. (add_reporthed above)

    out_open=close_file(OUT,"OUT","report file"); ! In add_report

    if(DBGIO) call new_dbg_leave(sub);

end subroutine add_reportline

subroutine add_ftpgtsline()   ! TESTOUT

    character(len=14), parameter ::  sub = 'add_ftpgtsline'
    logical :: file_exist

    if(DBGIO) call wrtdbg(sub,"================================== Enter add_ftpgtsline ==================================");

    ! Does the file already exist - if no then create it if yes then append to it

    out_open=open_app_file(OUT,pathname_ftpgts,"OUT","ftpgts file");

    write(OUT,'(a)')trim(ftpmsg); if(DEBUG)then; write(msg,'("write ftpgts line: ",a)')trim(ftpmsg); call wrtlog(sub,msg); endif

    out_open=close_file(OUT,"OUT","ftpgts file");

    if(DBGIO) call wrtlog(sub,"Leave add_ftpgtsline");

end subroutine add_ftpgtsline

subroutine add_2tesacline()   ! TESTOUT

    character(len=14), parameter ::  sub = 'add_2tesacline'
    logical :: file_exist

    if(DBGIO) call wrtlog(sub,"================================== Enter add_2tesacline ==================================");

    out_open=open_app_file(OUT,pathname_2tesac,"OUT","2tesac file");

    write(OUT,'(a)')trim(tesmsg); if(DEBUG)then; write(msg,'("write 2tesac line: ",a)')trim(tesmsg); call wrtlog(sub,msg); endif

    out_open=close_file(OUT,"OUT","2tesac file");

    if(DBGIO) call wrtlog(sub,"Leave add_2tesacline")

end subroutine add_2tesacline

!=========================================================================================
! This will pass the CALLER through any write statements, so we see who has caled error_exit
subroutine error_exit(caller,errstatus,nerrmsg,errmsg)	! program error, as opposed to  profile error

    character(*), intent( in )     :: caller            ! where did the error occur?
    integer, intent( in )          :: errstatus;        ! Not actually used
    integer, intent( in )          :: nerrmsg;          ! How many error lines are there
    character(len=*), intent( in ) :: errmsg(*);        ! What happened

    integer :: i,ie						! counters

    character(len=10),parameter :: sub = 'error_exit';

    if(DEBUG) call new_dbg_enter(sub);

    errors_warnings_exist = .true.;             ! There are fatal errors

    JOB_STATUS = errstatus; ! This is a FATAL ERROR Which individual error was caused (see error_codes.info)
    EXIT_STATUS = 4;        ! This is a FATAL ERROR - set in error_exit, not in set_error_exit

    call wrterr(sub,SEPEQLS);
    write(msg,'("FATAL ERROR ",i3.3," reported for profile ",a," on call from ",a)')errstatus,trim(profile),trim(caller); 
    if(DEBUG)then; msgtmp = trim(msg); write(msg,'("(0) ",a)')trim(msgtmp); endif;
    if(.not. log_open)then
	write(msg,'("FATAL ERROR reported BEFORE or DURING forlog file opening")'); call wrterr(caller,msg); ! in error_exit
	write(msg,'("FATAL ERROR the error message will be written to STD (standard output)")'); call wrterr(caller,msg); ! in error_exit
    elseif(.not. err_open)then
	write(msg,'("FATAL ERROR reported BEFORE or DURING error file opening")'); call wrterr(caller,msg); ! in error_exit
	write(msg,'("- the error message will be written to STD & LOG")'); call wrterr(caller,msg); ! in error_exit
    elseif(.not. trc_open)then
	write(msg,'("FATAL ERROR reported BEFORE or DURING trace file opening")'); call wrterr(caller,msg); ! in error_exit
	write(msg,'("- the error message will be written to STD, LOG & ERR")'); call wrterr(caller,msg); ! in error_exit
    else
	call wrterr(caller,msg); ! in error_exit
    endif

    ! See if this was called from a subroutine that is trying to WRITE stuff
    if(WRTINDX > 0)then
	write(HEDF,'("FILMSG ",a," ERR l: ",i1," var/filmsg: ")')WRTFILE,WRTINDX; 1000 format(a,a1,a9,x,a)
	filmsg(l:l) = 'F'; call wrtflm(caller,'A FATAL error has occurred');
	write(msg,1000)HEDF,filmsg(l:l),filmsg,'A FATAL error occurred opening the output file'; call wrterr(caller,msg);

    endif

    ! Next, write out the rest of the error messages
    do ie=1,nerrmsg
	write(msg,'("",a)')trim(errmsg(ie));
	if(DEBUG)then; msgtmp = trim(msg); write(msg,'("(",i1,") ",a)')ie,trim(msgtmp); endif;
	call wrterr(caller,msg); ! in error_exit
    enddo
    call wrterr(sub,SEPEQLS);

    ! add the repmsg to the report file - to log the FAILURE

    write(msg,'(a20,i3.3,"  4  ",a8," FATAL ERROR ",i3)')repmsg(1:20),errstatus,filmsg,errstatus
    if(err_open)then
	write(repmsg,'(a," see: ",a)')trim(msg),trim(pathname_error);
    else
	write(repmsg,'(a," error fileis  NOT OPEN - tell Vicki where STD output goes")')trim(msg)
    endif

    if(trim(pathname_report) == 'NOT DEFINED')then
        write(msg,'("PROGRAM_ERROR the report filename is NOT defined")');		
	call wrterr(caller,msg); ! in error_exit
        write(msg,'("PROGRAM_ERROR the report filename is NOT defined, no entry made")');
	call wrterr(caller,msg); ! in error_exit
        write(msg,1001)errstatus,EXIT_STATUS;
	call wrterr(caller,msg) ! in error_exit
        1001 format("ERROR EXIT from add_reportline with JOB STATUS: ",i3," and EXIT STATUS ",i3)
    else
        call add_reportline('error_exit');									! In error_exit
    endif

    write(msg,1002)trim(caller),JOB_STATUS,EXIT_STATUS;	
    call wrterr(caller,msg) ! in error_exit
    1002 format("PROGRAM_ERR EXIT from ",a," with JOB_STATUS: ",i3," and EXIT_STATUS: ",i3)

    if(DEBUG) call new_dbg_leave(sub);

    call close_files();

    call direct_exit(sub,EXIT_STATUS,'exit from error_exit');

end subroutine error_exit

  !==========================================================================
integer function set_exit_status(jobstatus)

    integer, intent( in ) :: jobstatus

    ! Note that for FATAL errors, the call eot error exit sets EXIT_STATUS = 4, and exits directly

    set_exit_status = 0;		! EXIT_STATUS = 0 - Assume it needs no attention
    if(JOB_STATUS >= 300)then;
	set_exit_status = 4;		! EXIT_STATUS = 4  = Real Big problem, FATAL errors, notify the.powers.tthat.be
    elseif(JOB_STATUS >= 200)then;
	set_exit_status = 3;		! EXIT_STATUS = 3  - Big problems, NON-FATAL errors, notify the.powers.tthat.be
    elseif(JOB_STATUS >= 100) then;
	set_exit_status = 2;		! EXIT_STATUS = 2  - WARNINGS,  notify the.powers.that.be
    elseif(jobstatus>=4)then;
	set_exit_status = 1;		! EXIT_STATUS = 1  - Not a perfect profile, attention needed
    endif

end function set_exit_status

!========================================================================================
  ! This get an env var that is REQUIRED - it will exit with an error
  ! if the variable is not found, or is empty
subroutine get_required_envvar(caller,envvar,envval)
    character(*), intent( in ) :: caller
    character(*),  intent( in ) :: envvar	! What is the name of the env var?
    character(*),  intent( out) :: envval	! What is the value to send back?

    character(len=14), parameter ::  sub = 'get_req_envvar'

    integer :: status

    call GETENV(envvar,envval);
    if(len(trim(envval)) == 0)then
	if(envvar == 'report_tabfile')then
	    ! special case, we do not have the name of the report file, cannot call
	    ! normal error_exit - just return, qcplus will test and call the special eror
	else
	    write(errmsg(1),'("FATAL ERROR 890 Failed to retrieve required env var")');
	    write(errmsg(2),'("FATAL ERROR 890 The variable is ",a)')trim(envvar);
	    write(errmsg(1),'("FATAL ERROR 890 (1) Failed to retrieve required env var")');
	    write(errmsg(2),'("FATAL ERROR 890 (2) The variable is ",a)')trim(envvar);
	    call error_exit(caller,890,2,errmsg); ! get_req_envvar in communication - failed to get require envvar
	endif
    endif
    if(DEBUG)then;
	write(msg,'("len: ",i3," var: ",a," val: ",a)')len_trim(envval),envvar,envval;
	if(log_open)then;
	    ! TEST/TEST call wrtdbg(sub,msg);
	else;
	    write(msg,'("DBG LEV",i2.2,"   get_req_envvar len: ",i3," var: ",a," val:")')DBLEV,len_trim(envval),envvar;
	    write(msg,'(a," ",a)')trim(msg),trim(envval);
	    ! TEST/TEST write(STD,'(a)')trim(msg);					! DBG In communication Direct write in cm.get_req_envvar
	endif
    endif

end subroutine get_required_envvar

  !==========================================================================
  !
  ! This gets env vars that are NOT required, but will be defaulted if then
  ! are not defined in the env
subroutine get_default_envvar(caller,envvar,envval)
    character(*), intent( in ) :: caller
    character(*),  intent( in ) :: envvar       ! What is the name of the env var?
    character(*),  intent( out) :: envval       ! What is the value to send back?
    character(SZC) :: answer

    call GETENV(envvar,answer);
    if(len(trim(answer)) > 0)then
	envval = trim(answer);
    else
	if(DEBUG)then
	    if(envvar(1:3) .ne. "DBG") then
		write(logmsg,'("WARNING 109 A defaulted envvar was NOT found: ",a)')trim(envvar); ! LGN
		if(log_open)then
		    call lognote(caller,109,logmsg); ! LGN WARNING 109 subroutine get_default_envvar
		else
		    call wrtdbg(caller,logmsg);  ! LOG not open yet, write to STD
		endif
	    endif
	endif
    endif

end subroutine get_default_envvar

!=========================================================================
logical function leap(iyr)

integer, intent( in ) :: iyr

if(mod(iyr,4)/=0)then;
    leap = .false.
elseif(mod(iyr,100)/=0)then;
    leap = .true.
elseif(mod(iyr,400)/=0)then;
    leap = .false.
else
    leap = .true.
endif
end function leap

!=========================================================================
subroutine squeeze(inpstr,outsqz)
! This functin squeezes the 'excess' blanks out of a character string
! producing an output string with only a single blank between fields
! inpstr: 'This has   many    double  spaces'
! outsqz: 'This has  many double spaces'

character(*), intent( in  ) :: inpstr    ! input string to squeeze
character(*), intent( out ) :: outsqz    ! the squeezed string
character(len=SZC) :: dummy             ! dummy char string to hold intermediate values
integer :: igap, iend                   ! where a 2-sp gap occurrs, and the length of the filtered sring

dummy = trim(inpstr); igap = index(trim(dummy),"  "); iend = len(trim(dummy));
do while(igap > 0)
    igap = index(trim(dummy),"  "); iend = len(trim(dummy));
    dummy = dummy(1:igap) // dummy(igap+2:iend);
enddo
write(outsqz,'(a)')trim(dummy);
return
end subroutine squeeze

!=========================================================================
subroutine squeezerep(inpstr,outsqz)
! This functin squeezes the 'excess' blanks out of a character string
! producing an output string with only a single blank between fields
! This ALSO replaces / with space, and : with space
! inpstr: 'This has   many    double  spaces'
! outsqz: 'This has  many double spaces'

character(*), intent( in  ) :: inpstr    ! input string to squeeze
character(*), intent( out ) :: outsqz    ! the squeezed string
character(len=SZC) :: dummy             ! dummy char string to hold intermediate values
integer :: igap, iend                   ! where a 2-sp gap occurrs, and the length of the filtered sring
integer :: isch                         ! search for a character

dummy = trim(inpstr); igap = index(trim(dummy),"  "); iend = len(trim(dummy));
isch = index(dummy,"/");
do while (isch > 0)
    dummy(isch:isch) = " ";
    isch = index(dummy,"/");
enddo

isch = index(dummy,":");
do while (isch > 0)
    dummy(isch:isch) = " ";
    isch = index(dummy,":");
enddo

do while(igap > 0)
    igap = index(trim(dummy),"  "); iend = len(trim(dummy));
    dummy = dummy(1:igap) // dummy(igap+2:iend);
enddo
write(outsqz,'(a)')trim(dummy);
return
end subroutine squeezerep

!--------------------------------------------------------------------------------------------------
! To check a given string is a number or not
!---------------------------------------------------------------------------------------------------
logical function is_numeric(str)
    character(len=*), intent(in) :: str
    real :: converted_val
    integer :: iostat_inicator
    read(str,*,iostat=iostat_inicator) converted_val
    if(iostat_inicator == 0) then
	is_numeric = .true.
    else
	is_numeric = .false.
    endif
end function is_numeric
!--------------------------------------------------------------------------------------------------
! To check a given string contains only digits with or without blank spaces.
!---------------------------------------------------------------------------------------------------
logical function is_all_numeric(str)
    character(len=*), intent(in) :: str
    real :: converted_val
    integer :: iostat_indicator, idx;
    
    do idx=1,len_trim(str)
        if(str(idx:idx) /= ' ')then
            read(str(idx:idx),*,iostat=iostat_indicator) converted_val
            if(iostat_indicator /= 0) then
	        is_all_numeric = .false.
                exit; !breaks the loop.
            endif
            is_all_numeric = .true.;
        endif    
    enddo
end function is_all_numeric
!--------------------------------------------------------------------------------------------------
! compost the name of a profile qc/nc/phy/etc file
!--------------------------------------------------------------------------------------------------
character(MAXPROFCHR) function compose_profname(caller,floatname,ncyc,extn)

    character(*), intent( in )     :: caller
    character(len=*), intent(in)  :: floatname		! Float "name" aomlid_wmoid
    integer,          intent(in)  :: ncyc		! cycle number
    character(len=*), intent(in)  :: extn		! File extension

    character(len=MAXPROFCHR) :: filename

    if(ncyc < 1000)then
	write(filename,'(a,"_",i3.3,a)')trim(floatname),ncyc,trim(extn)
    else
	write(filename,'(a,"_",i4.4,a)')trim(floatname),ncyc,trim(extn)
    endif

    compose_profname = filename

    return
end function compose_profname
	
subroutine add_reporthead()

    integer, parameter :: NHEAD = 89; ! As of 20161221 - note last element is NOT to be written
    integer :: i
    character(len=107), dimension(NHEAD) :: head =  (/ &
 "! Status:                                                                                                  ", &
 "!                                                                                                          ", &
 "! For JOB_STATUS      EXIT_STATUS is                                                                       ", &
 "!  JS =   0-  3  then ES = 0 -> good or acceptable profile                                                 ", &
 "!  JS =   4- 99  then ES = 1 -> 1 -> bad/incomplete profile, or date/posn/land/speed problem               ", &
 "!  JS = 100-199  then ES = 2 -> WARNINGS occurred, processing continued                                    ", &
 "!  JS = 200-299  then ES = 3 -> NON-FATAL ERRORS occurred, processing continued                            ", &
 "!  JS = 300-999  then ES = 4 -> FATAL ERRORS occurred, processing TERMINATED                               ", &
 "!                                                                                                          ", &
 "! Profile results                                                                                          ", &
 "! Job_Status one-s column 0-9:                                                                             ", &
 "!                                                                                                          ", &
 "!  0: partially QCed (only 1 data line)                                                                    ", &
 "!  1: passed complete                                                                                      ", &
 "!  2: passed incomplete (zerolines or checksum eror)                                                       ", &
 "!  3: no data lines, profile=000                                                                           ", &
 "!  4: failed incomplete (zerolines or checksum eror)                                                       ", &
 "!  5: no data lines, profile>000                                                                           ", &
 "!  6: profile failed (primary)                                                                             ", &
 "!  7: secondary failed, primary ok                   (NOT IMPLEMENTED IN V1.0)                             ", &
 "!  8: analysys/climatology failed all other tests ok (NOT IMPLEMENTED IN V1.0)                             ", &
 "!  9: deep pressure test failed                                                                            ", &
 "!                                                                                                          ", &
 "! Date/Position/Land/Speed check                                                                           ", &
 "! JobStatus ten-s colume (10-90:                                                                           ", &
 "!                                                                                                          ", &
 "!dpls = 1111 > add  0, date=1 ok  posn=1 ok  land=1 ok  spd=1 ok                                           ", &
 "!dpls = 1114 > add 10, date=1 ok  posn=1 ok  land=1 ok  spd=4 bad                                          ", &
 "!dpls = 1141 > add 20, date=1 ok  posn=1 ok  land=4 bad spd=1 ok                                           ", &
 "!dpls = 1144 > add 30. date=1 ok  posn=1 ok  land=4 bad spd=4 bad                                          ", &
 "!dpls = 1400 > add 40, date=1 ok  posn=4 bad land=0 n/a spd=0 n/a                                          ", &
 "!dpls = 4110 > add 50, date=4 bad posn=1 ok  land=1 ok  spd=0 n/a                                          ", &
 "!dpls = 4140 > add 60, date=4 bad posn=1 ok  land=4 bad spd=0 n/a                                          ", &
 "!dpls = 4400 > add 70, date=4 bad posn=4 bad land=0 n/a spd=0 n/a                                          ", &
 "!dpls = 8440 > add 80, date=8 int posn=4 bad land=4 bad spd=0 n/a                                          ", &
 "!dpls = 8400 > add 90, date=8 int posn=4 bad land=0 n/a spd=0 n/a                                          ", &
 "!(8=int=interpolated)                                                                                      ", &
 "!                                                                                                          ", &
 "! Note that posn qc may be bad either impossible location (off the globe) or on lnad                       ", &
 "!                                                                                                          ", &
 "!Output file production - columns: QRMTtBbG                                                                ", &
 "!    Q = qc profile file                            y = file requested/created  SUCCEEDED                  ", &
 "!    R = R<WMOID>_nnn.nc CORE ARGO profile file     n = file not requested                                 ", &
 "!    M = <WMOID>_meta.nc file                       F = file requested/creation FAILED (fatal error)       ", &
 "!                                                   f = file requested/creation FAILED (non-fatal error)   ", &
 "!                                                   z = zero levels, no R file creaed (ncprof only)        ", &
 "!    T = <WMOID>_tech.nc file                       1 = 1 good level, not sent (TESAC only)                ", &
 "!    t = <WMOID>_traj.nc filei                      o = old, too old to send to GTS                        ", &
 "!    B = BR<WMOID>_nnn.nc Bio-ARGO profile file     w = profile zerolines, wait to send to GTS             ", &
 "!    b = B<WMOID>_traj.nc Bio-ARGO trajectory file  0 = zero levels, not sent to GTS                       ", &
 "!    G = GTS/TESAC file                             1 = 1 good level, not sent to GTS                      ", &
 "!                                                   r = file was requested, could not be produces          ", &
 "!The itype is shorthand for the instruemnt type                                                            ", &
 "!                                                                                                          ", &
 "!DPLS - Date/Posn/Land/Speed                                                                               ", &
 "!    d = date         '.'=good, 4=bad, 9=missing                                                           ", &
 "!    p = position     '.'=good, 4=bad, 9=missing                                                           ", &
 "!    l = land/water   '.'=good=water, 4=bad=land, '_'=cannot be tested                                     ", &
 "!    s = speed check  '.'=good, 4=bad, 9=missing                                                           ", &
 "!                                                                                                          ", &
 "!The date/lat/lon/number of levels are next, then the three overall parameter flags for P/T/S              ", &
 "!                                                                                                          ", &
 "!These are the possible flags or indicators for the individual parameter rest:                             ", &
 "!                                                                                                          ", &
 "!For flags (gid,grsvac): '.'=good, 4=bad, 9=missing,'_'=cannot be tested                                   ", &
 "!                                                                                                          ", &
 "!For indicators (CJF):                                                                                     ", &
 "!                                                                                                          ", &
 "!Individual test flags for pres                                                                            ", &
 "!   g  = gross (global) range test                                                                         ", &
 "!   i  = pres increasing                                                                                   ", &
 "!   d  = deepest pres                                                                                      ", &
 "!Individual test flags for temp & psal                                                                     ", &
 "!    g = gross (global) range test                                                                         ", &
 "!    r = regional range test                                                                               ", &
 "!    s = spike check                                                                                       ", &
 "!    v = vertical gradient                                                                                 ", &
 "!    a = analysis test                                                                                     ", &
 "!    c = climatology test                                                                                  ", &
 "!    d = density gradient/inversion (psal only)                                                            ", &
 "!    C = constant/stuck value test                                                                         ", &
 "!    J = jump check (sensor drift)                                                                         ", &
 "!    F = frozen profile indicator (0=good,1=may be frozen                                                  ", &
 "!                                                                                                          ", &
 "!    D = density inversion test, applies to both temp and psal                                             ", &
 "!                                                                                                          ", &
 "!                   status  files                                             pres temp       psal     dg  ", &
 "!Profile             JS ES  QRMTtBbG itype dpls date     lat  lon  nlev P T S  gid  grsvacCJF  grsvacCJF D ", &
 "12345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567" /);


do i=1,NHEAD-1 ! The last element is just a character counter
    write(OUT,'(a)')trim(head(i))
enddo
return
end subroutine add_reporthead
! ===========================================================================================================
integer function lt(chrstr) ! returns len(trim(chrstr))

    character(*), intent( in )     :: chrstr

    lt = len(trim(chrstr));

    return
end function lt
! ===========================================================================================================

subroutine direct_exit(caller,ES,reason);
    character(*), intent( in ) :: caller; ! Who called me?
    integer, intent(in)        :: ES      ! Exit Status to be returned
    character(*), intent( in ) :: reason; ! Whey are we exiting?

    character(len=11),parameter :: sub = 'direct_exit';

    if(DEBUG)then; write(msg,1003)trim(caller),ES,trim(reason); call wrtstd(sub,msg); endif ! DBG
    1003 format("DBG EXIT QC+ direct exit requested by ",a,", Exit Status: ",i3," Reason: ",a);
    write(msg,'(" =-=-=-=-=-= Leave QCPLUS - ",a20,"=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=")')profile; tsend=timestampmsg(msg)
    write(STD,'(a)')trim(msg);; ! In direct_exit in communication
 
    call exit(ES); ! SAV In direct_exit in communication 

end subroutine direct_exit

! ===========================================================================================================

subroutine flush_forlog(unit,pathname,type,dscr,cmnt)

    integer, intent(in)          :: unit ! file handler
    character(len=*), intent(in) :: pathname ! file name
    character(len=*), intent(in) :: type ! file type (STD,LOG,etc)
    character(len=*), intent(in) :: dscr ! file description
    character(len=*), intent(in) :: cmnt ! Where the file was closed (&| reason)

    character(len=14), parameter ::  sub = 'flush_forlog  '

    if(unit == LOG)then
	write(msg,'("FLUSH FORLOG unit: ",i2," type: ",a3," dscr: ",a," cmnt: ",a)')unit,type,dscr,trim(cmnt);
	write(LOG,'(a)')trim(msg); if(DEBUG) write(DBG,'(a)')trim(msg); 
	log_open=close_file(unit,type,dscr);
	log_open=open_app_file(unit,pathname,type,dscr);
    else
	write(msg,'("W FLUSH_FORLOG w The unit sepcificed was not the LOG unit: ",i3)')unit
	write(STD,'(a)')trim(msg);
    endif

end subroutine flush_forlog

! ===========================================================================================================
end module communication
! ===========================================================================================================
