! N==========================================================================================================
! integer function extract_config.f90
! 
! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who    what
! 20180724 VH     Improve warnings for config value problems
!                 Finalize cyc<6 warning/lognote
! 20180418 VH     If CONFIG_incoming_fillval found, only issue warning if cyc<6, otherwide add lognote
! 20180417 VH     General WARNING revisions.
! 20170712 all    Release of qcplus v1.1 - many changes to all, some documented/tracked, some not
! 20160511 all    Initial release of qcplus v1.1.0
!
! ===========================================================================================================
! Notes
! Function to read config parameters for netcdf files while passing parameters from qc files
!
! uses: qc_config_params_lst, nc_config_params_lst, cnt_config_table_param    (from read_configtable.f90)
!       npqc, pqcline                                                         (from read_pqc.f90)
!==================================================================================================

integer function extract_config(icyc)

use communication                       ! module
use data_arrays                         ! module
use ncmeta_arrays                       ! module

implicit none

!==================================================================================================
! varialbe/array declarations

 character(len=14),parameter :: FUNCT_NAME = 'extract_config'
 integer :: iH, iT, iTS, i_look, i_found, icyc, i_mission , tmp_mission_num, i_config_param
 !integer :: get_number_for_direction
integer :: prepare_launch_config
character(len=SZC) :: str_pqc_tmp1;
 character(len=40) :: str_pqc_tmp2
 character(len=128) :: str_pqc_tmp3;
 character(len=256) :: tmp_mission_comment ="";
 logical :: config_param_name_exists = .false., new_mission_number_found = .false. , is_first_cycle = .false.
 double precision :: config_val_tmp
 integer :: conf_sampl_match_cnt;
 
! Notes from Claudia
! --------------------
! loop through header to find exact match of "CONFIG_MISSION_NUMBER".
! compare this new number to those found in previous qc files.
! If the mission number (n_mission) already exists there is nothing to do.
! If the mission number is new, then N_MISSIONS gets a plus 1 and the rest
! of this function applies.
! If exact match of "CONFIG_MISSION_NUMBER" is not found: use cycle number
! and issue a warning (we do want the meta.nc file, so a non-fatal error won't
! work). We have to think about this case some more and chat with Vicki. XXX
! Problem is: with Argos floats there will only be a launch config, so 
! CONFIG_MISSION_NUMBER=cycle number makes no sense. Vicki will work on
! algorithm for park & profile Argos floats ...
!
! i_match vector is needed to find out what the maximum number of parameters
! for a float is (the sum of it becomes the dimension N_CONFIG_PARAM).
!	char CONFIG_PARAMETER_NAME(N_CONFIG_PARAM, STRING128) ;
!	double CONFIG_PARAMETER_VALUE(N_MISSIONS, N_CONFIG_PARAM) ;
! For every new mission the function has to look for the config parameter name
! to those stored in configuration_parameter_name and assign the correct
! n_config_param (if found) or do n_config_param=n_config_param+1 (if not found).
! Then configuration_parameter_value(cnt_missions,n_config_param) can be assigned.
! configuration_parameter_cycles can be renamed to fill CONFIG_MISSION_NUMBER.

integer :: result = 0; ! number of table values read
character(len=16),parameter :: sub = 'extract_config  '
character(len=15),parameter :: typ = 'NCMETA EXTCNFG'

! ===========================================================================================================
! executable statements

if(DEBUG)then; write(msg,'("==== ",a," BEG ==== icyc: ",i3," ===============")')typ,icyc; call wrtdbg(sub,msg); endif
! To caclulate how many config_mission_numbers avaialable for this float.
if(trans_type == 'ARGOS')then ! CS/JN shifted whole if statement up
!-----------   Only if the float is of type "ARGOS " -----------
!NOTE: Extraction of CONFIG is different for ARGOS type float as they don't change the mission often
!      But they could have 2 alternating missions with two different nominal pressures. (Park & Profile)
!      (As per discussion with Claudia on 03/09/2016)
   cnt_missions = 1;
   configuration_mission_numbers(cnt_missions) = 1;
   if(DEBUG)then;
      write(msg,'("ARGOS float: cnt_missions and configuration_mission_numbers(cnt_missions) set to 1")') ! DBG
      call wrtdbg(sub,msg);
   endif
   n_config_param = n_launch_config_param
   do i_config_param=1, n_config_param
      configuration_parameter_name(i_config_param)  = launch_configuration_parameter_name(i_config_param)
      configuration_parameter_value(1,i_config_param) = launch_configuration_parameter_value(i_config_param) ! cnt_missions = 1 for mission # 1
   enddo

   if(nominal_prof_ncyc2 > 0)then ! Means it does have a second mission
      if (cnt_missions == 1) then ! CS/JN addition to handle floats without the CONFIG_MISSION_NUMBER lines in all cycles
         cnt_missions = 2;
         configuration_mission_numbers(cnt_missions) = 2;
      endif
      do i_config_param=1, n_config_param
         configuration_parameter_name(i_config_param)  = launch_configuration_parameter_name(i_config_param)
         if(trim(configuration_parameter_name(i_config_param)) == 'CONFIG_ProfilePressure_dbar')then
            configuration_parameter_value(2,i_config_param) = DBLE(nominal_prof_pres2)  ! cnt_missions = 2 for mission # 2
         else
            configuration_parameter_value(2,i_config_param) = launch_configuration_parameter_value(i_config_param) ! cnt_missions = 2 for mission # 2
         endif
      enddo
   endif
   extract_config=SUCCESS  ! Assuming all config variables extracted
   return   !  if "ARGOS" function will be done here
endif

 i_found=0;
 do iH=1,npqc
      str_pqc_tmp1=pqcline(iH); str_pqc_tmp2=str_pqc_tmp1(1:40);
      str_pqc_tmp3=str_pqc_tmp1(41:168);
      if(trim(str_pqc_tmp2(1:40)) == 'CONFIG_MISSION_NUMBER')then
         i_found=1;
         read(str_pqc_tmp3, *) tmp_mission_num
         do i_mission = 1,cnt_missions
           if(configuration_mission_numbers(i_mission) == tmp_mission_num) then
	        ! in case config mission number already exists, no need to continue
                extract_config=SUCCESS  ! Assuming all config variables extracted - JN/CS not sure if needed?
                return ! go back to calling function
	   endif
         enddo
!        in case the qc file has a new value as "CONFIG_MISSION_NUMBER" add it to config_mission_number array
         if(tmp_mission_num /= FILLINT )then
            cnt_missions = cnt_missions + 1
            configuration_mission_numbers(cnt_missions) = tmp_mission_num
            if(DEBUG)then;
		 write(msg,'("N_MISSION extract_config @1 increment+1 cnt_missions, cmn(nm):  ",i3," added number: ",i3)') & ! DBG
		    cnt_missions, configuration_mission_numbers(cnt_missions)						     ! DBG
		call wrtdbg(sub,msg);
	    endif
         endif
      endif
 enddo

! END OFFSET -  no longer offse by 1sp
if(i_found == 0)then
    if(trim(trans_type) == 'IRIDIUM')then !(only for Iridium??? )
	! in case the qc file has no field "CONFIG_MISSION_NUMBER" use cycle number as mission number 
	cnt_missions = cnt_missions + 1
	configuration_mission_numbers(cnt_missions) = icyc
        if(DEBUG)then;
            write(msg,'("N_MISSION extract_config @2 increment+1 cnt_missions, cmn(nm):  ",i3," added number: ",i3)') & ! DBG
                 cnt_missions, configuration_mission_numbers(cnt_missions)                                              ! DBG
            call wrtdbg(sub,msg);
        endif
    else
	!write(msg,*)icyc , " DBG:>>>>>>>>>> cnt_missions = ", cnt_missions, "comment ",tmp_mission_comment;  call wrtlog(sub,msg);
	if(cnt_missions > 0)then
	endif
    endif
    write(wrnmsg,'("WARNING 163 ct_config QC file is missing CONFIG_MISSION_NUMBER for cycle : ",i4.4)')icyc; ! WRN
    call warning(sub,163,wrnmsg); ! WARNING 163 in extract_config ! WRN
endif
! BEG OFFSET -  offse by 1sp 
! if(cnt_missions == 0) then
!    cnt_missions = 1 ! So that the N_MISSIONS variables will not be empty
!    configuration_mission_numbers(cnt_missions) = 1
!
!     if(DEBUG)then;
!         write(msg,'("N_MISSION extract_config @3 set cnt_missions=1, cmn(nm):  ",i3," added number: ",i3)') & ! DBG
!             cnt_missions, configuration_mission_numbers(cnt_missions)                                                ! DBG
!         call wrtdbg(sub,msg);
!     endif
! endif
!
!write (msg,*)"  DBG:>>>> n_mission count (Extr Config)  ", cnt_missions; call wrtlog(sub,msg); 
if(cnt_missions > 0 )then ! not reading configs if there is no mission defined yet !.and. tmp_mission_num /= FILLINT 

! To scan through the pqc data and to fill up the cofig_param_name and config_param_value arrays 
! for the nectdf files for the variables which are found matching in the CONFIG_TABLE
 conf_sampl_match_cnt=0;
 do iH=1,npqc
   i_look=1;  config_param_name_exists = .false.
   if (INDEX(trim(pqcline(iH)), '==================================================================================')>0) then
      exit  ! If it's the starting of position block, no need to extract anymore
   endif
   do iTS=1,cnt_config_table_skip_param
      str_pqc_tmp1=pqcline(iH); str_pqc_tmp2=str_pqc_tmp1(1:40);
!      write(*,'(a,a,a,a)') trim(str_pqc_tmp2),'--',trim(qc_config_skip_lst(iTS)),'--'
      call to_upper(str_pqc_tmp2);  ! to match the case with the skip table entries
      call to_upper(qc_config_skip_lst(iTS));  ! Just in case both the names are in different case
      if (trim(str_pqc_tmp2(1:40)).eq.'NUMBER OF COLUMNS') then
         i_look=0;
         exit;						! exit the loop, not the program
      elseif (INDEX(trim(str_pqc_tmp2),'. COLUMN')> 0) then
         i_look=0;
         exit;
      elseif (str_pqc_tmp2(1:5).eq.'CALIB') then
         i_look=0;
         exit;                                          ! exit the loop, not the program
      elseif (str_pqc_tmp2(1:20).eq.'====================') then
         i_look=0;
         exit;						! exit the loop, not the program
      elseif (str_pqc_tmp2(1:2).eq.'MC') then
         i_look=0;
         exit   
      elseif (trim(str_pqc_tmp2).eq.trim(qc_config_skip_lst(iTS))) then
         i_look=0;
         exit;						! exit the loop, not the program
      endif
   enddo	
   !write (msg, *) "DBG:>>> n_config_param  :", n_config_param; call wrtlog(sub,msg); 
    
   if(i_look>0) then
      str_pqc_tmp1=pqcline(iH); str_pqc_tmp2=str_pqc_tmp1(1:40);
      str_pqc_tmp3=str_pqc_tmp1(41:168); i_found = 0
      do iT=1,cnt_config_table_param
         call to_upper(str_pqc_tmp2);
         if (trim(str_pqc_tmp2).eq.trim(qc_config_params_lst(iT))) then  
           ! Found a parameter matching th e left coulmn from config table
            i_found = 1
	    ! Note from Claudia: This parameter is not required for  Iridium floats
            if(trim(nc_config_params_lst(iT)) == 'CONFIG_TransmissionRepetitionPeriod_seconds')exit
	  
	    ! Check for the "multiple value config line here and extract the values "
	    ! set a counter for counting the occurence of the line match in the file against table.
	    if(trim(str_pqc_tmp2) == 'CONFIG SAMPLING SCHEME (DBAR)')then
	        conf_sampl_match_cnt=conf_sampl_match_cnt + 1;
	        call extract_conf_sampling_scheme(icyc,conf_sampl_match_cnt,str_pqc_tmp3);
	    endif
            do i_config_param = 1, n_config_param  
               if(configuration_parameter_name(i_config_param) == nc_config_params_lst(iT))then
		   if(DEBUG .AND. DBLEV==72)then
		       write (msg, *) " str_pqc_tmp3 = "//str_pqc_tmp3;  call wrtdbg(sub,msg);
	           endif 
                   ! Note: Some files have config fill value as "-999.0000"
	           if(is_numeric(trim(str_pqc_tmp3)) .and. trim(str_pqc_tmp3) /= trim(CONFIG_incoming_fillval))then 
                      read(str_pqc_tmp3, *) config_val_tmp
		      call convert_config_param (qc_config_params_lst(iT), nc_config_params_lst(iT), &
					    config_val_tmp)
		      if(config_val_tmp /= FILLNC) configuration_parameter_value(cnt_missions,i_config_param) = config_val_tmp 
		      if(DEBUG .AND. DBLEV==72)then
		         write (msg, *) configuration_parameter_name(i_config_param), " =  ", config_val_tmp
		         call wrtdbg(sub,msg);
		      endif
	           else 
                      if(trim(str_pqc_tmp3) == trim(CONFIG_incoming_fillval))then
			  write(wrnmsg,'("WARNING 161 icyc ",i5," Config param is FILL VALUE! @1 ",a)')icyc,trim(str_pqc_tmp2); ! WRN
			  if(thiscyc<6)then;
		              call warning(sub,161,wrnmsg); ! WARNING 161 WRN<6 cnfg param fill @1 ! WRN
			  else
			      call lognote(sub,161,wrnmsg); ! WARNING 161 LGN<6 cnfg param fill @1
			   endif
	                  !configuration_parameter_value(cnt_missions,i_config_param) = DBLE(99999)
                      else
                         write(errmsg(1),'("NON-FATAL ERROR 270 in extract_config")')
			 write(errmsg(2),'("NON-FATAL ERROR 270 in ",a," icyc: ",i4," Config param value NOT a number @1")')sub,icyc
                         write(errmsg(3),'("NON-FATAL ERROR 270 test line: ",a)')trim(str_pqc_tmp3);
                         write(errmsg(4),'("NON-FATAL ERROR 270  pqc line: ",a)')trim(pqcline(iH));
                         call error_report(sub,270,4,errmsg); ! extract_config param value NOT a number @1
                      endif

	           endif
	           exit
	       endif 
            enddo
	    if(i_config_param > n_config_param) then ! That means loop ran completely and didn't get into exit statement.
	       n_config_param=n_config_param+1;
	       if(DEBUG .AND. DBLEV==72)then
		    write (msg,*)"  DBG:>>>> icyc: ", icyc; call wrtdbg(sub,msg);
		    write (msg, *) "DBG:>>> config _param name found:", nc_config_params_lst(iT); 
		    call wrtdbg(sub,msg);
		    write (msg,*)"  DBG:>>>> config _param val : ", str_pqc_tmp3; call wrtdbg(sub,msg);
	       endif
               configuration_parameter_name(n_config_param)=nc_config_params_lst(iT) ! add the param_name into the array.
	       !write (msg, *) "DBG:>>> n_config_param :",n_config_param;  call wrtlog(sub,msg) 
               
	       if(DEBUG .AND. DBLEV==72 )then
                    write (msg, *) " Found new config param - "//nc_config_params_lst(iT); call wrtdbg(sub,msg) 
	            write (msg, *) " n_config_param  =  ", n_config_param ; call wrtdbg(sub,msg)
	       endif
	       ! Note: Some files have config fill value as "-999.0000"
	       if(is_numeric(trim(str_pqc_tmp3)).and. trim(str_pqc_tmp3) /= trim(CONFIG_incoming_fillval))then
                  read(str_pqc_tmp3, *) config_val_tmp
		  call convert_config_param (qc_config_params_lst(iT), nc_config_params_lst(iT), config_val_tmp)
		  !write (msg, *) "DBG:>>> cnt_missions :",cnt_missions;  call wrtlog(sub,msg) 
		  if(config_val_tmp /= FILLNC) configuration_parameter_value(cnt_missions,n_config_param) = config_val_tmp
               else 
                  if(trim(str_pqc_tmp3) == trim(CONFIG_incoming_fillval))then
                     write(wrnmsg,'("WARNING 162 icyc ",i5," Config param is FILL VALUE! @2",a)')icyc,trim(str_pqc_tmp2); ! WRN
		     if(thiscyc<6)then;
		         call warning(sub,162,wrnmsg); ! WARNING 162 WRN<6 cnfg param fill @2 ! WRN
		     else;
		         call lognote(sub,162,wrnmsg); ! WARNING 162 LGN<6 cnfg param fill @2
		     endif
		      !configuration_parameter_value(cnt_missions,n_config_param) = DBLE(99999)
                  else
                      write(errmsg(1),'("NON-FATAL ERROR 271 in extract_config")')
		      write(errmsg(2),'("NON-FATAL ERROR 271 in ",a," icyc: ",i4," Config param value NOT number! @2")')sub,icyc
                      write(errmsg(3),'("NON-FATAL ERROR 271 test line: ",a)')trim(str_pqc_tmp3)
                      write(errmsg(4),'("NON-FATAL ERROR 271  pqc line: ",a)')trim(pqcline(iH))
                      call error_report(sub,271,4,errmsg); ! extract_config param Config param value is NOT a number@1
                  endif
	       endif
	    endif
         endif
      enddo
      if(i_found==0) then
         write(msg,'(a20,": ",a)')qcfile,trim(str_pqc_tmp2); write(MSC,'(a)')trim(msg);				! NCMETA MISS_CNFG
         if(DEBUG) write(DBG,'("DBG NCMETA MISS_CNFG: ",a)')trim(msg);						! DBG
      endif
   endif
 enddo
endif
extract_config=SUCCESS  ! Assuming all config variables extracted

999 continue ! This is THE_END

if(DEBUG)then; write(msg,'("==== EXTNC META <<<<< Leave ",a," ret= ",i3)')sub,extract_config;call wrtdbg(sub,msg); endif
if(DEBUG)then; write(msg,'("==== EXTNC META = END ====================")');call wrtdbg(sub,msg); endif
if(DEBUG)then; write(msg,'("==== ",a," END ====================")')typ;call wrtdbg(sub,msg); endif

return;
end function extract_config
! ===========================================================================================================
