! ===========================================================================================================
! module nctech_arrays.f90
!
! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who    what
! 20180920 CA     Change intval(1) = 99999; intval = 99999; for vargo
! 20170712 all    Release of qcplus v1.1 - many changes to all, some documented/tracked, some not
! 20160511 all    Initial release of qcplus v1.1.0
!
! ===========================================================================================================
! Notes:

! This module contains variable/array definitions used by write_nctech to create the nc technical file.
! ===========================================================================================================

module nctech_arrays

use communication                                                ! module
use data_arrays                                                  ! module
use netcdf_arrays                                                ! module

implicit none

! ===========================================================================================================
! moedule varialbe/array declarations
! ===========================================================================================================

integer,parameter :: MAXTECH = 25000        ! max nbr of tech records
character(len=128), dimension(MAXTECH) :: technical_parameter_value
character(len=128), dimension(MAXTECH) :: technical_parameter_name
integer :: technical_parameter_cycles(MAXTECH)
integer :: cnt_tech_table_param,cnt_tech_table_skip_param

character(len=40), dimension(MAX_PARM_TABLE) ::  qc_tech_params_lst  !Tech parameters
character(len=40), dimension(MAX_PARM_TABLE) ::  qc_tech_skip_lst    !non-Tech parameters
character(len=128), dimension(MAX_PARM_TABLE) :: nc_tech_params_lst  !new NC Tech parameters

integer :: TECHNICAL_PARAMETER_NAME_id,  TECHNICAL_PARAMETER_NAME_dims(ND2)
integer :: TECHNICAL_PARAMETER_VALUE_id, TECHNICAL_PARAMETER_VALUE_dims(ND2)
integer :: CYCLE_NUMBER_id,              CYCLE_NUMBER_dims(ND1)

integer :: name_start(ND2),  name_count(ND2)
integer :: value_start(ND2), value_count(ND2)
integer :: cycle_start(ND1), cycle_count(ND1)

character(len=1), dimension(1) :: charval
integer :: intval !  ... used to have intval(1), gfrotran on vargo does not like this very much ...

! ===========================================================================================================

contains

! ===========================================================================================================
! module functions/subroutines
! ===========================================================================================================

subroutine def_nctech_vars(caller,nctype,ncid)

	character(*), intent( in ) :: caller        ! who is calling me?
	character(*), intent( in ) :: nctype        ! prof/traj/tech
	integer,      intent( in ) :: ncid          ! file id


	integer :: iret
        integer :: ret = 0;
	integer :: loc = 1;
	character(len=15), parameter :: sub = 'def_nctech_vars'


	write(msg,*)'NCTECH def  TECHNICAL_PARAMETER_NAME for tech'; call wrtlog(sub,msg);
	TECHNICAL_PARAMETER_NAME_dims(2) = N_TECH_PARAM_dimid
	TECHNICAL_PARAMETER_NAME_dims(1) = STRING128_dimid
	 ret=nf_def_var(ncid, 'TECHNICAL_PARAMETER_NAME', NF_CHAR, ND2, TECHNICAL_PARAMETER_NAME_dims, TECHNICAL_PARAMETER_NAME_id)
	loc = 100; call cke(sub, ret, loc);

	 ret=nf_put_att_text(ncid, TECHNICAL_PARAMETER_NAME_id, 'long_name', 27, 'Name of technical parameter')
	loc = 101; call cke(sub, ret, loc);
	charval(1)=''
	 ret=nf_put_att_text(ncid, TECHNICAL_PARAMETER_NAME_id, '_FillValue', 1,charval);
	loc = 102; call cke(sub, ret, loc);

	write(msg,*)'NCTECH def  TECHNICAL_PARAMETER_VALUE for tech'; call wrtlog(sub,msg);
	TECHNICAL_PARAMETER_VALUE_dims(2) = N_TECH_PARAM_dimid
	TECHNICAL_PARAMETER_VALUE_dims(1) = STRING128_dimid
	 ret=nf_def_var(ncid, 'TECHNICAL_PARAMETER_VALUE', NF_CHAR, ND2, TECHNICAL_PARAMETER_VALUE_dims, TECHNICAL_PARAMETER_VALUE_id)
	loc = 103; call cke(sub, ret, loc);
	
	 ret=nf_put_att_text(ncid, TECHNICAL_PARAMETER_VALUE_id, 'long_name', 28, 'Value of technical parameter')
	loc = 104; call cke(sub, ret, loc);
	charval(1)=''
	 ret=nf_put_att_text(ncid, TECHNICAL_PARAMETER_VALUE_id, '_FillValue', 1,charval);
	loc = 105; call cke(sub, ret, loc);

	write(msg,*)'NCTECH def  CYCLE_NUMBER for tech'; call wrtlog(sub,msg);
	CYCLE_NUMBER_dims(1) = N_TECH_PARAM_dimid
	 ret=nf_def_var(ncid, 'CYCLE_NUMBER', NF_INT, ND1, CYCLE_NUMBER_dims, CYCLE_NUMBER_id)
	loc = 106; call cke(sub, ret, loc);
	cycnum_conventions = tech_cycnum_conventions; nlt = len(trim(cycnum_conventions));
	 ret=nf_put_att_text(ncid, CYCLE_NUMBER_id, 'conventions', nlt, cycnum_conventions);
	loc = 107; call cke(sub, ret, loc);
	 ret=nf_put_att_text(ncid, CYCLE_NUMBER_id, 'long_name', 18, 'Float cycle number')
	loc = 108; call cke(sub, ret, loc);
	intval = 99999;        ! CYCLE_NUMBER FILL value ... used to have intval(1)
	 ret=nf_put_att_int(ncid, CYCLE_NUMBER_id, '_FillValue', NF_INT, 1, intval)
	loc = 109; call cke(sub, ret, loc);

end subroutine def_nctech_vars
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

subroutine put_nctech_data(caller,nctype,ncid)

    character(*), intent( in ) :: caller        ! who is calling me?
    character(*), intent( in ) :: nctype        ! prof/traj/tech
    integer,      intent( in ) :: ncid          ! file id
    character(len=15), parameter :: sub = 'put_nctech_data'
    integer :: iret
    integer :: ret = 0;
    integer :: loc = 1;

! for testing
!    character(len=128), dimension(10) :: nametest
!    character(len=128), dimension(10) :: valuetest
!    integer, dimension(10) :: cycletest

    write(msg,*)'NCTECH put TECHNICAL_PARAMETER_NAME for tech'; call wrtlog(sub,msg);
      name_start(1) = 1
	name_start(2) = 1
	name_count(1) = STRING128_len
	name_count(2) = n_tech_param
        ! NOTE NF_UNLIMITED This is the call that increments the UNLIMITED dimension for tech
	 ret=nf_put_vara_text(ncid, TECHNICAL_PARAMETER_NAME_id, name_start, name_count, technical_parameter_name)
	if(DBLEV==64 .and. DBLEVEXIT)then;
	    call close_ncfile(sub,ncid);                                   ! LEV64
	endif
      if(DEBUG)then;  write(msg,*) 'ret - name: ',ret; call wrtdbg(sub,msg); endif

    write(msg,*)'NCTECH put TECHNICAL_PARAMETER_VALUE for tech'; call wrtlog(sub,msg);
	value_start(1) = 1
	value_start(2) = 1
	value_count(1) = STRING128_len
	value_count(2) = n_tech_param
	 ret=nf_put_vara_text(ncid, TECHNICAL_PARAMETER_VALUE_id, value_start, value_count, technical_parameter_value)
      if(DEBUG)then;  write(msg,*) 'ret - value: ',ret; call wrtdbg(sub,msg); endif

    write(msg,*)'NCTECH put CYCLE_NUMBER for tech'; call wrtlog(sub,msg);
	cycle_start(1) = 1
	cycle_count(1) = n_tech_param
	 ret=nf_put_vara_int(ncid, CYCLE_NUMBER_id, cycle_start, cycle_count, technical_parameter_cycles)
      if(DEBUG)then;
	write(msg,*) 'ret - cycle: ',ret; call wrtdbg(sub,msg);							! DBG
	write(msg,*) 'name: ',name_start(1),name_start(2),name_count(1),name_count(2); call wrtdbg(sub,msg);		! DBG
	write(msg,*) 'value: ',value_start(1),value_start(2),value_count(1),value_count(2); call wrtdbg(sub,msg);	! DBG
	write(msg,*) 'cycle: ',cycle_start(1),cycle_count(1); call wrtdbg(sub,msg);					! DBG
    endif

end subroutine put_nctech_data

! ===========================================================================================================
end module nctech_arrays
! ===========================================================================================================
