! ===========================================================================================================
! integer function qc03_impossible_location.f90
!
! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who    what
! 20170712 all    Release of qcplus v1.1 - many changes to all, some documented/tracked, some not
! 20160511 all    Initial release of qcplus v1.1.0
!
! ===========================================================================================================
! Notes
!
! Note that this evaluates ALL positions - the actual profile LAT/LON is NOT selected here.,
! The return value will there reflect the success/failure of all operations
!
! Function return values:
!  0 = PASS - Success, all positions passed
!  1 = FAIL - Failure, some or all positions failed
! -1 = NOTD - cannot be determined
!

integer function qc03_impossible_location(caller)

use communication                       ! module
use data_arrays                         ! module

implicit none

! ===========================================================================================================
! varialbe/array declarations

character(*), intent( in ) :: caller

character(len=14), parameter :: sub = 'QC03'
character(len=21) ,parameter :: QCTEST='Impossible Location'
integer :: result;

integer :: nbad, ngud;
logical :: impossible
integer :: num_pos_lines;
integer :: mxtflag; ! Maximum local test flag
! ===========================================================================================================
! executable statements

result = NOTD ! cannot be determined

if(DBGIO) call dbg_enter(sub)
if(caller == 'read_pqc')then;  num_pos_lines = ppos; else; num_pos_lines = npos;endif
if(num_pos_lines==0) go to 33;	! No positions, skip to the end
nbad = 0; ngud = 0;
do i=1,num_pos_lines
    if(this_lat(i) > 98.0 .or. this_lon(i) > 998.0)then
	pos_posiqc(i) = '9'; ! This is a 999 line
	if(DEBUG)then; write(msg,1001)i,this_lat(i),this_lon(i); call wrtdbg(sub,msg); endif ! LEV03
	1001 format("LEV03 POS pos: ",i2," lat: ",f9.3,"lon: ",f10.3," set qc to 9");
    else
	impossible = impossible_location(this_lat(i),this_lon(i))
	if(impossible)then
	    pos_posiqc(i) = '4'; nbad=nbad+1; ! This is an inpossible location
	    if(DEBUG)then; write(msg,1002)i,this_lat(i),this_lon(i); call wrtdbg(sub,msg); endif ! LEV03
	    1002 format("LEV03 POS pos: ",i2," lat: ",f9.3,"lon: ",f10.3," set qc to 4");
	else
	    pos_posiqc(i) = '1'; ngud=ngud+1; ! This is a good position
	    if(DEBUG)then; write(msg,1003)i,this_lat(i),this_lon(i); call wrtdbg(sub,msg); endif ! LEV03
	    1003 format("LEV03 POS pos: ",i2," lat: ",f9.3,"lon: ",f10.3," set qc to 1");
	endif
    endif
    if(DEBUG)then; write(msg,1000)i,this_lat(i),this_lon(i); call wrtdbg(sub,msg); endif ! LEV03
    1000 format("LEV03 POS pos: ",i2," lat: ",f9.3,"lon: ",f10.3," qc: ",a1);
enddo

! after the do loop, report the results
if(nbad > 0)then;
    result = FAIL;
elseif(ngud > 0)then;
    result = PASS;
else
    result = NOTD
endif

33 continue;		! No positions, skip to the end

mxtflag = 0; if(result == PASS)then; mxtflag = 1; elseif(result == FAIL)then; mxtflag = 4; endif
if(DEBUG) write(DBG,'("QCTEST_FLAG QC03 result: ",i2," mxtflag: ",i2," nprf: ",i2)')result,mxtflag,nprf
do i=1,nprf; QCTEST_FLAG(i,3) = mxtflag; enddo

if(DBGIO) call dbg_leave(sub,result);

! For the individual/full-profile tests, report the result directly to all files

write(msg,'("Impossible Location Test    ",a)')report(result); call wrttrc(sub,msg);

! The result to return to the main is based on the first data field - pres/temp/psal/doxy for primary profile

qc03_impossible_location = result;
if(result == PASS)then; result = 1; elseif(result == FAIL)then; result = 4; else; result = 0; endif

return
end function qc03_impossible_location
! ==========================================================================================================
