! ===========================================================================================================
! integer function qc05_impossible_speed.f90
!
! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who    what
! 20170712 all    Release of qcplus v1.1 - many changes to all, some documented/tracked, some not
! 20160511 all    Initial release of qcplus v1.1.0
!
! ===========================================================================================================
! Included:
!
! subroutine speed_check_jamstec(caller); - Provided for future reference, but currently not functinal
! subroutine speed_checkk(ispd_flag,bad_row) - The speed check for the profile (ie, not traj)
! double precision function distance( lat1, long1, lat2, long2 ) - Distance between two lat/lon pairs
!
! ===========================================================================================================
! Notes
!
! Function return values:
!  0 = PASS - Success, all positions passed
!  1 = FAIL - Failure, some or all positions failed
! -1 = NOTD - cannot be determined
!

integer function qc05_impossible_speed(caller)

use communication                       ! module
use data_arrays                         ! module

implicit none

! ===========================================================================================================
! varialbe/array declarations

character(*), intent( in ) :: caller

character(len=14), parameter :: sub = 'QC05'
character(len=21) ,parameter :: QCTEST='Impossible Speed'
integer :: result;

integer :: ispd_flag;
integer :: bad_row(MAXPOS),jj  ! bad_row(i)=4 when pos i fails speed check
integer :: indx			! used to transfer pos2 flags to regular pos array
integer :: num_pos_lines2;
character(len=14) :: source_type
integer :: mxtflag; ! Maximum local test flag
! ===========================================================================================================
! executable statements
if(caller == 'read_pqc')then; num_pos_lines2 = ppos2;source_type='read_pqc';
else; num_pos_lines2 = npos2; source_type='qcplus'; endif

result = NOTD ! cannot be determined

ispd_flag=0;jj=0;
if(DBGIO) call dbg_enter(sub)
if(num_pos_lines2 .eq. 0  )then
    result = NOTD
    write(msg,'("SPEED CHECK There are NO positions present. set result to NOTD, cannot be determined")'); call wrttrc(sub,msg);
    if(DBLEV== 5)then; write(msg,*)'LEV05 QC05 at 2 set result to NOTD: ',NOTD; call wrtdbg(sub,msg); endif
    go to 999; ! Skip to THE_END
endif

call speed_checkk(ispd_flag,bad_row,source_type)
if(index_999_1 .eq. 1 .or. pos_posiqc(1)=='9')then
    pos_speedc(1)='0'
    jj=1
endif

do i = 1,num_pos_lines2
    indx = i + jj;		! jj = offset if first pos line is 999 line
    if(bad_row(i) .eq. 4)then
        pos_speedc(i+jj)='4'
    elseif(ispd_flag == NOTD)then
	pos_speedc(i+jj)='0'		! Canot be determined
    else
        pos_speedc(i+jj)='1'
    endif
enddo

if(trim(source_type) == 'qcplus' .and. index_999_2 .eq. npos .and. index_999_2 .gt. 0)then
    pos_speedc(npos)='0'
elseif(trim(source_type) == 'read_pqc' .and. index_999_2 .eq. ppos .and. index_999_2 .gt. 0)then
    pos_speedc(ppos)='0'
endif

if(DBLEV== 5)then; write(msg,*)'LEV05 before final evaluation, ispd_flag: ',ispd_flag;; call wrtdbg(sub,msg); endif
if(ispd_flag .eq. 4)then
   if(DBLEV== 5)then; write(msg,*)'LEV05 QC05 at 3 set result to FAIL: ',FAIL; call wrtdbg(sub,msg); endif
   result= FAIL;	! -1
elseif(ispd_flag == NOTD)then
    result= NOTD;
else
   if(DBLEV== 5)then; write(msg,*)'LEV05 QC05 at 4 set result to PASS: ',PASS; call wrtdbg(sub,msg); endif
   result= PASS;	! 1
endif 

999 continue ! This is THE_END
       
mxtflag = 0; if(result == PASS)then; mxtflag = 1; elseif(result == FAIL)then; mxtflag = 4; endif
if(DEBUG) write(DBG,'("QCTEST_FLAG QC05 result: ",i2," mxtflag: ",i2," nprf: ",i2)')result,mxtflag,nprf
do i=1,nprf; QCTEST_FLAG(i,5) = mxtflag; enddo

if(DBGIO) call dbg_leave(sub,result);

! For the individual/full-profile tests, report the result directly to all files

write(msg,'("Impossible Speed Test       ",a)')report(result); call wrttrc(sub,msg);

! The result to return to the main is based on the first data field - pres/temp/psal/doxy for primary profile

qc05_impossible_speed = result;
if(result == PASS)then; result = 1; elseif(result == FAIL)then; result = 4; else; result = 0; endif

return
end function qc05_impossible_speed	! primary

! ===========================================================================================================
!
! The JAMSTEC speed check
!
subroutine speed_check_jamstec(caller);

use communication                       ! module
use data_arrays                         ! module

implicit none

! ===========================================================================================================
! varialbe/array declarations

character(*), intent( in ) :: caller

character(len=8), parameter :: sub = 'QC05_jamstec'
character(len=29) ,parameter :: QCTEST='Impossible Speed JAMSTEC'

write(msg,*)'speed_check_jamstec is not yet functional'; call wrttrc(sub,msg);

return
end subroutine speed_check_jamstec

! ==========================================================================================================
! subroutine speed_checkk(ispd_flag,bad_row)
! Notes:
!   Revision date : 06-May013 VMH   - made number of speed check results (spd_flag array) into
!                                     a parameter, npp, and check if it goes out of bounds
!                   17-Oct-11 (ER   - Fixed this bug: if a position fails the previous profile
!                                     test, the rest were getting a '4' even if they were good
!                                     positions
!                   14-Oct-11 (ER   - Fixed incorrect assignment of bad_previous_row vector
!                   07-Sep-11 (ER   - Fixed this bug: when there is only 1 position and it
!                                     fails the previous profile test it wasn't adding a '-'
!                                     before the POSITION-QC column in qc file.
!                   24-Sep-09 (ER   - ihr=hour(jst)*100+mn(1) became ihr=hour(jst)*100+mn(k) )
!                                   - stop doing the no-positions-in-profile  
!                                     test since it is already done in qc_t
!                                   - algorith to determine bad positions changed
!                                   - new subroutine flag_worse_positions
!                                   - new subroutine pick_bad_pos
!                                   - new subroutine print_flag_table
!                                   - new subroutine print_nbr_bad_flags_per_pos
!                                   - new subroutine print_bad_positions
!                                   - new function distance: returns distance between 
!                                     2 coordinates.  It allowed the standardization of
!                                     the calculus of distance. (Method to calculate distance 
!                                     was not uniform: pos i v. pos j and pos i v.
!                                     previous profile used the same method, while pos i and 
!                                     pos j v. mean position used a different one.)
!                                   - adding many comments for readability.
!                                   - changes in layout of statements to make
!                                     them easier to read.)
!                    09-Sep-09 (YHD - a bug fixed on line 469)
!                    03-Jun-09 (YHD - Loosen the restriction on speed check.)
!                                     It requires at least one 4 to consider
!                                     a position bad.)
!                    10-Jun-08 (YHD - Decide which location is bad if there is
!                                     only one '4' left in the flag_table.
!                                     Refer to June 10, 2008 note.)
!                    24-Oct-07 (YHD - 3 loops on flag_table; any 4's left in
!                                     flag_table gets a status=7).
!                    04-Oct_07 (YHD - Go thru loops 2 times for best results)
!                    02-Oct-07 (YHD - Count 4's in hori/vert in flag_table)
!                    26-Sep-07 (YHD - Check a horizontal sum of flag table).
!                    17-Sep-07 (YHD - More correction on spd table)
!                    06-Sep-07 (YHD - 'mxs'=100 introduced for maximum
!                                     speed table array size and print out speed
!                                     table in trace output.)
!                    08-May-07 (YHD - A brand new approach for the speed check
!                                     a) cycle 0 or 1 case
!                                     b) cycle N against cycle N-1
!                                     c) within cycle N
!                    23-Nov-05 (YHD - Make sure that bounce float does not
!                                     check the distance only case.)
!                    22-Nov-05 (YHD - 'longdiff' is wrong if a position is
!                                     skipped case.
!                    23-Mar-05 (YHD - print out the which part and location
!                                     of positions when the speed check fails;
!                                     write this to *.trace file')
!                    01-Feb-05 (YHD - The criteria for speed check is still
!                                     tight (i.e. checks internal and external 
!                                     for 3 m/s) to catch any bad position or
!                                     time errors.  Internal speed check fails
!                                     always the first position so far.)
!                    27-Jan-05 (YHD - Make time2-time1 to be more than 30 min)
!                    25-Jan-05 (YHD - Use time1 to be i; time2 to be i+1 for
!                                     within position check and make sure that
!                                     time2-time1 be gt 10 min.
!                                     time1 used be the time for the first 
!                                     position.)
!                    21-Jan-05 (YHD - For '000' profile, use second time in 
!                                     the position and make sure its time is
!                                     larger than the lauch time).
!                    12-May-04 (YHD - More change on the restriction;
!                                     See 05/11/04 note)
!                    11-May-04 (YHD - Loosen the restriction on speed check
!                                     between the meta and the first profile)
!                    15-Mar-04 (YHD - For profile '000' case, use launch time
!                                     position as previous profile.)
!                    02-Oct-03 (YHD - All dimensions check)
!                    20-May-03 (YHD - At least 10 min. difference needed
!                                     in surface positions.)
!                    14-May-03 (YHD - if ispd_flag=0 make ispd_flag=1)
!                    25-Jun-02 (YHD - For '000' or '001' profiles, apply
!                                     within speed checks only)
!                    10-Aug-01 (YHD - dateline crossing case)
!                    19-Jun-01 (YHD - speed between the profiles)
!                    01-Jun-01 (YHD - impossible speed test added)
!    Speed_check.f
!    =============
!
!    Looks for outliers in the ARGOS position for one surface cycle.
!    yhd, 07-21-2000
!

! ===========================================================================================================

subroutine speed_checkk(ispd_flag,bad_row,source_type)
 
use communication                       ! module
use data_arrays                         ! module

implicit none

! ===========================================================================================================
! the interface replaces the "intent" declarations

interface
  subroutine pick_bad_pos(np,flag_table,bad_row,xlon,ylat,pos1,pos2)
    integer  :: np, pos1, pos2
    real, dimension(*) :: xlon, ylat 
    integer, dimension(*) :: bad_row
    integer, dimension(:,:) :: flag_table
  end subroutine pick_bad_pos 
end interface

! ===========================================================================================================

character(len=12), parameter :: sub = 'speed_checkk'

double precision :: distance ! declare function type

! Constants
real, parameter :: max_speed = 3.0  ! m/s
real, parameter :: max_dist = 10000  ! maximum distance in meters
real, parameter :: max_dista = 50000 ! maximum distance in meters 
integer, parameter :: mxs = 100 ! max. nbr of position in profile
real, parameter :: pi = 3.14159265358979  
real, parameter :: daysec = 24.*60.*60. ! seconds in a day
real, parameter :: factor = 1.852*1.e3*60.

integer status
integer, parameter :: nnp = 3000; ! nnp=max number of positions

integer np,year(MAXPOS),month(MAXPOS),day(MAXPOS),hour(MAXPOS),mn(MAXPOS),sec(MAXPOS)
integer :: ntime,ispd_flag,ihr,pnum,ip,spd_flag(nnp),flag_table(mxs,mxs)

integer :: iyr_prev,imo_prev,ida_prev,ihr_prev,imin_prev,isec_prev
integer :: ibad_cnt,ibad_wthin,ibad_prv,ist,jst,k

integer :: bad_row(MAXPOS)  ! bad_row(i)=4 when pos i fails speed check
integer :: bad_previous_row(mxs) !bad_previous_row(i)=4 when 
                                 !pos(i) fails speed check against
                                 !previous profile or launch   
integer poss_bad_cnt(mxs),hori_row_cnt(mxs),vert_col_cnt(mxs),nloop
integer max_bad_cnt 
data    ibad_cnt,ibad_wthin,ibad_prv /3*0/

real    xlon(MAXPOS),ylat(MAXPOS),speed(nnp),dx,dy,u,v,longdiff,longdiff_pro,sumh,sumv
double precision :: d_dist  ! hold distancia between two pairs of coordinates
real    time1,time2
real    xlon_prev,ylat_prev,speed_prev,xlon1,xlon2,ylat1,ylat2
double precision t1,t2,t_sec,dt
character itime_prev*14,cnp*14,cnp1*14
!character profile_id*(*)
character(len=50) :: title
character(len=10) :: str_i
character(*), intent(in) :: source_type

data    nloop /0/

!np=npos
if(source_type == 'read_pqc')then;np=ppos2; else; np=npos2;endif
year = this_year2;	! year = this_year;
month=this_mon2;	! month=this_mon;
day=this_day2;		! day=this_day;
hour=this_hh2;		! hour=this_hh;
mn=this_mm2;		! mn=this_mm;
sec=this_sc2;		! sec=this_sc;
xlon=this_lon2;		! xlon=this_lon
ylat=this_lat2;		! ylat=this_lat

if(prev_good_lon /= FILL .or. ppos2_good /= 0)then	! We have a good previous good posn
    pnum = 1;	! Signal that we have a good previous position
    xlon_prev=prev_good_lon   
    ylat_prev=prev_good_lat   
    write(msg,'("QC05 prev_good_posn set to a real position: ",2f12.5)')ylat_prev,xlon_prev; call wrttrc(sub,msg);
else
    if(launch_lon > 200.0 .or. launch_lat > 100.0)then
	write(msg,'("QC05 prev_good_posn - tried to use launch_lon/lat, but they are FILL, return NOTD, cannot be determined")');
	write(msg,'("QC05 prev_good_posn set IS FILL VALUE (prev & laun), return NOTD")')
	call wrttrc(sub,msg); ispd_flag = NOTD; return;
    endif
    xlon_prev=launch_lon
    ylat_prev=launch_lat
    write(msg,'("QC05 prev_good_posn set to launch position: ",2f11.5)')ylat_prev,xlon_prev; call wrttrc(sub,msg);
endif

if (DEBUG)then;
    write(msg,*)"<<<<< DEBUG from SPEED_CHECK >>>>>,np,ylat_prev,xlon_prev = ",np,ylat_prev,xlon_prev
    call wrtlog(sub,msg);
endif
if (np.gt.mxs)then
  write(errmsg(1),'("FATAL ERROR 704 QC05 during speed check err at 1")');
  write(errmsg(2),'("FATAL ERROR 704 QC05 positions np > mxs case: ",2i5)')np,mxs
  call error_exit(sub,704,2,errmsg); ! qc05 - number of posn test exceeds max #1
endif

! initializing variables that will hold results of tests 
do k=1,np
  bad_row(k)=0 
  bad_previous_row(k)=0 
  do i=1,np
    flag_table(k,i)=0
  enddo
enddo

!======================================================================= 
!  part a) profile number is 0 or 1 case:
!          All positions/times are compared against the launch postion/time.
!  part b) profile number > 1 and compared against the previous cycle
!          postion/time.
!
if (pnum.lt.1)then
  if(DBLEV== 5)then; write(msg,*)'Launch date/position:'; call wrtlog(sub,msg); endif
else
  if(DBLEV== 5)then; write(msg,*)'previous profile date/position:'; call wrtlog(sub,msg); endif
endif

if(DBLEV== 5)then; write(msg,*)'Total number of positions:',np; call wrtlog(sub,msg); endif

!if (itime_prev(1:14).eq.'              ')then
!  itime_prev(1:14)='99999999999999'
!else
!  read(itime_prev(1:4),'(i4)') iyr_prev
!  read(itime_prev(5:6),'(i2)') imo_prev
!  read(itime_prev(7:8),'(i2)') ida_prev
!  read(itime_prev(9:10),'(i2)') ihr_prev
!  read(itime_prev(11:12),'(i2)') imin_prev
!  read(itime_prev(13:14),'(i2)') isec_prev
!endif

if(DBLEV== 5)then; write(msg,*)'LEV05 SPEEC_HECK ppos: ',ppos,' ppos2_good: ',ppos2_good; call wrtdbg(sub,msg); endif
if(prev_good_posn_date /= FILL14CH)then
    read(prev_good_posn_date,'(i4,5i2)') iyr_prev,imo_prev,ida_prev,ihr_prev,imin_prev,isec_prev
    write(msg,'("QC05 prev_date from prev_good_posn_date: ",i4,5i2)')iyr_prev,imo_prev,ida_prev,ihr_prev,imin_prev,isec_prev;
    call wrttrc(sub,msg);
elseif(ppos2_good /= 0 )then
    iyr_prev=prev_year2(ppos2_good);
    imo_prev=prev_mon2(ppos2_good);
    ida_prev=prev_day2(ppos2_good);
    ihr_prev=prev_hh2(ppos2_good);
    imin_prev=prev_mm2(ppos2_good);
    isec_prev=prev_sc2(ppos2_good);
    write(msg,'("QC05 prev_date from prev_ymdhsm(ppos2_good): ",i4,5i2)')iyr_prev,imo_prev,ida_prev,ihr_prev,imin_prev,isec_prev;
    call wrttrc(sub,msg);
else
    iyr_prev=launch_year
    imo_prev=launch_mon
    ida_prev=launch_day
    ihr_prev=launch_hh
    imin_prev=launch_mm
    isec_prev=launch_sc
    write(msg,'("QC05 prev_date from launch date/time: ",i4,5i2)')iyr_prev,imo_prev,ida_prev,ihr_prev,imin_prev,isec_prev;
    call wrttrc(sub,msg);
endif

! Calculate t1 -- Time of previous profile or launch
ihr=ihr_prev*100+imin_prev

call julday(iyr_prev,imo_prev,ida_prev,ihr,time1)!launch or previous tim
t1=time1+float(isec_prev)/daysec

! Latitude, Longitude of previous profile or launch
xlon1=xlon_prev
ylat1=ylat_prev
icnt=0      ! a counter of number of speed check test
do i=1,np

  icnt=icnt+1
  if(icnt > nnp)then
      write(errmsg(1),'("FATAL ERROR 705 QC05 during speed check err at 2")');
      write(errmsg(2),'("FATAL ERROR 705 QC05 number of position tests (", i5,") exceed maximum (",i5,") at sc1")')icnt,nnp;
      call error_exit(sub,705,2,errmsg); ! qc05 - number of posn test exceeds max #2
  endif

  ! Time of position i 
  ihr = hour(i)*100+mn(i)
  call julday(year(i),month(i),day(i),ihr,time2)
  t2=time2+float(sec(i))/daysec

  ! Time elapsed betw previous profile or launch and pos i (in secs)
  dt=(t2-t1)*daysec
  if(DBLEV== 5)then; write(msg,*)"Time Diff of prev prof to this (QC05)",dt,",",(DT_MAX_DAYS * daysec); call wrtlog(sub,msg); endif

  ! Distance betw pos i and previous profile or launch
  xlon2=xlon(i)
  ylat2=ylat(i)
  d_dist = distance(ylat1, xlon1, ylat2, xlon2) 
  if (dt .gt. 0 .and. (dt .lt. (DT_MAX_DAYS * daysec) ))then ! dt_max has to be in seconds (dt_max_days * daysec)
    speed(icnt)=d_dist/dt
    if(DBLEV== 5)then; write(msg,*)"Time Diff is < 0 or gt than max days1",speed(icnt); call wrtlog(sub,msg); endif
  elseif(xlon1 > 200 .or. ylat1 > 100)then
    ! All previous, including launch are FILL
    write(msg,'("Time Diff meaningless, xlon1/ylat1 are FIL value")'); call wrttrc(sub,msg);
    speed(icnt)=0;
  else
    ! Time elapsed <= 0 or > dt_max !!!! 
    if(DBLEV== 5)then; write(msg,*)"Time Diff is < 0 or gt than max days2",speed(icnt); call wrtlog(sub,msg); endif

    speed(icnt)=0
    if (pnum.le.1)then
      if(DBLEV== 5)then; write(msg,*)'----> speed_check (part a), warning: dt <= zero'; call wrtlog(sub,msg); endif
    else
      if(DBLEV== 5)then; write(msg,*)'----> speed_check (part b), warning: dt <= zero'; call wrtlog(sub,msg); endif
    endif
  endif

  if (speed(icnt)>max_speed)then
    spd_flag(icnt)=4
    flag_table(i,np+1)=4
    bad_previous_row(i)=4  ! Flag pos. as bad related to 
                           ! previous profile or launch
  else
    spd_flag(icnt)=1
    if(TIME_DIFF_SMALL)then
        flag_table(i,np+1)=1
    else
        flag_table(i,np+1)=0  ! Is this supposed to be "4" ?????? Confirm with Claudia.
    endif
       
    if(DBLEV== 5)then; write(msg,*)"icnt during prev prof,speed(icnt) = ",icnt,speed(icnt); call wrtlog(sub,msg); endif
  endif

enddo   ! do i=1,np

! Print results of this test for debugging purposes 
if(DBLEV== 5)then
  write(msg,'("First part of speed check against launch or previous profile:")'); call wrtlog(sub,msg); ! formerly format 93
  do i=1,np
    if (pnum.le.1)then
       write(msg,91)i,speed(i),year(i),month(i), day(i),hour(i),mn(i),sec(i),xlon(i),ylat(i),spd_flag(i); call wrtlog(sub,msg);
    else
       write(msg,92)i,speed(i),year(i),month(i), day(i),hour(i),mn(i),sec(i),xlon(i),ylat(i),spd_flag(i); call wrtlog(sub,msg);
    endif
  enddo
endif
 91   format('pos-laun',i2,' sp=',f7.4,1x,i4.4,'/',i2.2,'/',i2.2,1x,i2.2,':',i2.2,':',i2.2,1x,f9.4,f9.4,' flag=',i1)
 92   format('pos-prev',i2,' sp=',f7.4,1x,i4.4,'/',i2.2,'/',i2.2,1x,i2.2,':',i2.2,':',i2.2,1x,f9.4,f9.4,' flag=',i1)
 93   format('First part of speed check against launch or previous profile:')

!======================================================================= 
!    part c) Internal within profile positions check
!    velocity from point i to point i+1
!    Date-line crossing case
!    longdiff(xlon(i)-xlon(1)) < 180.0 (west -> east)
!    longdiff(xlon(i)-xlon(1)) > 180.0 (east -> west)
!
if(DBLEV== 5)then; write(msg,'("LEV 05 Second part of speed check within positions:")'); call wrtdbg(sub,msg); endif
do k=1,np
  ist=k+1
  jst=k
  if(DBLEV== 5)then; write(msg,*)"np = ",np; call wrtlog(sub,msg); endif
  icnt=icnt+1
  if(DBLEV== 5)then; write(msg,*)"icnt within profile posi chk = ",icnt; call wrtlog(sub,msg); endif

  if(icnt > nnp)then
      write(errmsg(1),'("FATAL ERROR 706 QC05 during speed check err at 3")');
      write(errmsg(2),'("FATAL ERROR 706 QC05 number of position tests (", i5,") exceed maximum (",i5,") at sc2")')icnt,nnp;
      call error_exit(sub,706,2,errmsg); ! qc05 - number of posn test exceeds max #3
  endif

  if (np.le.1) spd_flag(icnt)=0

  ! Calculate t1, time of position k                 
  ihr=hour(jst)*100+mn(k)
  call julday(year(jst),month(jst),day(jst),ihr,time1)
  t1=time1+float(sec(jst))/daysec

  ! Latitude, Longitude of position k
  xlon1=xlon(jst)
  ylat1=ylat(jst)
  
  if(DBLEV== 5)then; write(msg,*)"icnt within pos= ",icnt; call wrtlog(sub,msg); endif
  speed(icnt)=0
  spd_flag(icnt)=1

  do i=ist,np

    ! Latitude, Longitude of position i
    xlon2=xlon(i)
    ylat2=ylat(i)
    ! Distance betw positions k and i 
    d_dist = distance(ylat1, xlon1, ylat2, xlon2)

    ! Position i time
    ihr=hour(i)*100+mn(i)
    call julday(year(i),month(i),day(i),ihr,time2)    ! time2
    t_sec=float(sec(i))/daysec
    t2=time2+t_sec

    if ((time2-time1).le.30.0/(24.0*60.0))then

      ! Time elapsed betw positions k and i <= 30 min
      ! Speed Check cannot be done for these positions

      spd_flag(icnt)=5
      if(DBLEV== 5)then 
        write(msg,94)i-1,i,speed(icnt),year(i),month(i),day(i),hour(i),mn(i),sec(i),xlon(i),ylat(i),spd_flag(icnt);
	94 format('LEV05 @ 1, pos',i2,'-pos',i2,' sp=',f8.4,1x,i4.4,'/',i2.2,'/',i2.2,1x,i2.2,':',i2.2,':',i2.2,1x,f9.4,f9.4,' flag=',i1)
	call wrtlog(sub,msg);
        write(msg,*)'-->i=',i,' time2-time1<30 min case: time1=',time1,' time2=',time2; call wrtlog(sub,msg);
      endif
      flag_table(k,i)=5
    else 

      icnt=icnt+1

      if(icnt > nnp)then
	  write(errmsg(1),'("FATAL ERROR 707 QC05 during speed check err at 4")');
	  write(errmsg(2),'("FATAL ERROR 707 QC05 number of position tests (", i5,") exceed maximum (",i5,") at sc3")')icnt,nnp;
	  call error_exit(sub,707,2,errmsg); ! qc05 -  no of posn tests exceeds mas ##
      endif


      ! Time elapsed betw previous profile and pos i (in seconds)
      dt=(t2-t1)*daysec 
      if(DBLEV== 5)then; write(msg,*)"dt,DT_MAX_DAYS * daysec =",dt,(DT_MAX_DAYS * daysec) ; call wrtlog(sub,msg); endif
      if (dt .gt. 0 .and. dt .lt. (DT_MAX_DAYS * daysec))then
        speed(icnt)=d_dist/dt
        if(DBLEV== 5)then; write(msg,*)"Time Diff is < 0 or gt than max days 12",speed(icnt); call wrtlog(sub,msg); endif
      else
        speed(icnt)=0
        if(DBLEV== 5)then; write(msg,*)"Time Diff is < 0 or gt than max days 2",speed(icnt); call wrtlog(sub,msg); endif
        if(DBLEV== 5)then; write(msg,*)'----> speed_check (part c), warning: dt <= zero'; call wrtlog(sub,msg); endif
      endif
      spd_flag(icnt)=1
      flag_table(k,i)=1
      if (speed(icnt) .gt. max_speed)then
        spd_flag(icnt)=4
        flag_table(k,i)=4
!             bad_previous_row(icnt)=4  ! Flag pos. as bad related to 
!                                       ! previous profile or launch
      endif
      if(DBLEV== 5)then
        write(msg,95)k,i,speed(icnt),year(i),month(i),day(i),hour(i),mn(i),sec(i),xlon(i),ylat(i),spd_flag(icnt)	!LEV05
	95 format('LEV05 @2, pos',i2,'-pos',i2,' sp=',f8.4,1x,i4.4,'/',i2.2,'/',i2.2,1x,i2.2,':',i2.2,':',i2.2,1x,f9.4,f9.4,' flag=',i1)
	call wrtdbg(sub,msg)
      endif
    endif
  enddo  ! do i=ist,np
enddo  ! do k=1,np

if(DBLEV== 5)then; write(msg,*)'icnt=',icnt; call wrtlog(sub,msg); endif

! Only one position on the profile (np=1)
! Only need to check against the previous profile (launch)
if (np.eq.1)then
  if (bad_previous_row(1).eq.4)then
    bad_row(1)=4
    write(msg,*)'Only 1 pos. and failed against previous profile.'; call wrttrc(sub,msg);
    ispd_flag=4
    !pos_speedc(1)='4'
    return
  else
    write(msg,*)'Only 1 position and passed.'; call wrttrc(sub,msg);
    ispd_flag=1
    !pos_speedc(1)='1'
    return
  endif
endif

! Only two positions (np=2) and test between positions failed 
if ( (np.eq.2) .AND. (bad_row(1).eq.4) )then 
  write(msg,*)'Only 2 positions and failed within positions.'; call wrttrc(sub,msg);
  ispd_flag=4
  !pos_speedc(1)='4'
  !pos_speedc(2)='4'
  return
endif

! Print the speed flag table 
title = 'Speed Flag Table: ' !profile_id 
call print_flag_table(np,flag_table,title)

! Print bad positions (related to previous profile or launch) 
title="Failed speed check against previous profile:"
call print_bad_positions(np, bad_previous_row, title)

! Check between positions only (doesn't include check against
! previous profile or launch) 
! in each iteration, the position(s) with most bad flags
! are flagged as bad.
! This process is repeated until no position has more 
! than 1 bad flag.
i=0
do 
  i=i+1 
  write(str_i,'(i10)')i
  title="Speed Flag Table Loop # " // trim(adjustl(str_i)) 
  call flag_worse_positions(np,flag_table,bad_row,max_bad_cnt)
  write(msg,*)'Loop=',i; call wrttrc(sub,msg);
  write(msg,*)'max_bad_cnt=',max_bad_cnt; call wrttrc(sub,msg);
  call print_flag_table(np,flag_table,title)
  call print_nbr_bad_flags_per_pos(np,bad_row) 
  if (max_bad_cnt.le.1)then
    exit 
  endif
enddo

if (max_bad_cnt.eq.1)then
  ! After previous loop, all positions have at most 1 bad flag. 
  ! But each bad flag involves 2 positions, so we need to pick 
  ! the bad one.  Criterion used: The bad position is the one  
  ! further away from the median of all positions --excluding  
  ! the pair in the "dispute" 
  do k=1,np
    do i=1,np
      if (flag_table(k,i).eq.4)then
        call pick_bad_pos(np,flag_table,bad_row,xlon,ylat,k,i)
      endif;
    enddo;
  enddo;
endif;

ispd_flag=4 ! Initially assuming every position fails one of 
              ! the two tests. If just one position is allright  
              ! ispd_flag will be changed to 1
do k=1,np 
  if ((bad_row(k).eq.4) .OR. (bad_previous_row(k).eq.4))  then
    bad_row(k) = 4 ! for the case that bad_previous_row(k)=4 and
                   ! bad_row(i) ne 4
    !pos_speedc(k)='4'
    write(msg,*)'SPEED CHECK: Pos ',k,' gets a -" sign front of class.'; call wrttrc(sub,msg);
    write(msg,*)'SPEED CHECK: Pos ',k,' gets a -  sign front of class.'; call wrtlog(sub,msg);
  else
    ispd_flag=1
    !pos_speedc(k)='1'
  endif
enddo 

return
end subroutine speed_checkk


! --------------------------------------------------------------------
! subroutine flag_worse_positions
!    E. Ramos 09/2009
!    Flag the 'worse' positions inside the profile
!    'Worse' defined as those positions (p) with the biggest count of:
!      flag_table(p,1..n+1)=4 and flag_table(1..p-1,p)=4   
! --------------------------------------------------------------------
subroutine flag_worse_positions(np,flag_table,bad_row,max_bad_cnt)

use communication
use data_arrays

implicit none

! ===========================================================================================================
! varialbe/array declarations

integer, parameter :: mxs = 100
integer np, flag_table(100,100), bad_row(*) 
integer  k
integer pos_bad_cnt(mxs) ! how many bad points in a given pos.
integer max_bad_cnt ! maximum # of bad counts of any position 

! counting bad points of each position 
do k=1,np
   pos_bad_cnt(k)=0   
enddo 
do k=1,np
  do i=1,np
    if (flag_table(k,i).eq.4)then
      pos_bad_cnt(k)=pos_bad_cnt(k)+1
      pos_bad_cnt(i)=pos_bad_cnt(i)+1
    endif
  enddo
enddo

! find out the maximum number of bad counts of any position 
max_bad_cnt = 0
do k=1,np
   if (pos_bad_cnt(k).gt.max_bad_cnt)then  
     max_bad_cnt = pos_bad_cnt(k)
   endif
enddo 

if (max_bad_cnt.gt.1)then 
  ! flag positions with the max number of bad counts (the worse!!)
  do k=1,np
    if (pos_bad_cnt(k).eq.max_bad_cnt)then
      bad_row(k)=4
      !pos_speedc(k)='4'
      ! reverse the position's bad flags 
      ! first vertically
      do i=1,k-1
        if (flag_table(i,k).eq.4)then
           flag_table(i,k)=-4
        endif
      enddo
      ! then horizontally
      do i=k+1,np
        if (flag_table(k,i).eq.4)then
           flag_table(k,i)=-4
        endif
      enddo
    endif;
  enddo
endif

return
end subroutine flag_worse_positions


! ---------------------------------------------------------------------
! subroutine print_flag_table
!    E. Ramos 09/2009
!    Print the Flag Table
! ---------------------------------------------------------------------
subroutine print_flag_table(np,flag_table,title)

use communication			! module

implicit none

! ===========================================================================================================
! varialbe/array declarations

integer, intent( in ) :: np
integer, intent( in ) :: flag_table(100,100)
character(*), intent( in ) :: title

character(len=16), parameter :: sub = 'print_flag_table'

integer i, k 
character cnp*14,cnp1*14

cnp(1:4)='(3x,'
cnp(7:14)='(i2,1x))'
write(cnp(5:6),'(i2.2)') np+1
cnp1(1:4)='(i3,'
cnp1(7:14)='(i2,1x))'
write(cnp1(5:6),'(i2.2)') np+1
write(msg,115); call wrttrc(sub,msg);
 115  format(59('-'))
write(msg,*)title ; call wrttrc(sub,msg);
write(msg,cnp) (i,i=1,np+1); call wrttrc(sub,msg);
do k=1,np
  write(msg,cnp1) k,(flag_table(k,i),i=1,np+1); call wrttrc(sub,msg);
enddo
write(msg,115); call wrttrc(sub,msg);

end subroutine print_flag_table


! ---------------------------------------------------------------------
! subroutine print_nbr_bad_flags_per_pos 
!    E. Ramos 09/2009
!    Print the number of bad flags per position 
! ---------------------------------------------------------------------
subroutine print_nbr_bad_flags_per_pos(np,bad_row)

use communication                       ! module
use data_arrays                         ! module

implicit none

! ===========================================================================================================
! varialbe/array declarations

character(len=16), parameter :: sub = 'prnt_#_bad_flags'

integer np, bad_row(*)
integer k

write(msg,'("bad flags per position:",100i3)') (bad_row(k),k=1,np); 
call wrttrc(sub,msg);
!write(msg,*)'bad flags per position: ',(bad_row(k),k=1,np); 
call wrtlog(sub,msg);

return
end subroutine print_nbr_bad_flags_per_pos

! ---------------------------------------------------------------------
! subroutine print_bad_positions 
!    E. Ramos 09/2009
!    Print the positions that failed the test against the previous 
!    profile or launch
!---------------------------------------------------------------------

subroutine print_bad_positions(np, bad_pos, title)

use communication                       ! module
use data_arrays                         ! module

implicit none

! ===========================================================================================================
! varialbe/array declarations



integer np, bad_pos(*)
!integer k, i, cnt_bad
integer k,  cnt_bad
character(len=50) :: title
character(len=80) :: line
character(len=3) :: ci

character(len=16), parameter :: sub = 'prnt_bad_positns'

line=title 
cnt_bad=0
do k=1,np
   if (bad_pos(k).eq.4)then
     write(ci,'(i3)')k
     ci=trim(adjustl(ci))
     if (cnt_bad.eq.0)then
       line = trim(line)//" YES: " // ci
     else
       if (len(trim(line)) + len(ci).gt.80)then
          write(msg,*)line; call wrttrc(sub,msg);
          line=ci
       else 
          line=trim(line) // "," // ci
       endif
     endif 
     cnt_bad=cnt_bad+1 
   endif
enddo
if (cnt_bad.eq.0)then
  line=trim(line)//" NO"
endif
write(msg,*)line; call wrtlog(sub,msg);
write(msg,*)line; call wrttrc(sub,msg);

return
end subroutine print_bad_positions


! ---------------------------------------------------------------------
! subroutine pick_bad_pos
!    E. Ramos 09/2009
!    Decides which position is the bad one based on it being further 
!    away of the mean of all positions excluding the 2 "contendents"
! ---------------------------------------------------------------------
subroutine pick_bad_pos(np,flag_table,bad_row,xlon,ylat,pos1,pos2)


use communication                       ! module
use data_arrays                         ! module

implicit none

double precision :: distance ! declare function type

integer :: np, pos1, pos2
real, dimension(*) :: xlon, ylat
integer, dimension(*) :: bad_row
integer, dimension(:,:) :: flag_table

real :: sum_lat, sum_lon, mean_lat, mean_lon, d1, d2
integer :: k, n 

character(len=12), parameter :: sub = 'pick_bad_pos'

write(msg,*)'Potentially bad positions:',pos1,pos2; call wrtlog(sub,msg);
write(msg,*)'Potentially bad positions:',pos1,pos2; call wrttrc(sub,msg);
write(msg,*)'Potentially bad positions:',pos1,pos2; call wrtlog(sub,msg);

sum_lat=0.0
sum_lon=0.0
n=0
do k=1,np
    if ((bad_row(k) /= 4))then
    sum_lat=sum_lat+ylat(k)
    sum_lon=sum_lon+xlon(k)
    n=n+1
  endif
enddo
if (n.ge.1)then
  mean_lon=sum_lon/n
  mean_lat=sum_lat/n
  d1=distance(mean_lat, mean_lon, ylat(pos1), xlon(pos1))
  d2=distance(mean_lat, mean_lon, ylat(pos2), xlon(pos2))
  !d1=(xlon(pos1)-mean_lon)**2+(ylat(pos1)-mean_lat)**2
  !d2=(xlon(pos2)-mean_lon)**2+(ylat(pos2)-mean_lat)**2
  write(msg,*)'mean_lat,mean_lon,d1,d2:',mean_lat,mean_lon,d1,d2; call wrtlog(sub,msg);
  write(msg,*)'mean_lat,mean_lon,d1,d2:',mean_lat,mean_lon,d1,d2 ; call wrttrc(sub,msg);
  write(msg,*)'mean_lat,mean_lon,d1,d2:',mean_lat,mean_lon,d1,d2 ; call wrtlog(sub,msg);
  if (d1.gt.d2)then
    write(msg,*)'Choosen bad position:',pos1; call wrtlog(sub,msg);
    write(msg,*)'Choosen bad position:',pos1; call wrttrc(sub,msg);
    bad_row(pos1)=4
  else
    write(msg,*)'Choosen bad position:',pos2; call wrtlog(sub,msg);
    write(msg,*)'Choosen bad position:',pos2; call wrttrc(sub,msg);
    bad_row(pos2)=4
  endif
else
  write(msg,*)'Cannot calculate mean. Both pos.flagged as bad'; call wrtlog(sub,msg);
  write(msg,*)'Cannot calculate mean. Both pos.flagged as bad'; call wrtlog(sub,msg);
  bad_row(pos1)=4
  bad_row(pos2)=4
endif

return
end subroutine pick_bad_pos

! ---------------------------------------------------------------------
! function distance 
!    E. Ramos 09/2009
!    Calculate distance between 2 pairs of coordinates 
! ---------------------------------------------------------------------
double precision function distance( lat1, long1, lat2, long2 )

  implicit none
  real :: lat1, long1, lat2, long2 
  real :: longdiff, dx, dy
  real, parameter :: factor = 1.852*1.e3*60.
  real, parameter :: pi = 3.14159265358979

  longdiff=long2-long1
  if (longdiff.ge.180.0) longdiff=longdiff-360.0
  if (longdiff.le.-180.0) longdiff=longdiff+360.0
  dx=(longdiff)*factor*cos(pi/180*(lat1+lat2)/2.0)
  dy=(lat2-lat1)*factor

  distance = sqrt(dx*dx + dy*dy)
 
end function distance 
