! ===========================================================================================================
! integer function qc22_ns_mixed_air_water.f90
!
! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who    what
! 20190416 VH     This test is only related to STS floats, eliminate the confusing warning. Eliminate
!                 the confusing STS/NST warning 155.
! 20180417 VH     General WARNING revisions.
! 20170712 all    Release of qcplus v1.1 - many changes to all, some documented/tracked, some not
! 20160511 all    Initial release    of qcplus v1.1.0
!
! ===========================================================================================================
! Notes
!
! Note that, if needed, this flag is TREATED DIFFERENTLY.  If a leve is good, this flag
! will be ZERO. (The current value of the level flag is NOT copied.)
!
! In the surface levels that are deemed to be "PROB_BAD", the value of nsflag will be set to
! PROB_BAD. In the FINAL flag evaluation, this will be transferred to the P/T/S flag array.
!
! Function return values:
!  1 = Success, profile passed
! -1 = Failure, profile failed
!  0 = Unable to determine
!

integer function qc22_ns_mixed_air_water(caller)

use communication                       ! module
use data_arrays                         ! module

implicit none

! ===========================================================================================================
! varialbe/array declarations

character(*), intent( in ) :: caller

character(len=4), parameter :: sub = 'QC22'
character(len=21) ,parameter :: QCTEST='NS_Mixed_air_water Test'
integer :: result;

integer :: ifl, itype, beg, end;
integer :: ilow;	! locatin of lower pres to compare
real    :: plow;	! lower pres to compare
real    :: thisp, lastp ! used for the test comparison
real    :: gap,diff	! required/allowable gap between successive levels above 5db

integer :: ngood,nbad,nskip;
integer :: cprf

! from data_arrays:
! integer, parameter :: p1r= 1, p1a= 2, p2r= 3, p2a= 4, p3r= 5, p3a= 6, p4r= 7, p4a= 8; ! PRES indices
! integer, parameter :: t1r= 9,         t2r=10,         t3r=11,         t4r=12;         ! TEMP indices
! integer, parameter :: s1r=13, s1a=14, s2r=15, s2a=16, s3r=17, s3a=18, s4r=19, s4a=20; ! PSAL indices

integer :: cpdat(MAXPRF) = (/  1,  3,  5,  7 /) ! previously / p1r, p2r, p3r, p4r /
integer :: ctdat(MAXPRF) = (/  9, 10, 11, 12 /) ! previously / t1r, t2r, t3r, t4r /
integer :: csdat(MAXPRF) = (/ 13, 15, 17, 19 /) ! previously / s1r, s2r, s3r, s4r /

! ===========================================================================================================
! executable statements

ifl = 1; ! KLUGE FIX THIS

result = NOTD; ! "can not be determined."

if(DBGIO) call dbg_enter(sub);

! Where is the near-surface profile that needs tested? Currently, there is only one per float.
iprf=0; cprf=0; ! We need to find the near_surface profile - first look for the "regular" idat
if(STS_prof(0))then; 
    nsprofile_is_sts = .true.;
    gap = 0.50;			! For STS floats, (APEXIridium), require 0.5 gap above 5db
    do iprf=1,nprf; if(STS_prof(iprf))then; idat=cpdat(iprf); cprf=iprf; endif; enddo
    write(nsqcline,'(a40,i1)')"NEAR_SURFACE STS PROFILE                ",cprf;
                     !1234567890123456789012345678901234567890
    if(DEBUG)then; write(msg,'("NEAR SURF QC22 The near_surface profile, ",i1,", is from an STS float.")')cprf; endif
elseif(NST_prof(0))then;
    nsprofile_is_nst = .true.;
    gap = 0.00;			! For NST floats, (APEX/Argos), require monotonic above 5db (gap=0.0)
    do iprf=1,nprf; if(NST_prof(iprf))then; idat=cpdat(iprf); cprf=iprf; endif; enddo
    write(nsqcline,'(a40,i1)')"NEAR_SURFACE NST PROFILE                 ",cprf;
    if(DEBUG)then; write(msg,'("NEAR SURF QC22 The near_surface profile, ",i1,", is from an NST float.")')cprf; endif
endif
if(DEBUG) call wrtdbg(sub,msg);
near_surface_profile = cprf; nspdat = cpdat(cprf) ; nstdat = ctdat(cprf); nssdat = csdat(cprf);	! Store for later use

! Which pres column should we use, regular of adjusted?
if(do_pres_adj)then;
    idat = idat + 1;
    if(DEBUG) write(DBG,'(a," Use the adjusted pres, idat: ",i1,".")')trim(msg),idat;
else
    if(DEBUG) write(DBG,'(a," Use the regular pres, idat: ",i1,".")')trim(msg),idat;
endif
if(DEBUG) call wrtdbg(sub,msg);

! The pres is stored so th shallowest pres is "on top" - look up from the deepest pres to first level above 5db
ngood = 0; nbad = 0; nskip = 0;
nsflag = 0; ! Intitalize the array
! This loop looks upwrd for the pres jus shallower than 5db
do ilev=nlev,1,-1
    if(profflag(idat,ilev) == MISSING)then
	if(DBLEV>999) write(msg,'("Dummy placeholder message")')
    elseif(profdata(idat,ilev) < 5.0)then;
	ilow = ilev;				! The lowest level ABOVE 5db
	plow = profdata(idat,ilev);		! The pres at that level
	go to 20;				! Found first level above 5db
    endif
enddo

result = GOOD;	! There are NO shallow levels - all deeper than 5db
if(DEBUG)then; write(msg,'("QC22 - All levels deeper than 5db, no levels are bad")'); call wrtlog(sub,msg); endif
go to 999 ! THE_END

20 continue;					! Found first level above 5db

if(DEBUG)then; write(msg,'("QC22 - deepest level avove 5dp is level ",i4," pres: ",f12.12)')ilow,plow; call wrtlog(sub,msg); endif
if(DEBUG)then; write(msg,'("QC22 = analyze using a gap of ",f6.2)')gap; call wrtlog(sub,msg); endif

lastp = profdata(idat,ilow);
do ilev=ilow-1,1,-1
    thisp = profdata(idat,ilev); diff = lastp - thisp;
    if(thisp >= lastp) exit; ! exit the loop, not the program
    lastp = thisp;
enddo
if(ilev > 0)then;
    if(DEBUG)then; write(msg,1001)ilev; call wrtlog(sub,msg); endif
    1001 format("QC22 bad levels ilev: ",i4," > 0) mark ilev and above as probably bad")
    ilow = ilev;
    do ilev=ilow,1,-1
	nsflag(ilev) = PROB_BAD;
	! DO IN FINAL EVAL if(nsflag(ilev) > profflag(idat,ilev)) profflag(idat,ilev) = nsflag(ilev);
    enddo
    result = PROB_BAD;
else
    result = GOOD;
endif

999 continue ! THE_END

if(DBGIO) call dbg_leave(sub,result);

! The result to return to the main is based on the first data field - pres/temp/psal/doxy for primary profile

near_surface_result = result;
qc22_ns_mixed_air_water = result;

return
end function qc22_ns_mixed_air_water
! ===========================================================================================================
