! ===========================================================================================================
! integer function read_pqc.f90
!
! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who    what
! 20181109 JN     Eliminate extraneous unfiltered debug statements (AAAA, DDDD, etc)
! 20180417 VH     General WARNING revisions.
! 20170809 VH     Special case ICE DETECT ... MEANING may be unfilled in cycle 000 - ignore this and
!                         issue a CRITICAL WARNING (later eliminated)
! 20170712 all    Release of qcplus v1.1 - many changes to all, some documented/tracked, some not
! 20160511 all    Initial release of qcplus v1.1.0
!
! ===========================================================================================================
! Notes
! ===========================================================================================================
! TEMP units DEGC DEG C deg...
! The units for TEMP will handle non-standard unis: they will be handled in two different ways.
! TEMP units can be any of:  (DEGC, (DEG C, (degc, (degC, etc.  The search will be for either (DEG or (deg.
! For COLUM names, the unist will be standardized to (DEG C) in the output qc file by write_qcfile.
! For DRIFT names, the units will NOT be changed, just recognized by read_pqc.
! ===========================================================================================================
! Decoding the pos block:
!================================================================
!   LATITUDE  LONGITUDE YEAR/MO/DY HR:MN:SC XMITS SAT CLASS
!   LATITUDE  LONGITUDE YEAR/MO/DY HR:MN:SC XMITS SAT POSITION_QC
!   LATITUDE  LONGITUDE YEAR/MO/DY HR:MN:SC XMITS SAT POSITION-QC
!   12.81093  -49.29844 2013/03/14 13:06:00   000  5     0
!1234567890123456789012345678901234567890123456789012345678901234567890
!         1         2         3         4         5         6         7
!================================================================
! yearmody=cline(23:33)  hrmnsc=cline(35:42)
! ===========================================================================================================

integer function read_pqc(caller,prevqc,first_read)

use communication                       ! module
use data_arrays                         ! module

implicit none

! ===========================================================================================================
! varialbe/array declarations

character(len=*), intent(in) :: caller
character(len=*), intent(in) :: prevqc 
logical,          intent(in) :: first_read

character(len=256) :: stem
character(len=3) :: testch
integer :: status, ios, ic, prvcyc, nback
integer :: nl,ip,it,is ! shorthand locations for flags for p,t,s
integer :: ip1,ip2,ip3,it1,it2,it3,it4,it5,is1,is2,is3,is4,is5,is6
integer :: depth
real    :: prevdatvals(0:MAXCOLS), tmp;
real    :: prev_tmp_lat, prev_tmp_lon;  ! LAT/LON from header, before qc status found
character(len=14) :: prev_tmp_posn_date
logical :: file_exist
logical :: prev_juld_found, prev_juld_good 
integer :: on_land		! local result of land_or_water
integer :: iyr,imo,idy,ihh,imn,isc,idate(6)
equivalence (idate(1),iyr), (idate(2),imo),(idate(3),idy),(idate(4),ihh),(idate(5),imn),(idate(6),isc)
character(len=8), parameter :: sub = 'read_pqc'
character(len=16) :: c16;
character(len=17) :: c17;
character(len=80)  ::  dummy_line
character(len=1024) :: pos_block_label

integer:: isl,idt,ie		! Used to find the profile from the input filename
character(len=128) :: qcerrloc	! Used to format error message lines

integer :: loc, nread_err     ! count the number of read errors

integer :: idx			! index location

integer :: nr			! Count the reads
! ===========================================================================================================
! executable statements
read_pqc = FAILURE;			! Assume failure
prevdatvals(0) = 0	! Fake flag for those tests that are not in the pqc file
fst_srf_actvtd=.false.
prev_hasMC_qc=hasMC_qc;
hasMC_qc=.false.;! resetting flags.
pre_deploy_last_idx=0;

if(DBLEV==35) call dbg_enter_file(sub,prevqc);

isl = index(prevqc,"/",.true.);	! true meas rindex - last occurrence, not first
idt = index(prevqc,".",.false.);	! normal index - first currence
qcfile = prevqc(isl+1:idt-1);
write(qcerrloc,'(a," prof: ",a20," qcfile: ",a)')trim(sub),profile,trim(qcfile);

inp_open=open_old_file(INP,prevqc,"INP","previous qc file");
if(DBLEV==35)then; write(msg,'("Open PQC: ",a)')trim(prevqc); call wrtstd(sub,msg); endif
HED=.true.; POS=.false.; DAT=.false.; ! we start n the header, duh
if(trim(caller) == 'get_pqc')then ! should not reset the values if get_all_qc is calling.
    prev_juld_found =.false.; prev_juld_good =.false.;
    prev_posn_found =.false.; prev_posn_good=.false.;
    prev_tmp_lat = FILL; prev_tmp_lon = FILL; prev_tmp_posn_date = FILL14CH;
    prev_good_lat = FILL; prev_good_lon = FILL; prev_good_posn_date = FILL14CH;
endif
if(DBLEV== 5)then; write(msg,*)'readpqc prev_good_posn_date: >',prev_good_posn_date,'<'; call wrtlev(sub,msg); endif
prev_icolinp = 0;	! always reset in case we need to read multiple times
npqc = 0;		! How name lines are we stashing away in pqcline?
PREV_TRANS_START_FOUND= .false.
ppos2_good = 0
nr=0;
 
1000 format("DBGpresadj read_pqc ",a1," mrvsp found: ",l1," mrvsp: ",f9.2," mrcyc found: ",l1," mrcyc: ",i6," cmnt: ",a);

do while(HED)
    read(INP,'(a)',end=101,err=961,iostat=ios) cline;
    ! 20170807 Special case of an error in a phy file tht WHOI can't take time to fix - so, as a one-time favor
    ! we're just going to fix it ourselves - thie only occurrs in cycl 000
    if(trim(cline) == "ICE DETECT FLOAT STATUS MEANING")then
	    write(cline,'("ICE DETECT FLOAT STATUS MEANING         N/A")')
	    write(wrnmsg,'("WARNING 149 read_pqc The ICE DETECT MEANING no VARL for pqc file: ",a)')trim(qcfile); ! WRN
	    call warning(sub,149,wrnmsg); ! ARNING 149 in read_pqc ! WRN
    endif
    if(DBGIO)then; write(msg,'("DBGIO HED read_pqc check_hed, cline: ",a)')trim(cline); call dbgmsg(sub,msg); endif
    call check_hed(sub,prevqc,cline);
    npqc = npqc + 1; pqcline(npqc) = trim(cline);
    read(cline,'(a40,a)')variable,value
    if(DBLEV==35) msg='none';
    if (trim(variable) == "JULD QC" .or. trim(variable) == "JULD_QC")then		! WE MAY NOT NEED THIS
	    prev_juld_found = .true.							! WE MAY NOT NEED THIS
	    PREV_JULD_QC = value
        if (value .eq. '1' .or. value .eq. '2')prev_juld_good=.true.			! WE MAY NOT NEED THIS
    elseif(trim(variable) == "POSITION_QC")then						! 1 means posn/date/land/speed passed
        if(trim(caller) == 'get_pqc')then; 
            prev_posn_found = .true.
	        if (value .eq. '1' .or. value .eq. '2')prev_posn_good=.true.
	    endif
    elseif(trim(variable) == "LATITUDE")then
	    read(value,*)prev_tmp_lat;
	    if(DBLEV==35) write(msg,'("read_pqc  LATITUDE: ",f8.2)')prev_tmp_lat
    elseif(trim(variable) == "LONGITUDE")then
	    read(value,*)prev_tmp_lon;
    	if(DBLEV==35) write(msg,'("read_pqc LONGITUDE: ",f8.2)')prev_tmp_lon
    elseif(trim(variable) == "DATE_TIME_LOCATION")then
	    read(value,*)prev_tmp_posn_date
	    if(DBLEV==35) write(msg,'("read_pqc DATE_TIME_LOCATION: ",a)')prev_tmp_posn_date
    elseif(caller == "get_pqc" .and. trim(variable) == "MOST RECENT VALID SURFACE PRES (DBAR)")then
	    read(value,*)pqc_mrvsp; found_pqc_mrvsp = .true.; pqc_mrvsp_comment = "MRVSP IN PQC FILE = NOT VALIDATED";
    elseif(caller == "get_pqc" .and. trim(variable) == "MOST RECENT VALID SURFACE PRES CYCLE")then
	    read(value,*)pqc_mrcyc; found_pqc_mrcyc = .true.;
    elseif(caller == "get_pqc" .and. trim(variable) == "MOST RECENT VALID SURFACE PRES COMMENT")then
	    pqc_mrvsp_comment = trim(value);
    elseif(trim(variable) == "JULD")then
	    read(value,*)PREV_JULD
	    if(DBLEV==35) write(msg,'("read_pqc JULD: ",f16.8)')prev_juld
    elseif(trim(variable) == "START OF TRANSMISSION (YYYYMMDDHHMMSS)")then
        prev_start_of_trans=trim(cline(41:))
        read(prev_start_of_trans,'(i4,i2,i2,i2,i2,i2)')iyr,imo,idy,ihh,imn,isc
        prev_start_of_trans_juld=juldays1950(idate)
	if(DEBUG)then; ! DBG
	    write(dblstr,*)prev_start_of_trans_juld; ! DBG
	    call wrtlog(sub,msg);
	endif; ! DBG
        PREV_TRANS_START_FOUND = .true.
	    if(DBLEV==35) write(msg,'("read_pqc START OF TRANSMISSION ",a)')prev_start_of_trans
    elseif(len_trim(variable)> 0 .and. index(variable, "START OF TRANSMISSION   ")> 0 )then !JN: newly added
        prev_start_of_trans=trim(cline(41:))
        read(prev_start_of_trans,'(i4,x,i2,x,i2,x,i2,x,i2,x,i2)')iyr,imo,idy,ihh,imn,isc
        prev_start_of_trans_juld=juldays1950(idate);
	if(DEBUG)then; ! DBG
	    write(dblstr,*)prev_start_of_trans_juld; ! DBG
	    call wrtlog(sub,msg);
	endif; ! DBG
        PREV_TRANS_START_FOUND = .true.
	    if(DBLEV==35) write(msg,'("read_pqc START OF TRANSMISSION ",a)')prev_start_of_trans
    elseif(variable(1:2) == "MC" )then
    	if(.not. hasMC_qc)hasMC_qc = .true.; ! indicates this qc file has measurement codes.
	    if(index(variable,"TEMP") > 0 .and. index(variable,"(DEGC)") == 0)then  ! standardized units for drift temperatures
		    is = index(variable,"(DEG");
		    if(is == 0) is = index(variable,"(deg");
		    if(is > 0)then
		        variable = variable(1:(is-1)) // "(DEGC)";
		        write(pqcline(npqc),'(a40,a)')variable,trim(value);
	        else
		    write(errmsg(1),'("NON-FATAL ERROR 240 in ",a," profile: ",a)')trim(sub),trim(qcerrloc);
		    write(errmsg(2),'("NON-FATAL ERROR 240 ",a," MC&TEMP bad units")')trim(qcerrloc);
		    write(errmsg(3),'("NON-FATAL ERROR 240 Drift line has MC, and TEMP, but the units were not recognized")')
		    write(errmsg(4),'("NON-FATAL ERROR 240 Drift line: ",a)')trim(variable);
		    call error_report(sub,240,4,errmsg); ! read_pqc MC&TEMP bad units
	    endif
	endif
	if(index(variable,"(YYYYMMDDHHMMSS)")>0)then           ! line has a time stamp.
	    read(value,'(i4,5i2)')iyr,imo,idy,ihh,imn,isc;
	    if (iyr .ne. 9999. .and. juldays1950(idate) < launch_time_juld)pre_deploy_last_idx=npqc;
	endif
    elseif(trim(variable) == "FSTSRF ACTIVATED STATUS")then
        if (trim(value) == '1')fst_srf_actvtd=.true.;
    endif
    if(DBLEV==35 .and. msg /= 'none') call wrtlev(sub,msg);
    if(variable(1:12) == "============") HED=.false.; ! Sometimes, long ago, the ===== line
    if(variable(1:12) == "   LATITUDE ") HED=.false.; ! is missing between hed & pos
    if(variable(4:9) == "COLUMN" .or. variable(5:10) =="COLUMN")then
    ! return to the old one - this one would include "NUMBER OF COLUMNS" if(index(variable ,"COLUMN ") > 0 )then
	prev_icolinp = prev_icolinp + 1; prev_inpcols(prev_icolinp) = trim(value)	! inp columns
    endif
enddo

! Did we find the previous position in the header?
if(prev_posn_found .and. prev_posn_good)then	! We found a good position, transfer the lat/lon/time
    if(trim(caller) == 'get_pqc')then;
        prev_good_lat = prev_tmp_lat;
        prev_good_lon = prev_tmp_lon;
        prev_good_posn_date = prev_tmp_posn_date;
        read_pqc = SUCCESS;				! Signal that we have a good previous posn/date
        write(msg,'("PQC Previous posn found for speed_check: lat: ",f7.2," lon: ",f8.2," date: ",a)') &
	    prev_good_lat,prev_good_lon,prev_good_posn_date
        call wrttrc(sub,msg);
    endif
else
endif

if(DBLEV==35)then
    write(msg,'("read_pqc found a good previsous posiiton, it IS irst_read, see if we have a profile")'); call wrtlev(sub,msg);
endif	

prev_ncolinp = prev_icolinp;

go to 103 ! End of header found, either ============ or LAT/LON line
101 continue
    write(msg,'("End-of-file reading HED, no positions retrieved")'); call wrttrc(sub,msg);
    go to 999; ! Skip to THE_END leaving return valuse FAILURE
103 continue ! HED has bee read, proceed to POS
HED=.false.; POS=.true.; ! after the header, read the position
! sometimes the '=====-' line is missing before the LATITUDE/LONGITUDE line
if(variable(1:12) == "============")then
    read(INP,'(a)',end=201,err=962,iostat=ios) cline; call check_inp(sub,prevqc,cline);
else
    write(wrnmsg,'("WARNING 133 read_pqc No ============ between HED and POS in pqc file: ",a)')trim(prevqc); ! WRN
    call warning(sub,133,wrnmsg); ! WARNING 133 rin ead_pqc ! WRN
endif
go to 203 ! LAT/LON line read after ============
201 continue
    write(msg,'("End-of-file during HED, no position retrieved")'); call wrttrc(sub,msg);
   go to 999; ! Skip to THE_END leaving return valuse FAILURE
203 continue ! LAT/LON line has bee read, proceed to POS
!-------------------------------------------------------
ppos=0;ppos2=0;
if(len_trim(cline)> 0 .and. index(cline, "   LATITUDE ")>0)then;
    pos_block_label = trim(cline);
else
    pos_block_label = "NON-FATAL ERROR 241 in read_pqc -  label for position block NOT FOUND"
    write(errmsg(1),'("NON-FATAL ERROR 241 in ",a," profile: ",a)')trim(sub),trim(qcerrloc);
    write(errmsg(2),'("NON-FATAL ERROR 241 ",a," No label line for position block")')trim(qcerrloc);
    write(errmsg(3),'("NON-FATAL ERROR 241 ",a,": trying to read line: ",a)')trim(cline)
    write(errmsg(4),'("NON-FATAL ERROR 241 ",a,": trying to read file: ",a)')trim(prevqc)
    call error_report(sub,241,4,errmsg); ! read_pqc No label line for position block
    read_pqc = FAILURE; ! Just to make sure it is a failure...
    return
endif
if(DBLEV==35)then; write(msg,*)"POS before getting LAT LON = ",POS; call wrtdbg(sub,msg); endif
do while(POS)
    read(INP,'(a)',end=301,err=963,iostat=ios) cline; call check_inp(sub,prevqc,cline);
    if(DBLEV==35)then;write(msg,'("PQC POS: ",a)')trim(cline);call wrtlev(sub,msg);endif
    if(cline(1:12) == "============")then
	POS=.false.; 
    else
        if(qc_descending .and. make_nctraj)then
	    if(index(cline,"FORTRAJ ")>0 )then
		idx=index(cline,"FORTRAJ ");
		cline=cline(:idx)
	    else
	    	!write(*,*)"DBG:>>> - Skipping NON-FORTRAJ ", trim(cline);
		cycle
	    endif
	endif
	ppos = ppos + 1;
	read(cline,*)prev_lat(ppos),prev_lon(ppos)
	ic=index(cline,"/"); 		! YEAR/MM/DY
	string4=cline(ic-4:ic-1); read(string4,*)prev_year(ppos);
	string2=cline(ic+1:ic+2);  read(string2,*)prev_mon(ppos);
	string2=cline(ic+4:ic+5);  read(string2,*)prev_day(ppos);
	ic=index(cline,":");		! HH:MM:SC
	string2=cline(ic-2:ic-1); read(string2,*)prev_hh(ppos)
	string2=cline(ic+1:ic+2); read(string2,*)prev_mm(ppos)
	string2=cline(ic+4:ic+5); read(string2,*)prev_sc(ppos)
        
        dummy_line=trim(cline(ic+9:len(trim(cline))))

	nread_err = 0;
	go to 420;	! Skip over the retry part
	400 continue
	nread_err = nread_err +1;
	ls = len_trim(pos_block_label) - 6; pos_block_label = pos_block_label(1:ls);
	if(nread_err > 1)then;
	    ! Just admit we can't read the rest of the postion line, just the lat/lon/date/time
	    ! Make sure any values that were actually read are re-initialized - they7 may be corrupted
	    prev_XMITS(ppos) = '0';
	    prev_SAT(ppos) = '0';
	    prev_posOrClass(ppos) = '0';
	    prev_RTPQC(ppos) = '0';
	    prev_RTTQC(ppos) = '0';
	    prev_RTSQC(ppos) = '0';
	    go to 430;	! Skip out
	endif
	420 continue
	if(len_trim(pos_block_label)>0 .and. index(pos_block_label, 'RTPQC RTTQC RTLQC RTSQC')>0)then
	    loc = 1;
	    read(dummy_line,*)prev_XMITS(ppos),prev_SAT(ppos),prev_posOrClass(ppos),&
		prev_RTPQC(ppos),prev_RTTQC(ppos),prev_RTLQC(ppos),prev_RTSQC(ppos)
	elseif(len_trim(pos_block_label)>0 .and. index(pos_block_label, 'RTPQC RTTQC RTSQC')>0)then
	    loc = 2;
	    read(dummy_line,*,end=400,err=400)prev_XMITS(ppos),prev_SAT(ppos),prev_posOrClass(ppos),&
		prev_RTPQC(ppos),prev_RTTQC(ppos),prev_RTSQC(ppos)
	elseif(len_trim(pos_block_label)>0 .and. index(pos_block_label, 'RTPQC RTTQC')>0)then
	    loc = 3;
	    read(dummy_line,*,end=400,err=400)prev_XMITS(ppos),prev_SAT(ppos),prev_posOrClass(ppos),&
	    prev_RTPQC(ppos),prev_RTTQC(ppos)
	else
	    ! Just admit we can't read the rest of the postion line, just the lat/lon/date/time
	    prev_XMITS(ppos) = '0';
	    prev_SAT(ppos) = '0';
	    prev_posOrClass(ppos) = '0';
	    prev_RTPQC(ppos) = '0';
	    prev_RTTQC(ppos) = '0';
	    prev_RTSQC(ppos) = '0';
	endif
	430 continue
	if(nread_err > 0)then
	    if(DEBUG)then
		write(errmsg(1),'("FATAL ERROR 440 DBG=T in ",a," profile: ",a)')trim(sub),trim(qcerrloc);
		write(errmsg(2),'("FATAL ERROR 440 DBG=T ",a," Reading position block")')trim(qcerrloc);
		write(errmsg(3),'("FATAL ERROR 440 DBG=T occurred trying to read QC values from position line")');
		write(errmsg(4),'("FATAL ERROR 440 DBG=T ALL QC values in the position line will be IGNORED")');
		call error_exit(sub,440,4,errmsg); ! DBG=T read_pqc - DBG=T error trying to read QC values from the position line
	    else
		write(errmsg(1),'("NON-FATAL ERROR 340 DBG=F in ",a," profile: ",a)')trim(sub),trim(qcerrloc);
		write(errmsg(2),'("NON-FATAL ERROR 340 DBG=F ",a," Error reading position flags")')trim(qcerrloc);
	        write(errmsg(3),'("NON-FATAL ERROR 340 DBG=F occurred trying to read QC values from position line")');
	        write(errmsg(4),'("NON-FATAL ERROR 340 DBG=F ALL QC values in the position line will be IGNORED")');
		call error_report(sub,340,4,errmsg); ! DBG=F read_pqc Error reading position flags
	    endif
	endif

	ic=len(trim(cline)); prev_classqc(ppos)=cline(ic-1:ic)
        prev_sign(ppos)=cline(ic-1:ic-1);
	if(prev_SAT(ppos) == 'I')has_irid_positioning = .true.   ! To find positioning systems for nc_meta_array
	!if(DBLEV== 5)then; write(msg,'("prev posns i: ",i3,f12.2)')ppos,prev_lat(ppos),prev_lon(ppos); call wrtlev(sub,msg); endif
	!if(DBLEV== 0)then; write(msg,'("prev posns i: ",i3,f12.2)')ppos,prev_lat(ppos),prev_lon(ppos); call wrtlev(sub,msg); endif
	if( prev_lat(ppos) <= 90.0 .and. prev_lon(ppos) <= 180.0)then
            if(DBLEV== 0)then;
		write(msg,*)"prev_lat & prev_lon(ppos) = ",":",prev_lat(ppos),":,:",prev_lon(ppos),":"; call wrtlev(sub,msg);
		write(msg,*)"prev_year& prev_hh(ppos) = ",":",prev_year(ppos),":,:",prev_hh(ppos),":"; call wrtlev(sub,msg);
	    endif
	    ppos2 = ppos2 +1;
	    prev_lat2(ppos2) = prev_lat(ppos);
	    prev_lon2(ppos2) = prev_lon(ppos);
	    prev_year2(ppos2) = prev_year(ppos);
	    prev_mon2(ppos2) = prev_mon(ppos);
	    prev_day2(ppos2) = prev_day(ppos);
	    prev_hh2(ppos2)=prev_hh(ppos);
	    prev_mm2(ppos2)=prev_mm(ppos);
	    prev_sc2(ppos2)=prev_sc(ppos);
            prev_XMITS2(ppos2)=prev_XMITS(ppos);
	    prev_SAT2(ppos2)=prev_SAT(ppos);
	    prev_posOrClass2(ppos2)=prev_posOrClass(ppos);
	    prev_RTPQC2(ppos2)=prev_RTPQC(ppos);
	    prev_RTTQC2(ppos2)=prev_RTTQC(ppos);
	    prev_RTLQC2(ppos2)=prev_RTLQC(ppos);
	    prev_RTSQC2(ppos2)=prev_RTSQC(ppos);
	    prev_RTPQC2(ppos2)=prev_RTPQC(ppos);
	    prev_classqc2(ppos2)=prev_classqc(ppos);
	    prev_sign2(ppos2)=prev_sign(ppos);
	endif
    endif
200 continue

enddo
 
if(qc_descending .and. make_nctraj .and. trim(caller) == 'get_all_qc' )then ! Calling subroutine to fill the missing POS QC flags. 
    call fill_missing_pos_flags(sub);    ! for the "FORTRAJ" lines with "0" flags.
endif

POS=.false.; DAT=.true.
if(DBLEV== 5)then; write(msg,*)'in readpqc, after HED, prev_posn_found: ',prev_posn_found; call wrtlev(sub,msg); endif
if(prev_posn_found)then
    FOUND_P_POS = .true. ;
    if(DBLEV== 5)then; write(msg,'("Posn block read, prev_posn_found T, skip further analysis")'); call wrtlev(sub,msg); endif
    go to 300		! SKIP_ANALYSIS
endif

if(ppos2 == 0)then 
    FOUND_P_POS=.false.;
    if(DEBUG)then; write(msg,'("ppos2==0, so set FOUND_P_POS=.false.")'); call wrtlev(sub,msg); endif
else
    do i=ppos2,1,-1
       if(.not.(prev_sign2(i) == "-"))then
       	   if(trim(caller) == 'get_all_qc' .and. make_nctraj)then
       	       if(DBLEV==35)then;
       	           write(msg,*)" DBG caller is ", caller, " for NON-PROF job. " ;call wrtdbg(sub,msg);
       	       endif
               if(prev_SAT2(i) == 'I')cycle; ! "Iridium" position can't be used as a good prev position for TRAJ jobs - Claudia.
           endif
	   on_land = land_or_water(sub,prev_lat2(i),prev_lon2(i))
	   if(DBLEV== 5)then; c16 = 'ppos2>0, onland='; c17=' set FOUND_P_POS='; endif
           if(on_land == 0)then				! pos_on_land)then pos_on_land USED ELSEWHERE
               FOUND_P_POS = .true. ;			! position is in water
	       if(DBLEV== 5)then; write(msg,'(a,"F i: ",i3,a,".trye,")')c16,i,c17; call wrtlev(sub,msg); endif
               read_pqc = SUCCESS;			! signal we found a good prev_posn
	       prev_good_lat = prev_lat2(i);
	       prev_good_lon = prev_lon2(i);
	       write(prev_good_posn_date,'(i4,5i2.2)')prev_year2(i),prev_mon2(i),prev_day2(i) &
	       ,prev_hh2(i),prev_mm2(i),prev_sc2(i);
	       ppos2_good=i;
	       !write(*,*)"DBG:>>> - setting prev good pos -  ", prev_good_lat,prev_good_lon;
	       !write(*,*)"DBG:>>> - good posn date  -  ", prev_year2(i),prev_mon2(i),prev_day2(i),prev_hh2(i),prev_mm2(i),prev_sc2(i);
	       go to 10 
	   elseif(on_land > 0)then
		FOUND_P_POS=.false.			! position is on land
		if(DBLEV== 5)then; write(msg,'(a,"T i: ",i3,a,".false.")')c16,i,c17; call wrtlev(sub,msg); endif
	   else
		! if on_lannd < 0 = THE TOPOGRAPHY FILE COUND NOT BE FOUND
		! The warning will be reorted DIRECTLY FROM lnad_or_later.
		FOUND_P_POS=.false.			! TOPOGRAPHY database not found
		if(DBLEV== 5)then; write(msg,'(a,"BAD i: ",i3,a,".false.,exit")')c16,i,c17; call wrtlev(sub,msg); endif
		go to 10				! Not gonna find it on succeeding tries
	   endif
       endif
    enddo
    10 continue

endif;
if (ppos2_good .gt. 0 .and. FOUND_P_POS)then
    ! Time elapsed betw previous profile or launch and pos i (???)
    call time_elapsed_in_days(sub);
endif

	
300 continue		! SKIP_ANALYSIS

if(.not. first_read)then;
    if(DBLEV==35)then;
	write(msg,'("PQC HED and POS read, as needed, NOT first_read,so do NOT read DAT")');
	call wrtlog(sub,msg);
    endif
    go to 999; ! Skip to THE_END
endif

!-------------------------------------------------------
! Added by CA
! NOTE THE OFFSET BEG HERE
hasLevFLAGS = .false.;
! prev_npres=0; prev_ntemp=0; prev_npsal=0; 
do icol=1,prev_ncolinp
    value = trim(prev_inpcols(icol));
    if(value == "PRESSURE (DBAR)")then
	hasPrvPRES = .true.; prev_inprescol(1) = icol;! Store pres info
    elseif(value == "TEMPERATURE-90 (DEG C)")then
	hasPrvTEMP = .true.; prev_intempcol(1) = icol; ! Store temp info
    elseif(value == "SALINITY (PSU)")then
	hasPrvPSAL = .true.; prev_inpsalcol(1) = icol; ! Store psal info
    elseif(value == "")then
	prev_inpcols(icol) = ""
    endif
enddo

! see if the level flags are present
do icol=1,prev_ncolinp
    value = trim(prev_inpcols(icol));
    if(value == "LEV CHECK_P FLAG")then; hasLevFLAGS = .true.; lev_inprescol = icol; endif
    if(value == "LEV CHECK_T FLAG")then; hasLevFLAGS = .true.; lev_intempcol = icol; endif
    if(value == "LEV CHECK_S FLAG")then; hasLevFLAGS = .true.; lev_inpsalcol = icol; endif
enddo

if(.not. hasLevFLAGS)then
    if(DBLEV==35)then; write(msg,*)'The lev flags are NOT present, locate the individual flags'; call wrtlev(sub,msg); endif
    ip1=0;ip2=0;ip3=0;it1=0;it2=0;it3=0;it4=0;it5=0;is1=0;is2=0;is3=0;is4=0;is5=0;is6=0;
    do icol=1,prev_ncolinp
	    value = trim(prev_inpcols(icol));
	    if(value == "DEEPEST PRESSURE CHECK FLAG")then; 	 ip1=icol;
	elseif(value == "DEEPEST PRESS CHECK_P FLAG")then; 	 ip1=icol;
	elseif(value == "PRESSURE INCREASING CHECK_P FLAG")then; ip2=icol;
	elseif(value == "GROSS CHECK_P FLAG")then; 		 ip3=icol;
	elseif(value == "GROSS CHECK_T FLAG")then; 		 it1=icol;
	elseif(value == "REGION CHECK_T FLAG")then; 		 it2=icol;
	elseif(value == "SPIKE CHECK_T FLAG")then; 		 it3=icol;
	elseif(value == "VERTICAL GRADIENT CHECK_T FLAG")then; 	 it4=icol;
	elseif(value == "CONSTANT VALUE CHECK_T FLAG")then; 	 it5=icol;
	elseif(value == "GROSS CHECK_S FLAG")then; 		 is1=icol;
	elseif(value == "REGION CHECK_S FLAG")then; 		 is2=icol;
	elseif(value == "SPIKE CHECK_S FLAG")then; 		 is3=icol;
	elseif(value == "VERTICAL GRADIENT CHECK_S FLAG")then; 	 is4=icol;
	elseif(value == "CONSTANT VALUE CHECK_S FLAG")then; 	 is5=icol;
	elseif(value == "DENSITY GRADIENT CHECK_S FLAG")then; 	 is6=icol;
	endif
    enddo
else
    if(DBLEV==35)then; write(msg,*)'The lev flags ARE present.'; call wrtlev(sub,msg); endif
endif

HED=.false.; POS=.false.; DAT=.true.;  ! Finally, read the data
! 
! Everything has been read except the data itself - initialize the prev< >data & prev< >flag arrays 
!
do ilev=1,MAXLEV
    prevpresdata(ilev) = FILL;
    prevtempdata(ilev) = FILL;
    prevpsaldata(ilev) = FILL;
    prevpresflag(ilev) = MISSING;
    prevtempflag(ilev) = MISSING;
    prevpsalflag(ilev) = MISSING;
enddo
ip = lev_inprescol;
it = lev_intempcol;
is = lev_inpsalcol;

nprevlev=0; ! Read from the first data line

if(DBLEV==35)then; write(msg,*)'Reading Data from the first data line.'; call wrtlev(sub,msg); endif

do while(DAT)
    read(INP,'(a)',end=401,err=966,iostat=ios) cline; call check_inp(sub,prevqc,cline);
    if(index(cline,"pres") > 0)then;		! Skip the two HDR data header lines,if they are present
	read(INP,'(a)',end=401,err=966,iostat=ios) cline; call check_inp(sub,prevqc,cline);
	read(INP,'(a)',end=401,err=966,iostat=ios) cline; call check_inp(sub,prevqc,cline);
    endif
    nprevlev=nprevlev + 1; nl = nprevlev
    read(cline,*,end=968,err=999)(prevdatvals(i),i=1,prev_ncolinp) ! Skip to THE_END on read err

    ! Do not need to use parst_dataline of any type, just read directly
    prevpresdata(nl)=prevdatvals(1);
    prevtempdata(nl)=prevdatvals(2);
    prevpsaldata(nl)=prevdatvals(prev_inpsalcol(1));
    if(hasLevFLAGS)then
	prevpresflag(nl)=prevdatvals(ip);
	prevtempflag(nl)=prevdatvals(it);
	prevpsalflag(nl)=prevdatvals(is);
    else
	prevpresflag(nl) = ifix(prevdatvals(ip1));
	prevpresflag(nl) = max(prevpresflag(nl),ifix(prevdatvals(ip2)));
	prevpresflag(nl) = max(prevpresflag(nl),ifix(prevdatvals(ip3)));
	write(msg,'("TEST FLAGS P: ",i3,1x,3i2)')prevpresflag(nl),ifix(prevdatvals(ip1)),ifix(prevdatvals(ip2)),ifix(prevdatvals(ip3));
        if(prevpresflag(nl) .eq. 0)prevpresflag(nl)=9;
	prevtempflag(nl) = ifix(prevdatvals(it1));
	prevtempflag(nl) = max(prevtempflag(nl),ifix(prevdatvals(it2)));
	prevtempflag(nl) = max(prevtempflag(nl),ifix(prevdatvals(it3)));
	prevtempflag(nl) = max(prevtempflag(nl),ifix(prevdatvals(it4)));
	prevtempflag(nl) = max(prevtempflag(nl),ifix(prevdatvals(it5)));
        if(prevtempflag(nl) .eq. 0)prevtempflag(nl)=9;
	prevpsalflag(nl) = ifix(prevdatvals(is1));
	prevpsalflag(nl) = max(prevpsalflag(nl),ifix(prevdatvals(is2)));
	prevpsalflag(nl) = max(prevpsalflag(nl),ifix(prevdatvals(is3)));
	prevpsalflag(nl) = max(prevpsalflag(nl),ifix(prevdatvals(is4)));
	prevpsalflag(nl) = max(prevpsalflag(nl),ifix(prevdatvals(is5)));
	prevpsalflag(nl) = max(prevpsalflag(nl),ifix(prevdatvals(is6)));
        if(prevpsalflag(nl) .eq. 0)prevpsalflag(nl)=9;
	if(DBLEV==35)then;
	    write(msg,'("PQC ilev: ",i4)')nl;	! nl = number of (previous) levels						! LEV35
	    write(msg,'(a," FLAGS P: ",i3,1x,3i2)')trim(msg),prevpresflag(nl),ifix(prevdatvals(ip1)),ifix(prevdatvals(ip2)),&	! LEV35
		ifix(prevdatvals(ip3));												! LEV35
	    write(msg,'(a,"    T: ",i3,1x,3i2)')trim(msg),prevpresflag(nl), ifix(prevdatvals(ip1)), ifix(prevdatvals(ip2)), &	! LEV35
		ifix(prevdatvals(ip3));												! LEV35
	    write(msg,'(a,"    S: ",i3,1x,6i2)')trim(msg),prevpsalflag(nl), ifix(prevdatvals(is1)), ifix(prevdatvals(is2)), &	! LEV35
	    ifix(prevdatvals(is3)), ifix(prevdatvals(is4)), ifix(prevdatvals(is5)), ifix(prevdatvals(is6));			! LEV35
	    call wrtlev(sub,msg);												! LEV35
	endif	! LEV35

    endif
enddo

401 continue

go to 303 ! End of POS found, position retrieved

301 continue
    write(msg,'("End-of-file during POS, possibly not all position retrieved")'); call wrttrc(sub,msg);
   go to 999; ! Skip to THE_END leaving return values FAILURE
!-------------------------------------------------------

303 continue
HED=.false.; POS=.false.; DAT=.false.; ! We are not reading the data

write(msg,*)"nprevlev = ",nprevlev; call wrttrc(sub,msg);

if(FOUND_P_POS)read_pqc = SUCCESS;
if(DBLEV== 5)then;
    write(msg,*)'read_pqc FOUND_P_POS: ',FOUND_P_POS;				call wrtlev(sub,msg);
    write(msg,*)'read_pqc    read_pqc: ',read_pqc;				call wrtlev(sub,msg);
    write(msg,*)'read_pqc (for SPEED) what are we returning?: ',read_pqc;	call wrtlev(sub,msg);
endif
!read_pqc = SUCCESS;			! Signal success

go to 999; ! Skip to THE_END
961 continue
    write(errmsg(1),'("FATAL ERROR 511 in ",a," profile: ",a)')trim(sub),trim(qcerrloc);
    write(errmsg(2),'("FATAL ERROR 511 ",a," Reading HED")')trim(qcerrloc);
    write(errmsg(3),'("FATAL ERROR 511 reading HED of prevqc file at 1")');
    write(errmsg(4),'("FATAL ERROR 511 trim(prevqc)")');
    write(errmsg(5),'("FATAL ERROR 511 read reported error ios: ",i2)')ios;
    call error_exit(sub,511,5,errmsg); ! read_pqc - error reading a HED line
962 continue
    write(errmsg(1),'("FATAL ERROR 512 in ",a," profile: ",a)')trim(sub),trim(qcerrloc);
    write(errmsg(2),'("FATAL ERROR 512 ",a," Reading POS @2")')trim(qcerrloc);
    write(errmsg(3),'("FATAL ERROR 512 reading POS LAT/LON line of prevqc file at 2")');
    write(errmsg(4),'("FATAL ERROR 512 trim(prevqc)")');
    write(errmsg(5),'("FATAL ERROR 512 read reported error ios: ",i2)')ios;
    call error_exit(sub,512,5,errmsg); ! read_pqc - error reading a pos line
963 continue
    write(errmsg(1),'("FATAL ERROR 513 in ",a," profile: ",a)')trim(sub),trim(qcerrloc);
    write(errmsg(2),'("FATAL ERROR 513 ",a," Reding POS @3")')trim(qcerrloc);
    write(errmsg(3),'("FATAL ERROR 513 reading POS of prevqc file at 3")');
    write(errmsg(4),'("FATAL ERROR 513 trim(prevqc)")');
    write(errmsg(5),'("FATAL ERROR 513 read reported error ios: ",i2)')ios;
    call error_exit(sub,513,5,errmsg); ! read_pqc - error reading a pos line #1 
964 continue
    write(errmsg(1),'("FATAL ERROR 514 in ",a," profile: ",a)')trim(sub),trim(qcerrloc);
    write(errmsg(2),'("FATAL ERROR 514 ",a," Reading POS @4")')trim(qcerrloc);
    write(errmsg(3),'("FATAL ERROR 514 reading POS of qc file at 4")');
    write(errmsg(4),'("FATAL ERROR 514 trim(qcfile)")');
    write(errmsg(5),'("FATAL ERROR 514 read reported error ios: ",i2)')ios;
    call error_exit(sub,514,5,errmsg); ! read_pqc - error reading a pos line #2
966 continue
    write(errmsg(1),'("FATAL ERROR 515 in ",a," profile: ",a)')trim(sub),trim(qcerrloc);
    write(errmsg(2),'("FATAL ERROR 515 ",a," Reading data @1")')trim(qcerrloc);
    write(errmsg(3),'("FATAL ERROR 515 reading data lines (see  err=966)")');
    write(errmsg(4),'("FATAL ERROR 515 ad error iostat: ",i2)') ios
    call error_exit(sub,515,4,errmsg); ! read_pqc - error reading a data line#3
968 continue
    write(errmsg(1),'("FATAL ERROR 516 in ",a," profile: ",a)')trim(sub),trim(qcerrloc);
    write(errmsg(2),'("FATAL ERROR 516 ",a," Reading data @2")')trim(qcerrloc);
    write(errmsg(3),'("FATAL ERROR 516 reading data lines (see err=968)")');
    write(errmsg(4),'("FATAL ERROR 516 Tried to read ",i3," cols from previous qc cline")')prev_ncolinp;
    write(errmsg(5),'("FATAL ERROR 516 cline: ",a)')trim(cline);
    call error_exit(sub,516,5,errmsg); ! read_pqc - error reading a data line #4
999 continue ! This is THE_END

inp_open=close_file(INP,"INP","previous qc file");

if(DBLEV==35) call dbg_leave(sub,read_pqc);

return

end function read_pqc
! ===========================================================================================================
!--------------------------------------------------------
! To find POS QC flags for "FORTRAJ" lines.
! Tests need to be performed after the loop since tests 
! expect the whole pos array to be filled per file.
!--------------------------------------------------------	
subroutine fill_missing_pos_flags(sub)

use communication                       ! module
use data_arrays                         ! module

implicit none

character(len=*), intent(in) :: sub
integer :: qc02_impossible_date		!  n/a ! s ! Impossible_Date
integer :: qc03_impossible_location	!  n/a ! s ! Impossible_Location
integer :: qc04_position_on_land	!  n/a ! s ! Position_on_Land
integer :: qc05_impossible_speed	!  n/a ! s ! Impossible_Speed
integer :: ret = 0;
integer :: idx,idx2;
logical :: is_999_line;

    do idx=1,ppos  ! Preparing to call the tests
        ! Calculate the POS QC Flags - prev_RTPQC(ppos),prev_RTTQC(ppos),prev_RTSQC(ppos) for FORTRAJ Lines.
        this_lat(idx) = prev_lat(idx);
        this_lon(idx) = prev_lon(idx);
        this_year(idx) = prev_year(idx);
        this_mon(idx) = prev_mon(idx);
        this_day(idx) = prev_day(idx);
        this_hh(idx) =  prev_hh(idx);
        this_mm(idx) =  prev_mm(idx);
        this_sc(idx) =  prev_sc(idx);
    
        if(this_lat(idx) < 98.0)then ! qc05_impossible speed test needs these fields.
	    idx2=idx2+1;
            this_lat2(idx2) = this_lat(idx)
	    this_lon2(idx2) = this_lon(idx)
	    this_year2(idx2) = this_year(idx)
	    this_mon2(idx2) = this_mon(idx)
	    this_day2(idx2) = this_day(idx)
	    this_hh2(idx2) = this_hh(idx)
	    this_mm2(idx2) = this_mm(idx)
	    this_sc2(idx2) = this_sc(idx)
        endif
        pos_dateqc(idx)='0';pos_onland(idx)='0';pos_speedc(idx)='0';pos_posiqc(idx)='0'; pos_posnqc(idx)='0';! Clear flags before callings tests.
    enddo

    !NOTE: The tests for FORTRAJ qc flags are called in "read_pqc.f90" instead of "get_all_qc" because
    !      the qc05_impossible_speed test has to be called before the prev_good positions are set for the coming qc file.
    
    !Although these calls are made from this subroutine, since the major calling functionality is "read_pqc" 
    !using the same "sub" as caller.   
    ret=qc02_impossible_date(sub); ! call test for prev_RTTQC
    ret=qc03_impossible_location(sub); 
    ret=qc04_position_on_land(sub); ! call test for prev_RTLQC
    ret=qc05_impossible_speed(sub); ! call test for prev_RTSQC

    !reset the values once it has been used in speed test.
    prev_posn_found =.false.; prev_posn_good=.false.;
    prev_good_lat = FILL; prev_good_lon = FILL; prev_good_posn_date = FILL14CH;

    do idx=1,ppos
        if(prev_lat(idx) <= 90.0 .and. prev_lon(idx) <= 180.0)then
            idx2=idx2+1; ! adding the index for non-999 array.
            is_999_line=.false.
        else
            is_999_line=.true.
        endif
        if(prev_RTTQC(idx) == '0' .and. pos_dateqc(idx) /= 'n')then !
            prev_RTTQC(idx)=pos_dateqc(idx);
            if(.not. is_999_line) prev_RTTQC2(idx2)=prev_RTTQC(idx); ! adding to non-999 array.
        endif
        if(prev_RTLQC(idx) == '0' .and. pos_onland(idx)/= 'n' )then 
            prev_RTLQC(idx)=pos_onland(idx);
            if(.not. is_999_line) prev_RTLQC2(idx2)=prev_RTLQC(idx); ! adding to non-999 array.
        endif
        if(prev_RTSQC(idx) == '0' .and. pos_speedc(idx)/= 'n' )then 
            prev_RTSQC(idx)=pos_speedc(idx);
            if(.not. is_999_line) prev_RTSQC2(idx2)=prev_RTSQC(idx); ! adding to non-999 array.
        endif
        pis_posnqc(idx)= prev_posOrClass(idx);
    enddo
    
    ! Now, call the subroutine which analyzes all the pos flags, compares with the PI supplied flag
    ! and determines the over all pos_qc flag which goes into pos_posnqc;
    ! Can not put this call inside the do-loop since the subroutines has it's own loop inside.
    	    
    call analyze_positions (sub,ret);
    
    ! Now we know what is the value in over all pos qc  "pos_posnqc" , so write it to prev_RTPQC flag.
    idx2=0;
    do idx=1,ppos
        if(prev_lat(idx) <= 90.0 .and. prev_lon(idx) <= 180.0)then
            idx2=idx2+1;
            is_999_line=.false.
        else
            is_999_line=.true.
        endif
        if(prev_RTPQC(idx) == '0' .and. pos_posnqc(idx)/= 'n' )then 
            prev_RTPQC(idx)=pos_posnqc(idx);
            if(.not. is_999_line) prev_RTPQC2(idx2)=prev_RTPQC(idx); ! adding to non-999 array.
        endif  
    !write(*,*)"DBG:>>> - flags: ", prev_lat(idx)," ",prev_lon(idx)," : " , prev_year(idx), " ",prev_mon(idx)&
    !, prev_day(idx), " " , prev_hh(idx), " ",prev_mm(idx), " ",prev_sc(idx), " " , prev_RTPQC(idx)," ", prev_RTTQC(idx)&
    !, " ",prev_RTLQC(idx)," ",prev_RTSQC(idx)
    
    enddo   
        
end subroutine fill_missing_pos_flags
! ===========================================================================================================
