! ===========================================================================================================
! integer function read_sensormodeltable.f90
 
! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who    what
! 20170712 all    Release of qcplus v1.1 - many changes to all, some documented/tracked, some not
! 20160511 all    Initial release of qcplus v1.1.0
!
! ===========================================================================================================
! User's Manual:
!
! ARGO USER'S MANUAL Version 3.03 20130828 (August 28th 2013)
! ARGO USER'S MANUAL Version 3.1  20141107 (November 7th, 2014)
! 
! QC Manual
! Argo Quality Control Manual Version 2.9.1 20141118 (18 November 2014)

! ===========================================================================================================
! Notes
! Function to read sensor model names to an array from the SENSOR_MODEL lookup table
!==================================================================================================
! non-fatal error 270-279
! ===========================================================================================================

integer function read_sensormodeltable()

use communication                       ! module
use data_arrays                         ! module
use ncmeta_arrays                       ! module

implicit none

! ===========================================================================================================
! varialbe/array declarations

character(len=168) :: line ! One line read from file
integer,parameter :: SENSOR_MODEL_FILE_UNIT = 10  ! Handler 
integer :: status, ios, lnblnk , idx
integer,parameter :: IO_EOF = -1  ! End of File 

integer :: result = 0; ! number of table values read
character(len=16),parameter :: sub = 'read_sensortable'
character(len=15),parameter :: typ = 'NCMETA RDTSNSR'

logical :: table_exists;
 
! ===========================================================================================================
! executable statements

if(DEBUG)then; write(msg,'("==== ",a," BEG ====================")')typ;call wrtdbg(sub,msg); endif

read_sensormodeltable=FAILURE;						! Assume failure
! Make sure the table exists
inquire(file=pathname_sensor_model_table,exist=table_exists)
if(.not. table_exists) go to 281
! Read sensor model table 
inp_open=open_old_file_nf(SENSOR_MODEL_FILE_UNIT,pathname_sensor_model_table,"SENSOR_MODEL","TABLE");
if(.not. inp_open)then
    write(msg,'("READ_CONFIGTABLE Failed to open talbe fie: ",a)')trim(pathname_config_table); call wrttrc(sub,msg);
    goto 282
endif
n_sensor_model_table = 0
read(SENSOR_MODEL_FILE_UNIT,'(168a)',end=101,err=283,iostat=ios) line
do while (ios==IO_OK)
   n_sensor_model_table = n_sensor_model_table + 1
   sensor_model_table(n_sensor_model_table,1) = trim(line(1:40))
   sensor_model_table(n_sensor_model_table,2)= trim(line(41:80))
   sensor_model_table(n_sensor_model_table,3)= trim(line(81:120))
   read(SENSOR_MODEL_FILE_UNIT,'(168a)',end=101,err=283,iostat=ios) line
enddo
101 continue
if (ios/=IO_EOF) goto 283 ! if there is any error, XXX needed?
close(unit=SENSOR_MODEL_FILE_UNIT);

read_sensormodeltable=SUCCESS;					! Report success

go to 999; ! Skip to THE_END
281 continue
    write(errmsg(1),'("NON-FATAL ERROR 261 in read_sensomodeltable")');
    write(errmsg(2),'("NON-FATAL ERROR 261 table file not found: ",a)')trim(pathname_sensor_model_table)
    call error_report(sub,261,2,errmsg); ! read_sensormodeltable table file not found
    go to 999; ! Skip to THE_END
282 continue
    read_sensormodeltable=FAILURE
    write(errmsg(1),'("NON-FATAL ERROR 262 in read_sensormodeltable")');
    write(errmsg(2),'("NON-FATAL ERROR 262 opening the SENSOR_MODEL_TABLE file")');
    write(errmsg(3),'("NON-FATAL ERROR 262 file: ",a)') trim(pathname_sensor_model_table);
    write(errmsg(4),'("NON-FATAL ERROR 262 desc: TABLE")')
    call error_report(sub,262,4,errmsg); ! read_sensormodeltable error opening table file
    go to 999; ! Skip to THE_END
283 continue
    read_sensormodeltable=FAILURE
    write(errmsg(1),'("NON-FATAL ERROR 263 in read_sensormodeltable")');
    write(errmsg(2),'("NON-FATAL ERROR 263 reading SENSOR_MODEL_TABLE")');
    write(errmsg(3),'("NON-FATAL ERROR 263 read reported error ios: ",i2)')ios;
    close(unit=SENSOR_MODEL_FILE_UNIT); ! read_sensormodeltable error reading table file
    call error_report(sub,263,3,errmsg); ! read_sensormodeltableerror reading table file
    go to 999; ! Skip to THE_END
    

999 continue ! This is THE_END

if(DEBUG)then; write(msg,'("NCSNSR Number of n_sensor_model_table; call wrtlog(sub,msg")'); endif

!write(msg,*) "DBG:>>> n_sensor_model_table!", n_sensor_model_table; call wrtlog(sub,msg);

if(DEBUG)then; write(msg,'("==== RDTNC META <<<<< Leave ",a," ret= ",i3)')sub,read_sensormodeltable;call wrtdbg(sub,msg); endif
if(DEBUG)then; write(msg,'("==== ",a," END ====================")')typ;call wrtdbg(sub,msg); endif

return;

end function read_sensormodeltable
! ===========================================================================================================
