! ===========================================================================================================
! integer function read_techtable.f90
 
! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who    what
! 20170712 all    Release of qcplus v1.1 - many changes to all, some documented/tracked, some not
! 20160511 all    Initial release of qcplus v1.1.0
!
! ===========================================================================================================
! User's Manual:
!
! ARGO USER'S MANUAL Version 3.03 20130828 (August 28th 2013)
! ARGO USER'S MANUAL Version 3.1  20141107 (November 7th, 2014)
! 
! QC Manual
! Argo Quality Control Manual Version 2.9.1 20141118 (18 November 2014)

! ===========================================================================================================
! Notes
! Function to read tech parameters for netcdf files while passing parameters from qc files
! very fatal error 960-963; fatal error 560-563; non-fatal error 260-263
! ===========================================================================================================

integer function read_techtable()

use communication                       ! module
use data_arrays                         ! module
use nctech_arrays                       ! module

implicit none

! ===========================================================================================================
! varialbe/array declarations

character(len=168) :: line ! One line read from file
integer,parameter :: TECH_FILE_UNIT = 10  ! Handler 
integer :: status, ios, lnblnk , idx
integer,parameter :: IO_EOF = -1  ! End of File 
character(len=40) :: one_tech_param

integer :: result = 0; ! number of table values read
character(len=16),parameter :: sub = 'read_techtable  '
character(len=15),parameter :: typ = 'NCTECH RDTTECH'

logical :: table_exists;
 
! ===========================================================================================================
! executable statements

if(DEBUG)then; write(msg,'("==== ",a," BEG ====================")')typ;call wrtdbg(sub,msg); endif
! if(DEBUG)then; call dbg_makncmsg(sub,'tech','RDT TECH BEG ================================================'); endif

read_techtable=FAILURE;						! Assume failure
! Make sure the tables exist
inquire(file=pathname_tech_table,exist=table_exists)
if(.not. table_exists) go to 964;
inquire(file=pathname_tech_table_skip,exist=table_exists)
if(.not. table_exists) go to 965;
    
!------------------------------------- Program Starts -----------------------------------------------
! Read tech table 
inp_open=open_old_file_nf(TECH_FILE_UNIT,pathname_tech_table,"TECH","TABLE");
if(.not. inp_open) goto 960
cnt_tech_table_param = 0
read(TECH_FILE_UNIT,'(168a)',end=101,err=961,iostat=ios) line
do while (ios==IO_OK)
   cnt_tech_table_param = cnt_tech_table_param + 1
   qc_tech_params_lst(cnt_tech_table_param)= line(1:40)
   nc_tech_params_lst(cnt_tech_table_param)= line(41:168)
   read(TECH_FILE_UNIT,'(168a)',end=101,err=961,iostat=ios) line
enddo
101 continue
close(unit=TECH_FILE_UNIT);
if (ios/=IO_EOF) goto 961 ! if there is any error, XXX needed?
! Read tech table skip
inp_open=open_old_file_nf(TECH_FILE_UNIT,pathname_tech_table_skip,"TECH","TABLE_SKIP");
if(.not. inp_open) goto 962
cnt_tech_table_skip_param = 0
read(TECH_FILE_UNIT,'(a)',end=102,err=963,iostat=ios) line
do while (ios==IO_OK)
   cnt_tech_table_skip_param = cnt_tech_table_skip_param + 1
   read(line(1:40),'(a)') qc_tech_skip_lst(cnt_tech_table_skip_param)
   read(TECH_FILE_UNIT,'(a)',end=102,err=963,iostat=ios) line
enddo
102 continue
close(unit=TECH_FILE_UNIT);
if (ios/=IO_EOF) goto 963 ! if there is any error, XXX needed?
!write (*,*) 'Content of tech table follows: '
!do idx=1,cnt_tech_table_param
!   write (*,'(a,a,a)') qc_tech_params_lst(idx), '   ', trim(nc_tech_params_lst(idx))
!enddo
!write (*,*) 'Content of tech table skip follows: '
!do idx=1,cnt_tech_table_skip_param
!   write (*,'(a)') qc_tech_skip_lst(idx)
!enddo

! If we get here, we must be doing ok

read_techtable=SUCCESS;					! reoirt success

go to 999; ! Skip to THE_END

960 continue
    write(errmsg(1),'("NON-FATAL ERROR 274 opening the TECH_TABLE file")');
    write(errmsg(2),'("NON-FATAL ERROR 274 file: ",a)') trim(pathname_tech_table);
    write(errmsg(3),'("NON-FATAL ERROR 274 desc: TABLE")')
    call error_report(sub,274,3,errmsg); ! read_techtable opening the TECH_TABLE
    go to 999; ! Skip to THE_END
961 continue
    write(errmsg(1),'("NON-FATAL ERROR 275 reading TECH_TABLE")');
    write(errmsg(2),'("NON-FATAL ERROR 275 read reported error ios: ",i2)')ios;
    close(unit=TECH_FILE_UNIT);
    call error_report(sub,275,2,errmsg); ! read_techtable error reading TECH_TABLE
    go to 999; ! Skip to THE_END
962 continue
    write(errmsg(1),'("NON-FATAL ERROR 276 opening the TECH_TABLE_SKIP file")');
    write(errmsg(2),'("NON-FATAL ERROR 276 file: ",a)') trim(pathname_tech_table_skip);
    write(errmsg(3),'("NPN-FATAL ERROR 276 desc: TABLE_SKIP")')
    call error_report(sub,276,3,errmsg); ! read_techtable error opening the TECH_TABLE_SKIP file
    go to 999; ! Skip to THE_END
963 continue
    write(errmsg(1),'("NON-FATAL ERROR 277 reading TECH_TABLE_SKIP")');
    write(errmsg(2),'("NON-FATAL ERROR 277 read reported error ios: ",i2)')ios;
    close(unit=TECH_FILE_UNIT);
    call error_report(sub,277,2,errmsg); ! read_techtable error reading TECH_TABLE_SKIP
    go to 999; ! Skip to THE_END
964 continue
    write(errmsg(1),'("NON-FATAL ERROR 278 in read_techtable")');
    write(errmsg(2),'("NON-FATAL ERROR 278 table file not found: ",a)')trim(pathname_tech_table)
    call error_report(sub,278,2,errmsg); ! read_techtable did not find TECH_TABLE
    go to 999; ! Skip to THE_END
965 continue
    write(errmsg(1),'("NON-FATAL ERROR 279 in read_techtable")');
    write(errmsg(2),'("NON-FATAL ERROR 279 table file not found: ",a)')trim(pathname_tech_table_skip)
    call error_report(sub,279,2,errmsg); ! read_techtable did not find TECH_TABLE_SKIP
    go to 999; ! Skip to THE_END
999 continue ! This is THE_END

if(DEBUG)then; write(msg,'("NCTECH Number of tech params: ",i4)')cnt_tech_table_param; call wrtdbg(sub,msg); endif
if(DEBUG)then; write(msg,'("NCTECH Number of skip params: ",i4)')cnt_tech_table_skip_param; call wrtdbg(sub,msg); endif

if(DEBUG)then; write(msg,'("==== RDTNC TECH <<<<< Leave ",a," ret= ",i3)')sub,read_techtable;call wrtdbg(sub,msg); endif
if(DEBUG)then; write(msg,'("==== ",a," END ====================")')typ;call wrtdbg(sub,msg); endif
! if(DEBUG)then; call dbg_makncmsg(sub,'tech','RDT TECH END ================================================'); endif


return;

end function read_techtable
! ===========================================================================================================
