! ===========================================================================================================
! character function write_ncmeta.f90
!
! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who    what
! 20180418 JN     Updated to continue creating Meta.NC file (even when there is no valid config mission number)
!                 using the default config mission number.(cnt_missions =1)
! 20170712 all    Release of qcplus v1.1 - many changes to all, some documented/tracked, some not
! 20160511 all    Initial release of qcplus v1.1.0
! ===========================================================================================================
! Notes:
!
! This function creates the nc meta file.
! ===========================================================================================================
! Function returns 'y' for successful file creation, "F" for failure
! ===========================================================================================================

character function write_ncmeta(ncmeta)

use communication                       ! module
use data_arrays                         ! module
use netcdf_arrays			! module
use ncmeta_arrays			! module

implicit none

! ===========================================================================================================
! varialbe/array declarations

character(len=128) , intent( in ) :: ncmeta

integer :: ncid

logical :: dimension_failure = .false.; character(len=21) :: failed_dimension = 'none';
 
character(len=12),parameter :: sub = 'write_ncmeta'
character(len=4),parameter :: nctype = 'meta'
character(len=5),parameter :: ncjob = 'WRITE'; character(len=4) :: nctup;  ! What job are we doing, nctupe_uppercase


! ===========================================================================================================
! executable statements

! Which file is being written, and wat is the filmsg index?
filmsg(l:l) = 'R'; call wrtflm(sub,'Initialize to Running')

call set_uppercase(nctype,nctup)
gloc = 2000; ! gloc - global location of the nc call
gnctype = nctype; ! For calls to cke

if(DEBUG)then; call dbg_ncbeg(sub,nctup,ncjob); endif

! 20170402 Test the dimensions - make sure they are all defined


! 'N_PARAM',                 N_PARAM_len
! 'N_SENSOR',                N_SENSOR_len
! 'N_CONFIG_PARAM',          N_CONFIG_PARAM_len
! 'N_LAUNCH_CONFIG_PARAM',   N_LAUNCH_CONFIG_PARAM_len
! 'N_MISSIONS',              N_MISSIONS_len                UNLIMITED for ncmeta
! 'N_POSITIONING_SYSTEM',    N_POSITIONING_SYSTEM_len
! 'N_TRANS_SYSTEM',          N_TRANS_SYSTEM_len

! NOTEST cnt_missions = 0; ! TEST N_MISSIONS

if(cnt_missions == 0)then;
    ! 20170419 - if ncmeta_noprof/cyc=zzz, increment cnt_missions - the "launch mission"
    cnt_missions = 1 ! Just safety measure since N_MISSIONS can not be "0" to create the Meta.NC file successfully
    configuration_mission_numbers(cnt_missions) = 1;
    if(ncmeta_noprof)then
	if(DEBUG) write(msg,'("NCMETA noprof, set cnt_misisons/value() to 1")'); call wrtdbg(sub,msg);
    else
	if(DEBUG) write(msg,'("NCMETA, set cnt_misisons/value() to 1")'); call wrtdbg(sub,msg);
	write(wrnmsg,'("WARNING 160 No valid CONFIG_MISSION_NUMBER is found in profile: ",a)')trim(profile); ! WRN
	call warning(sub,160,wrnmsg); ! WARNING 160 in write_ncmeta ! WRN
    endif
endif

ret=create_ncfile(sub,nctype,ncmeta,ncid); ! This also defines the dimensins
write(msg,'("write NCMETA: ",a)')trim(ncmeta); call wrtstd(sub,msg); ! STD

call wrtflm(sub,'After opening outputfile');

! Did the file fail to auccessfully open? - either out_open=F, or filmsg(l:l) has changed from  'R'
if(filmsg(l:l) /= 'R')then; write(msg,'(a,"out_opet=T, FILMSG no R: ",a)')HEDF,filmsg(l:l); call wrttrc(sub,msg); return; endif

write(msg,'(a,"output file opened: ",a)')WRTFILE,trim(ncmeta); call wrttrc(sub,msg);

if(DBLEV==63)then; write(DBG,3001)WRTFILE; call close_ncfile(sub,ncid); call direct_exit(sub,0,'LEV63'); endif
3001 format("LEV=63 Exit after nc file creation for ",a);

call def_common_vars(sub,nctype,ncid); 

call def_ncmeta_vars(sub,nctype,ncid);  ! in ncmeta_arrays.f90

call end_define_mode(sub,ncid);

call put_common_data(sub,nctype,ncid);
 
call put_ncmeta_data(sub,nctype,ncid);     ! in ncmeta_arrays.f90

call close_ncfile(sub,ncid);

! See if an error occurred in an array module
if(filmsg(l:l) == 'R')then;
    filmsg(l:l) = 'y'; call wrtflm(sub,'Normal exit, return y for SUCCESS');
elseif((filmsg(l:l) == 'f') .or. (filmsg(l:l) == 'F'))then
    if(DEBUG) write(msg,'(a,a1,x,a8,x,a)')HEDF,'R',filmsg,'ERROR exit, return F or f'; call wrtdbg(sub,msg);
else
    write(msg,'(a,a1,x,a8,x,a)')HEDF,filmsg(l:l),'W Non-standard file result'; call wrtlog(sub,msg);
endif

write(msg,'(a,"output file closed: ",a)')WRTFILE,trim(ncmeta); call wrtlog(sub,msg);

10 continue; ! go to SKIP_IT;
if(dimension_failure)then
    write(errmsg(1),'("NON-FATAL ERROR 216 in write_ncmeta profile: ",a," ncfile: ",a)')trim(profile),trim(ncfile);
    write(errmsg(2),'("NON-FATAL ERROR 216 occurred trying to oen the ncfile, a dimension failed")');
    write(errmsg(3),'("NON-FATAL ERROR 216 the dimensin that failed is ",a)')trim(failed_dimension);
    call error_report(sub,216,3,errmsg); ! write_ncmeta - The UNLIMITED dimension (N_MISSIONS here) is ZERO
endif
write_ncmeta = filmsg(l:l)
 
if(DEBUG)then; call dbg_ncret(sub,nctup,ncjob,write_ncmeta); endif
if(DEBUG)then; call dbg_ncend(sub,nctup,ncjob); endif

 return          ! Normal return
! ===========================================================================================================
end function write_ncmeta
! ===========================================================================================================
