! ===========================================================================================================
! character function write_nctech.f90
!
! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who    what
! 20170712 all    Release of qcplus v1.1 - many changes to all, some documented/tracked, some not
! 20160511 all    Initial release of qcplus v1.1.0
!
! ===========================================================================================================
! Notes:
!
! This function creates the nc tech file.
! ===========================================================================================================
! Function returns 'y' for successful file creation, "F" for failure
! ===========================================================================================================

character function write_nctech(nctech)

use communication                       ! module
use data_arrays                         ! module
use netcdf_arrays			! module
use nctech_arrays			! module

implicit none

! ===========================================================================================================
! varialbe/array declarations

character(len=128) , intent( in ) :: nctech

integer :: ncid;

character(len=12),parameter :: sub = 'write_nctech'
character(len=14),parameter :: typ = 'NCTECH WRITENC';
character(len=4),parameter :: nctype = 'tech' ! Controls which ncfile are we creating
character(len=5),parameter :: ncjob = 'WRITE'; character(len=4) :: nctup;  ! What job are we doing, nctupe_uppercase
integer :: ret = 0;


! ===========================================================================================================
! executable statements

! Which file is being written, and wat is the filmsg index?
filmsg(l:l) = 'R'; call wrtflm(sub,'Initialize to Running')

call set_uppercase(nctype,nctup)
gloc = 4000; ! gloc - global location of the nc call
gnctype = nctype; ! For calls to cke

if(DEBUG)then; call dbg_ncbeg(sub,nctup,ncjob); endif


! 'N_TECH_PARAM',            N_TECH_PARAM_len              ! UNLIMITED for nctech

ret=create_ncfile(sub,nctype,nctech,ncid); ! This also defines the dimensins
write(msg,'("write NCTECH: ",a)')trim(nctech); call wrtstd(sub,msg); ! STD

call wrtflm(sub,'After opening outputfile');

! Did the file fail to auccessfully open? - either out_open=F, or filmsg(l:l) has changed from  'R'
if(filmsg(l:l) /= 'R')then; write(msg,'(a,"out_opet=T, FILMSG no R: ",a)')HEDF,filmsg(l:l); call wrttrc(sub,msg); return; endif

write(msg,'(a,"output file opened: ",a)')WRTFILE,trim(nctech); call wrtlog(sub,msg);

if(DBLEV==64)then; write(DBG,3001)WRTFILE; call close_ncfile(sub,ncid); call direct_exit(sub,0,'LEV64'); endif
3001 format("LEV=64 Exit after nc file creation for ",a);

call def_common_vars(sub,nctype,ncid); 

call def_nctech_vars(sub,nctype,ncid);

call end_define_mode(sub,ncid);

call put_common_data(sub,nctype,ncid);
 
call put_nctech_data(sub,nctype,ncid);

call close_ncfile(sub,ncid);

! See if an error occurred in an array module
if(filmsg(l:l) == 'R') then;
    filmsg(l:l) = 'y'; call wrtflm(sub,'Normal exit, return y for SUCCESS');
elseif((filmsg(l:l) == 'f') .or. (filmsg(l:l) == 'F'))then
    if(DEBUG) write(msg,'(a,a1,x,a8,x,a)')HEDF,'R',filmsg,'ERROR exit, return F or f'; call wrtdbg(sub,msg);
else
    write(msg,'(a,a1,x,a8,x,a)')HEDF,filmsg(l:l),'W Non-standard file result'; call wrtlog(sub,msg);
endif

write_nctech = filmsg(l:l)
 
if(DEBUG)then; call dbg_ncret(sub,nctup,ncjob,write_nctech); endif
if(DEBUG)then; call dbg_ncend(sub,nctup,ncjob); endif

return          ! Normal return
! ===========================================================================================================
end function write_nctech
! ===========================================================================================================
